<?php

declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: 9548f1f81a43 $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2020-07-02 12:51 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
class liveUpdateHttp{

	static function getServerProtocol(bool $addslashes = true): string{
		return we_http_util::getServerProtocol($addslashes);
	}

	static function connectFopen(string $server, string $url, array $parameters = []): string{
		// try fopen first
		$address = 'https://' . $server . $url . ($parameters ? '?' . http_build_query($parameters) : '');
		return file_get_contents($address) ?: '';
	}

	static function connectProxy(string $server, string $url, array $parameters): string{
		$proxyhost = defined('WE_PROXYHOST') ? WE_PROXYHOST : '';
		$proxyport = (defined('WE_PROXYPORT') && WE_PROXYPORT) ? (int) WE_PROXYPORT : 80;
		$proxy_user = defined('WE_PROXYUSER') ? WE_PROXYUSER : '';
		$proxy_pass = defined('WE_PROXYPASSWORD') ? WE_PROXYPASSWORD : '';
		$errno = 0;
		$errstr = '';
		$response = fsockopen($proxyhost, $proxyport, $errno, $errstr, 20);

		if(!$response){
			return $errstr;
		}

		// send headers
		fputs($response, "GET https://" . $server . $url . ($parameters ? '?' . http_build_query($parameters) : '') . " HTTP/1.0\r\n");
		//fputs($response, "Proxy-Connection: Keep-Alive\r\n");
		fputs($response, 'User-Agent: PHP ' . PHP_VERSION . "\r\n");
		fputs($response, "Pragma: no-cache\r\n");
		if($proxy_user != ''){
			fputs($response, "Proxy-authorization: Basic " . base64_encode($proxy_user . ':' . $proxy_pass) . "\r\n");
		}
		fputs($response, "\r\n");

		$zeile = '';
		while(!feof($response)){
			$zeile .= fread($response, 4096);
		}
		fclose($response);

		return substr($zeile, (int) strpos($zeile, "\r\n\r\n") + 4) ?: '';
	}

	static function getCurlHttpResponse(string $server, string $url, array $parameters): string{
		$address = 'https://' . $server . $url;
		$session = curl_init();
		if(!$session){
			return '';
		}
		assert(is_resource($session));
		curl_setopt($session, CURLOPT_URL, $address);
		curl_setopt($session, CURLOPT_RETURNTRANSFER, 1);

		if($parameters){
			curl_setopt($session, CURLOPT_POST, 1);
			curl_setopt($session, CURLOPT_POSTFIELDS, http_build_query($parameters));
		}

		if(defined('WE_PROXYHOST') && WE_PROXYHOST != ''){
			$proxyhost = defined('WE_PROXYHOST') ? WE_PROXYHOST : '';
			$proxyport = (defined('WE_PROXYPORT') && WE_PROXYPORT) ? WE_PROXYPORT : 80;
			$proxy_user = defined('WE_PROXYUSER') ? WE_PROXYUSER : '';
			$proxy_pass = defined('WE_PROXYPASSWORD') ? WE_PROXYPASSWORD : '';

			if($proxyhost != ''){
				curl_setopt($session, CURLOPT_PROXY, $proxyhost . ":" . $proxyport);
				if($proxy_user != ''){
					curl_setopt($session, CURLOPT_PROXYUSERPWD, $proxy_user . ':' . $proxy_pass);
				}
				curl_setopt($session, CURLOPT_SSL_VERIFYPEER, false);
			}
		}

		$data = curl_exec($session);

		curl_close($session);

		return is_bool($data) ? '' : $data;
	}

	static function getHttpOption(): string{
		if(strtolower(ini_get('allow_url_fopen') ?: '') != 'on'){
			if(ini_get('allow_url_fopen') != 1){
				ini_set('allow_url_fopen', '1');
				if(ini_get('allow_url_fopen') != 1){
					if(function_exists('curl_init')){
						return 'curl';
					}
					return 'none';
				}
			}
		}
		return 'fopen';
	}

	static function getHttpResponse(string $server, string $url, array $parameters = []): ?string{
		switch(self::getHttpOption()){
			case 'fopen':
				return self::getFopenHttpResponse($server, $url, $parameters);
			case 'curl':
				return self::getCurlHttpResponse($server, $url, $parameters);
			default:
				return null; // return null otherwise php error
			//'Server error: Unable to open URL (php configuration directive allow_url_fopen=Off)';
		}
	}

	static function getFopenHttpResponse(string $server, string $url, array $parameters = []): string{
		return (defined('WE_PROXYHOST') && WE_PROXYHOST ?
				self::connectProxy($server, $url, $parameters) :
				self::connectFopen($server, $url, $parameters)
				);
	}

	/**
	 * returns html page with formular to init session on the server
	 *
	 * @return string
	 */
	static function getServerSessionForm(): string{
		$params = '';
		$LU_Variables = [
			'clientContent' => false,
			'clientDBcharset' => we_database_base::DB_CHARSET,
			'clientDBcollation' => we_database_base::DB_COLLATION,
			'clientDomain' => urlencode($_SERVER['SERVER_NAME']),
			'clientExtension' => DEFAULT_DYNAMIC_EXT,
			'clientHotfixNr' => (defined('WE_VERSION_HOTFIX_NR')) ? WE_VERSION_HOTFIX_NR : 0,
			'clientInstalledAppMeta' => '', //we_tool_lookup::getAllTools(true, false, true),
			'clientInstalledLanguages' => liveUpdateFunctions::getInstalledLanguages(),
			'clientInstalledModules' => we_modules_info::getUserEnabledModules(),
			'clientLng' => we_base_WE::$Language . '_UTF-8',
			'clientMySQLType' => we_database_base::getMysqlVer()[1],
			'clientMySQLVersion' => we_database_base::getMysqlVer()[0],
			'clientPhpExtensions' => implode(',', get_loaded_extensions()),
			'clientPhpVersion' => PHP_VERSION,
			'clientServerSoftware' => $_SERVER['SERVER_SOFTWARE'],
			'clientSessionID' => session_id(),
			'clientSessionName' => session_name(),
			'clientSubVersion' => WE_SVNREV,
			'clientSyslng' => WE_LANGUAGE,
			'clientUpdateUrl' => we_http_util::getServerUrl() . we_http_util::getScriptName(),
			'clientVersion' => WE_VERSION,
			'clientVersionBranch' => (defined('WE_VERSION_BRANCH')) ? WE_VERSION_BRANCH : '',
			'clientVersionName' => (defined('WE_VERSION_NAME')) ? WE_VERSION_NAME : '',
			'clientVersionSupp' => (defined('WE_VERSION_SUPP')) ? WE_VERSION_SUPP : '',
			'clientVersionSuppVersion' => (defined('WE_VERSION_SUPP_VERSION')) ? WE_VERSION_SUPP_VERSION : '',
			'testUpdate' => empty($_SESSION['weS']['testUpdate']) ? 0 : $_SESSION['weS']['testUpdate'],
		];
		foreach($LU_Variables as $LU_name => $LU_value){
			$params .= '<input type="hidden" name="' . $LU_name . '" value="' . (is_int($LU_value) || is_bool($LU_value) ? intval($LU_value) : urlencode((is_array($LU_value) ? base64_encode(we_serialize($LU_value, SERIALIZE_JSON)) : $LU_value))) . '" />';
		}

		return we_html_tools::getHtmlTop('', LIVEUPDATE_CSS, '
<body onload="document.getElementById(\'liveUpdateForm\').submit();">
<form id="liveUpdateForm" action="https://' . welu_update_const::SERVER . welu_update_const::SCRIPT . '" method="post">' .
						we_html_element::hiddens([
							'update_cmd' => 'startSession',
							'next_cmd' => we_base_request::string('update_cmd'),
							'detail' => we_base_request::string('detail')
						]) . $params . '
</form>
</body>
');
	}

}
