<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: bf50eb0c5668 $
 * $Author: Marc Krämer <marc.kraemer@aldente-it.de> $
 * $Date: 2019-11-25 19:36 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */

/**
 * Class weVersion
 *
 * Provides functions for exporting and importing backups.
 */
abstract class we_backup_contentBase extends we_base_model{
	/** @var array<string,mixed> */
	public $attribute_slots = [];

	/** @var string */
	public $ContentType = '';

	public function getBackup(): string{
		return we_serialize(['type' => constant(get_class($this) . '::BACKUP_XML_TAG'), 'table' => $this->table, 'data' => $this->getBackupData()], SERIALIZE_JSON);
	}

	public static function setBackup(string $data, we_backup_settings $settings): bool{
		$values = we_unserialize($data);
		$obj = self::getObject($values['type'], $values['table'], $settings);
		return $obj->setBackupData($values['data']);
	}

	private static function getObject(string $tag, string $table, we_backup_settings $settings): we_backup_contentBase{
		switch($tag){
			case we_backup_dbTable::BACKUP_XML_TAG:
				$rtable = we_backup_util::getRealTableName($table);
				if($rtable !== ''){
					$settings->current_table = $rtable;
					return new we_backup_dbTable($rtable);
				}
				throw new we_exception_base('not found');

			case we_backup_tableItem::BACKUP_XML_TAG:
				$rtable = we_backup_util::getRealTableName($table);
				if($rtable !== ''){
					$settings->current_table = $rtable;
					return new we_backup_tableItem($rtable);
				}
				throw new we_exception_base('not found');

			case we_backup_binary::BACKUP_XML_TAG:
				return new we_backup_binary();

			case we_backup_version::BACKUP_XML_TAG:
				return new we_backup_version();

			default:
				throw new we_exception_base($tag . ' not found');
		}
	}

	abstract protected function getTable(): string;

	abstract protected function getBackupData(): array;

	abstract protected function setBackupData(array $data): bool;
}
