<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: 0627c1498cde $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2020-07-02 12:07 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
class we_backup_settings implements Serializable{
	/** @var ?we_backup_settings  */
	private static $inst = null;

	public const OPTION_BACKUP_EXTERN = 'backup_extern';
	public const OPTION_BACKUP_HOOKS = 'hooks';
	public const OPTION_BACKUP_CUSTOMTAGS = 'customTags';
	public const OPTION_BACKUP_SETTINGS = 'settings';
//	const OPTION_BACKUP_TEMPORARY = 'temporary';
	public const OPTION_BACKUP_HISTORY = 'history';
	public const OPTION_BACKUP_VERSIONS = 'versions';
	public const OPTION_BACKUP_VERSION_BIN = 'versions_binarys';
	public const OPTION_BACKUP_CONFIG = 'configuration';
	public const OPTION_BACKUP_REBUILD = 'rebuild';
	public const OPTION_BACKUP_BINARY = 'backup_binary';
	public const OPTION_BACKUP_EXPORT2SERVER = 'export2server';
	public const OPTION_BACKUP_EXPORT2SEND = 'export2send';
	public const OPTION_BACKUP_IMPORTAFTER = 'do_import_after_backup';
	public const OPTION_BACKUP_COMPRESS = 'compress';

	/** @var string */
	public $backup_file = '';

	/** @var string */
	public $filename = '';

	/** @var string comment given by the user for the backup */
	public $comment = '';

	/** @var array<string,bool> */
	private $options = [];

	/** @var string */
	public $format = '';

	/** @var array<string,bool> */
	private $handle_options = [];

	/** @var int */
	public $offset = 0;

	/** @var string */
	public $current_table = '';

	/** @var array */
	public $tableMap = [];

	/** @var int */
	public $backup_steps = 5;

	/** @var bool */
	public $backup_log = false;

	/** @var string */
	public $backup_log_data = '';

	/** @var string */
	public $backup_log_file = '';

	/** @var int */
	public $retry = 0;

	/** @var array */
	public $update = [];

	/** @var we_backup_importBase|we_backup_exportBase */
	public $class = null;

	/** @var array{mem:int,exec:int,requestTime:int,lastMem:int,avgLen:array<string,int>} */
	public $limits = [
		'mem' => 0,
		'exec' => 0,
		'requestTime' => 0,
		'lastMem' => 0,
		'avgLen' => []
	];

	/** @var array */
	public $tables = [];

	/** @var array<string,string> */
	public $extern_files = [];

	/** @var int */
	public $extern_files_count = 0;

	/** @var int */
	public $row_counter = 0;

	/** @var int */
	public $row_count = 0;

	/** @var array */
	public $tablekeys = [];

	/** @var array<int,array{table_name:string,tablespace_name:string,database:string}>	 */
	public $allTables = [];

	/** @var int */
	public $current_table_id = -1;

	/** @var string[] */
	public $files_to_delete = [];

	/** @var int */
	public $files_to_delete_count = 0;

	/** @var int */
	public $offset_end = 0;

	/** @var string */
	public $weVersion = '';

	private function __construct(){
		$this->options = [
			self::OPTION_BACKUP_EXTERN => we_base_request::bool('handle_extern'),
			//self::OPTION_BACKUP_COMPRESS => we_base_request::bool('compress'),
			self::OPTION_BACKUP_BINARY => we_base_request::bool('handle_binary'),
			self::OPTION_BACKUP_REBUILD => we_base_request::bool('rebuild'),
			self::OPTION_BACKUP_EXPORT2SERVER => we_base_request::bool('export_server'),
			self::OPTION_BACKUP_EXPORT2SEND => we_base_request::bool('export_send'),
			self::OPTION_BACKUP_IMPORTAFTER => we_base_request::bool('do_import_after_backup'),
		];

		$this->handle_options = [
			we_modules_info::CORE => we_base_request::bool('handle_core'),
			we_modules_info::USERS => we_base_request::bool('handle_user'),
			we_modules_info::CUSTOMER => we_base_request::bool('handle_customer'),
			we_modules_info::SHOP => we_base_request::bool('handle_shop'),
			we_modules_info::WORKFLOW => we_base_request::bool('handle_workflow'),
			we_modules_info::NEWSLETTER => we_base_request::bool('handle_newsletter'),
			we_modules_info::OBJECT => we_base_request::bool('handle_object'),
			we_modules_info::SCHEDULER => we_base_request::bool('handle_schedule'),
			we_modules_info::EXPORT => we_base_request::bool('handle_export'),
			we_modules_info::VOTING => we_base_request::bool('handle_voting'),
			we_modules_info::GLOSSARY => we_base_request::bool('handle_glossary'),
			we_modules_info::SEARCH => we_base_request::bool('handle_search'),
			//self::OPTION_BACKUP_TEMPORARY => we_base_request::bool('handle_temporary'),
			self::OPTION_BACKUP_HISTORY => we_base_request::bool('handle_history'),
			self::OPTION_BACKUP_SETTINGS => we_base_request::bool('handle_settings'),
			self::OPTION_BACKUP_CONFIG => we_base_request::bool('handle_configuration'),
			self::OPTION_BACKUP_VERSIONS => we_base_request::bool('handle_versions'),
			self::OPTION_BACKUP_VERSION_BIN => we_base_request::bool('handle_versions_binarys'),
			self::OPTION_BACKUP_HOOKS => we_base_request::bool('handle_hooks'),
			self::OPTION_BACKUP_CUSTOMTAGS => we_base_request::bool('handle_customtags'),
			//'backup' => $options[self::OPTION_BACKUP_EXTERN],
		];
		$this->setTables();
		$this->comment = we_base_request::string('backup_comment'); //FIXME: this is not written - why?! hidden flag missing!?
		$execTime = ini_get('max_execution_time');
		$this->backup_log = we_base_request::bool('backup_log');
		$this->backup_log_file = WE_BACKUP_PATH . we_backup_util::LOG_FILE;
		$this->limits['mem'] = min(60 * 1024 * 1024, we_base_util::convertIniSizes(ini_get('memory_limit') ?: 0));
		$this->limits['exec'] = min(30, ($execTime > 120 ? ($execTime > 2000 ? 5 : 15) : $execTime));
		$this->allTables = DB_WE::$inst->table_names();
	}

	private function setTables(): void{
		$tableMap = we_backup_tableMap::getMap();

		$tables = [];
		foreach($this->handle_options as $group => $enabled){
			if($enabled && isset($tableMap[$group])){
				$tables = array_merge($tables, $tableMap[$group]);
			}
		}

		$this->tables = $tables;
	}

	public function isHandled(string $option): bool{
		if(!isset($this->handle_options[$option])){
			throw new we_exception_base('option not found');
		}
		return $this->handle_options[$option];
	}

	public function option($option): bool{
		if(!isset($this->options[$option])){
			throw new we_exception_base('option not found');
		}
		return $this->options[$option];
	}

	public function logSettingsExport(): void{
		we_backup_util::addLog('Export to server: ' . ($this->options[we_backup_settings::OPTION_BACKUP_EXPORT2SERVER] ? 'yes' : 'no'));
		we_backup_util::addLog('Export to local: ' . ($this->options[we_backup_settings::OPTION_BACKUP_EXPORT2SEND] ? 'yes' : 'no'));
		we_backup_util::addLog('File name: ' . $this->backup_file);
		we_backup_util::addLog('Export external files: ' . ($this->options[we_backup_settings::OPTION_BACKUP_EXTERN] ? 'yes' : 'no'));
		we_backup_util::writeLog();
	}

	public function logSettingsImport(): void{
		we_backup_util::addLog('File name: ' . $this->backup_file);
		we_backup_util::addLog('Format: ' . $this->format);
		we_backup_util::addLog('Import external files: ' . ($this->option(we_backup_settings::OPTION_BACKUP_EXTERN) ? 'yes' : 'no'));
		we_backup_util::writeLog();
	}

	public static function clearSessionData(): void{
		unset($_SESSION['weS']['module_sessions']['backup']);
		self::$inst = null;
	}

	public static function setSessionData(): void{
		$_SESSION['weS']['module_sessions']['backup'] = self::$inst = new self();
	}

	public static function i(): self{
		if(!self::$inst){
			throw new we_exception_base('not initialized');
		}
		return self::$inst; //@phan-suppress-current-line PhanPartialTypeMismatchReturn
	}

	public function serialize(): string{
		we_backup_util::writeLog();
		return serialize([
			$this->backup_file,
			$this->filename,
			$this->options,
			$this->format,
			$this->handle_options,
			$this->offset,
			$this->current_table,
			$this->tableMap,
			$this->backup_steps,
			$this->backup_log,
			$this->backup_log_file,
			$this->retry,
			$this->update,
			$this->class,
			$this->limits,
			$this->tables,
			$this->extern_files,
			$this->extern_files_count,
			$this->row_count,
			$this->row_counter,
			$this->tablekeys,
			$this->allTables,
			$this->current_table_id,
			$this->files_to_delete,
			$this->files_to_delete_count,
			$this->offset_end,
			$this->weVersion,
		]);
	}

	public function unserialize($serialized): void{
		[
			$this->backup_file,
			$this->filename,
			$this->options,
			$this->format,
			$this->handle_options,
			$this->offset,
			$this->current_table,
			$this->tableMap,
			$this->backup_steps,
			$this->backup_log,
			$this->backup_log_file,
			$this->retry,
			$this->update,
			$this->class,
			$this->limits,
			$this->tables,
			$this->extern_files,
			$this->extern_files_count,
			$this->row_count,
			$this->row_counter,
			$this->tablekeys,
			$this->allTables,
			$this->current_table_id,
			$this->files_to_delete,
			$this->files_to_delete_count,
			$this->offset_end,
			$this->weVersion,
			] = unserialize($serialized);
		self::$inst = $this;
	}

}
