<?php
declare(strict_types=1);

/**
 * These should be public consts only used INSIDE WE, we should not need to load this class in frontend
 *
 * $Rev: 7d142f7c469d $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2020-06-24 11:26 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
abstract class we_base_WE{
	private const MAIN_DOC = [
		'id' => 0,
		'doc' => null,
		'editmode' => false
	];
	private const TEMPLATE_DEFAULT = [
		'init' => 1,
		'head' => false,
		'currentDocID' => 0,
		'preContent' => 0,
		'first' => true,
	];

	/** @var bool indicates whether editmode is active or not */
	public static $editmode = false;

	/** @var bool indicates whether versioning is currently running */
	public static $versioningActive = false;

	/** @var bool indicates whether scheduler is currently running */
	public static $schedulerActive = false;

	/** @var bool indicates whether rebuild is currently running */
	public static $rebuildActive = false;

	/** @var we_base_sessionHandler */
	public static $session;

	/** @var array{id:int,doc:?we_document_webEdition,editmode:bool} */
	public static $mainDoc = self::MAIN_DOC;

	/** @var bool indicates whether save operation leads to a static document */
	public static $saveStatic = false;

	/** @var string */
	public static $Language = '';

	/** @var array{FrontendLanguages:string[],DefaultFrontendLanguage:string} */
	public static $Langs = [
		'FrontendLanguages' => ['de_DE'],
		'DefaultFrontendLanguage' => 'de_DE'
	];

	/** @var string */
	public static $transaction = '';

	/** @var bool true, if shown via weEditor */
	public static $we_EDITOR = false;

	/** @var array{init:int,head:bool,preContent:int,first:bool,currentDocID:int} */
	public static $template = self::TEMPLATE_DEFAULT;

	/** @var int gives the ID of the current parsed template	 */
	public static $currentParsedTemplate = 0;

	/** @var ?we_document_webEdition this is what was GLOBALS['we_doc'] */
	public static $we_doc = null;

	/** @var ?we_contents_objectFile this is what was GLOBALS['we_obj'] */
	public static $we_obj = null;

	/** @var string indicates whether debugmode is active */
	private static $DEBUG = '';

	public static function resetMainDoc(): void{
		self::$mainDoc = self::MAIN_DOC;
	}

	public static function resetTemplate(): void{
		self::$template = self::TEMPLATE_DEFAULT;
	}

	public static function getWeDoc(): we_document_webEdition{
		if(self::$we_doc === null){
			throw new we_exception_base('WE_DOC not set!');
		}
		return self::$we_doc; //@phan-suppress-current-line PhanTypeMismatchReturnNullable
	}

	public static function setWeDoc(?we_document_webEdition $doc): void{
		$GLOBALS['we_doc'] = self::$we_doc = $doc;
	}

	public static function getWeObj(): we_contents_objectFile{
		if(self::$we_obj === null){
			throw new we_exception_base('WE_OBJ not set!');
		}
		return self::$we_obj; //@phan-suppress-current-line PhanTypeMismatchReturnNullable
	}

	public static function setWeObj(?we_contents_objectFile $obj): void{
		$GLOBALS['we_obj'] = self::$we_obj = $obj;
	}

	public static function setDebugging(): bool{
		if(file_exists(($supportDebuggingFile = WE_CUSTOM_DIR . 'conf/debug.inc.php'))){
			include($supportDebuggingFile);//@phan-suppress-current-line PhanMissingRequireFile
			if(defined('SUPPORT_IP') && defined('SUPPORT_DURATION') && defined('SUPPORT_START')){
				if(SUPPORT_IP === $_SERVER['REMOTE_ADDR'] && (time() - SUPPORT_DURATION) < SUPPORT_START){
					self::$DEBUG = $_SERVER['REMOTE_ADDR'];
					return true;
				}
			}
		}
		return false;
	}

	public static function isDebug(): bool{
		return self::$DEBUG === $_SERVER['REMOTE_ADDR'];
	}

}
