<?php
declare(strict_types=1);

/* webEdition CMS
 *
 * $Rev: 2fcfd49ac317 $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2020-07-06 14:53 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */

class we_base_preferences{
	public static $config_files = [
		'conf_conf' => [
			'filename' => WE_CUSTOM_PATH . 'conf/we_conf.inc.php',
			'content' => '',
			'contentBak' => '',
			'contentDef' => '',
		],
		// we_conf_global.inc.php
		'conf_global' => [
			'filename' => WE_CUSTOM_PATH . 'conf/we_conf_global.inc.php',
			'content' => '',
			'contentBak' => '',
			'contentDef' => '',
		],
		// proxysettings.inc.php
		'conf_proxysettings' => [//FIXME: this file must be moved!
			'filename' => WE_CUSTOM_PATH . 'conf/proxysettings.inc.php',
			'content' => '',
			'contentBak' => '',
			'contentDef' => '',
		],
		// we_active_integrated_modules.inc.php
		'conf_active_integrated_modules' => [
			'filename' => WE_CUSTOM_PATH . 'conf/we_active_integrated_modules.inc.php',
			'content' => '',
			'contentBak' => '',
			'contentDef' => '',
		],
	];

	static function loadConfigs(): void{
		// First, read all needed files
		foreach(self::$config_files as &$config){
			try{
				$config['content'] = we_base_file::load($config['filename']);
			} catch (we_exception_file $e){
				$config['content'] = '';
			}
			try{
				$config['contentDef'] = we_base_file::load($config['filename'] . '.default');
			} catch (we_exception_file $e){
				$config['contentDef'] = '<?php ';
			}
			$config['contentBak'] = $config['content'];
		}
		//finally add old session prefs
		self::$config_files['oldPrefs'] = $_SESSION['prefs'] ?? [];
	}

	static function setConfigContent(string $type, string $content): void{
		self::$config_files['conf_' . $type]['content'] = $content;
	}

	static function unsetConfig(string $type): void{
		unset(self::$config_files['conf_' . $type]);
	}

	/**
	 * Checks the global configuration file we_conf_global.inc.php if every needed value
	 * is available and adds missing values.
	 *
	 * @param bool $updateVersion
	 * @param string $file
	 * @param string[] $keep
	 *
	 * @return         void
	 */
	static function check_global_config(bool $updateVersion = false, string $file = '', array $keep = []): void{
		self::loadConfigs();
		$processedConfigs = ($file ?
			['global' => 'contentBak'] :
			['global' => 'contentDef', 'conf' => 'contentDef']);

		$moveToConf = [];
		'@phan-var string[] $moveToConf';
		foreach($processedConfigs as $conf => $dataField){
			// Read the global configuration file
			$file_name = self::$config_files['conf_' . $conf]['filename'];
			//if we don't have content, make sure, we have at least an php-tag
			$oldContent = (trim(str_replace('?>', '', self::$config_files['conf_' . $conf]['contentBak']), "\n ") ?: "<?php \n");

			if($file && $file != $file_name){//=> this is intentended only for we_conf_global
				//we have the data from a backup file. We need to change e.g. DB-Settings, HTTP-User, ...
				$content = we_base_file::load($file);
				//leave settings in their current state
				foreach($keep as $settingname){
					$active = in_array($settingname, $moveToConf, true) ? ($conf === 'conf') : true;
					$content = self::changeSourceCode('define', $content, $settingname, (defined($settingname) ? constant($settingname) : ''), $active);
				}
			} else {
				$content = self::$config_files['conf_' . $conf][$dataField];
			}

			// load & Cut closing PHP tag from configuration file
			$content = trim(str_replace(['?>', "\n\n\n\n", "\n\n\n"], ['', "\n\n", "\n\n"], $content), "\n ");

			// Go through all needed values
			foreach(we_base_preferencesConfig::ALL_CONFIG[$conf] as $define => $value){
				if(!preg_match('/define\(["\']' . $define . '["\'],/', $content)){
					// Add needed variable
					$active = in_array($define, $moveToConf, true) ? ($conf === 'conf') : ($conf === 'global' ? true : defined($define));
					$content = self::changeSourceCode('add', $content, $define, (defined($define) ? constant($define) : $value[2]), $active, $value[0], $value['encode'] ?? false);
					//define it in running session
					if(!defined($define) && $active){
						define($define, $value[2]);
					}
				}
			}
			if($conf === 'global' && $updateVersion){
				$content = self::changeSourceCode('define', $content, 'CONF_SAVED_VERSION', WE_SVNREV, true);
			}
			self::$config_files['conf_' . $conf]['contentBak'] = $oldContent;
			self::$config_files['conf_' . $conf]['content'] = $content;
		}
// Check if we need to rewrite the config file
		self::saveConfigs();
	}

	static function saveConfigs(): void{
		foreach(self::$config_files as $file){
			if(isset($file['content']) && $file['content'] != $file['contentBak']){ //only save if anything changed
				we_base_file::save($file['filename'] . '.bak', $file['contentBak']);
				we_base_file::save($file['filename'], trim($file['content'], "\n "));
			}
		}

		$prefs = !empty($_SESSION['prefs']) && is_array($_SESSION['prefs']) ? $_SESSION['prefs'] : [];
		foreach($prefs as &$p){
			if(is_array($p)){
				$p = implode(',', $p);
			}
		}
		if(!empty($prefs)){
			we_users_user::writePrefs((int) $prefs['userID'], $prefs);
		}
	}

	static function userIsAllowed(string $setting): bool{
		if(we_base_permission::hasPerm([we_base_perm::ADMINISTRATOR])){
			return true;
		}
		foreach(we_base_preferencesConfig::ALL_CONFIG as $name => $config){
			if(!isset($config[$setting])){
				continue;
			}
			switch($name){
				case 'user':
					return true;
				default:
				case 'global':
					return (empty($config[$setting]['perm']) ? we_base_permission::hasPerm([we_base_perm::ADMINISTRATOR]) : we_base_permission::hasPerm($config[$setting]['perm']));
			}
		}
		return false;
	}

	public static function changeSourceCode(string $type, string $text, string $key, /* mixed */ $value, bool $active = true, string $comment = '', bool $encode = false): string{
		switch($type){
			case 'add':
				return trim($text, "\n\t ") . "\n\n" .
					self::makeDefine($key, $value, $active, $comment, $encode);
			case 'define':
				$match = [];
				if(preg_match('|/?/?define\(\s*(["\']' . preg_quote($key) . '["\'])\s*,\s*([^\r\n]+)\);[\r\n]?|Ui', $text, $match)){
					return str_replace($match[0], self::makeDefine($key, $value, $active), $text);
				}
		}

		return $text;
	}

	private static function makeDefine(string $key, /* mixed */ $val, bool $active = true, string $comment = '', bool $encode = false): string{
		return ($comment ? '//' . $comment . "\n" : '') . ($active ? '' : '//') . 'define(\'' . $key . '\', ' .
			($encode ? 'base64_decode(\'' . base64_encode((string) $val) . '\')' :
			(is_bool($val) || $val === 'true' || $val === 'false' ? ($val ? 'true' : 'false') :
			(is_array($val) ? '[' . ($val ? '"' . implode('", "', $val) . '"' : '') . ']' :
			(!is_numeric($val) ? '"' . ($val === null ? '' : self::_addSlashes($val)) . '"' : intval($val)))
			)
			) . ');';
	}

	private static function _addSlashes(string $in): string{
		return str_replace(["\\", '"', "\$"], ["\\\\", '\"', "\\\$"], $in);
	}

	/**
	 * This function returns preference for given name; Checks first the users preferences and then global
	 *
	 * @param          string                                  $name
	 *
	 * @see            getAllGlobalPrefs()
	 *
	 * @return         string
	 */
	public static function getUserPref(string $name): string{
		return (string) ($_SESSION['prefs'][$name] ?? (defined($name) ? constant($name) : ''));
	}

	/**
	 * The function saves the user pref in the session and the database; The function works with user preferences only
	 *
	 * @param          string                                  $name
	 * @param          string                                  $value
	 *
	 * @see            setUserPref()
	 *
	 * @return         boolean
	 */
	public static function setUserPref(string $name, string $value): bool{
		if(isset($_SESSION['prefs'][$name]) && !empty($_SESSION['prefs']['userID'])){
			$_SESSION['prefs'][$name] = $value;
			we_users_user::writePrefs((int) $_SESSION['prefs']['userID']);
			return true;
		}
		return false;
	}

	public static function we_writeLanguageConfig(string $default, array $available = []): bool{
		$locales = '';
		sort($available);
		foreach($available as $Locale){
			$locales .= "	'" . $Locale . "',";
		}

		return we_base_file::save(WE_CUSTOM_PATH . 'conf/we_conf_language.inc.php', '<?php
declare(strict_types = 1);

/**
 * webEdition CMS configuration file
 * NOTE: this file is regenerated, so any extra contents will be overwritten
 */
return [\'FrontendLanguages\' => [' . $locales . '],
\'DefaultFrontendLanguage\' => \'' . $default . '\'
];', 'w+'
		);
	}

	public static function writeDefaultLanguageConfig(): void{
		$file = WE_CUSTOM_PATH . 'conf/we_conf_language.inc.php';
		if(!file_exists($file) || !is_file($file)){
			self::we_writeLanguageConfig((WE_LANGUAGE === 'Deutsch' ? 'de_DE' : 'en_GB'), ['de_DE', 'en_GB']);
		}
	}

	public static function showFrameSet(): void{
		$tabname = we_base_request::string("tabname", we_base_request::string('we_cmd', "setting_ui", 1));

// generate the tabs
		$we_tabs = new we_gui_tabs();
		$validTabs = [];

		foreach(we_editor_preferences::TABS as $name => $list){
			[$icon, $perm] = $list;
			'@phan-var array $perm';
			if(empty($perm) || we_base_permission::hasPerm($perm)){
				$we_tabs->addTab(g_l('prefs', '[tab][' . $name . ']'), $icon, ($tabname === 'setting_' . $name), "'" . $name . "'");
				$validTabs[] = $name;
			}
		}

		echo we_html_tools::getHtmlTop('', we_html_element::cssLink(CSS_DIR . 'we_tab.css') .
			we_html_element::jsScript(JS_DIR . 'preferences_frameset.js', 'self.focus();', ['id' => 'loadVarPreferences_frameset', 'data-prefData' => setDynamicVar([
					'tabs' => array_keys(we_editor_preferences::TABS),
					'validTabs' => $validTabs,
			])]), we_html_element::body(['id' => 'weMainBody', 'onload' => 'weTabs.setFrameSize()', 'onresize' => 'weTabs.setFrameSize()']
				, we_html_element::div(['style' => 'position:absolute;top:0;bottom:0;left:0;right:0;']
					, we_html_element::exIFrame('navi', we_html_element::div(['id' => "main"], $we_tabs->getHTML()), 'right:0;') .
					we_html_element::iframe('content', WEBEDITION_DIR . 'we_cmd.php?we_cmd[0]=we_gui_editorPreferences&' . ($tabname ? 'tabname=' . $tabname : ''), 'position:absolute;top:22px;bottom:40px;left:0;right:0;overflow: hidden;', 'border:0;width:100%;height:100%;overflow: scroll;') .
					we_html_element::exIFrame('we_preferences_footer', self::getPreferencesFooter(), 'position:absolute;bottom:0;height:40px;left:0;right:0;overflow: hidden;')
		)));
	}

	private static function getPreferencesFooter(): string{
		$okbut = we_html_button::create_button(we_html_button::SAVE, 'javascript:we_save();');
		$cancelbut = we_html_button::create_button(we_html_button::CLOSE, 'javascript:top.close()');

		return we_html_element::div(['class' => 'editfooter', 'style' => 'height:100%;'], we_html_button::position_yes_no_cancel($okbut, '', $cancelbut));
	}

	public static function getJSLangConsts(): string{
		return we_base_util::setLangString([
				'add_dictionary_question' => g_l('prefs', '[add_dictionary_question]'),
				'cannot_delete_default_language' => (g_l('prefs', '[cannot_delete_default_language]')),
				'clear_block_entry_question' => g_l('prefs', '[clear_block_entry_question]'),
				'clear_log_question' => g_l('prefs', '[clear_log_question]'),
				'delete_recipient' => g_l('alert', '[delete_recipient]'),
				'input_name' => g_l('alert', '[input_name]'),
				'language_already_exists' => (g_l('prefs', '[language_already_exists]')),
				'language_country_missing' => (g_l('prefs', '[language_country_missing]')),
				'max_name_recipient' => (g_l('alert', '[max_name_recipient]')),
				'not_entered_recipient' => (g_l('alert', '[not_entered_recipient]')),
				'recipient_exists' => (g_l('alert', '[recipient_exists]')),
				'recipient_new_name' => g_l('alert', '[recipient_new_name]'),
		]);
	}

}
