<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: 2fcfd49ac317 $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2020-07-06 14:53 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
class we_category_frames extends we_modules_frame{
	private const TAB_PROPERTIES = 1;

	/** @var we_category_view */
	protected $View;

	/** @var we_category_tree */
	protected $Tree;

	public function __construct(){
		$this->module = we_modules_info::CATEGORY;
		parent::__construct();
		$this->Tree = new we_category_tree($this->jsCmd);
		$this->View = new we_category_view();
	}

	public function getHTML(string $what = '', int $mode = 0, int $step = 0): string{
		switch($what){
			case self::FOOTER:
				return $this->getHTMLEditorFooter([
						'main' => [
							we_html_button::FOOTER_SAVE => [[we_base_perm::WORK_CATEGORY_EDIT], ['we_category_save']],
						],
						'optional' => [
							we_html_button::FOOTER_TRASH => ($this->View->isSaved() ? [[we_base_perm::WORK_CATEGORY_EDIT], ['confirm_we_category_delete', $this->View->Model->Text]] : [
							])
						]
				]);

			default:
				return parent::getHTML($what, $mode, $step);
		}
	}

	protected function getHTMLEditorHeader(): string{
		if(we_base_request::bool('home')){
			return parent::getHTMLEditorHeader();
		}

		$we_tabs = new we_gui_tabs();

		$we_tabs->addTab('', we_gui_tabs::WE_ICON_PROPERTIES, false, (string) self::TAB_PROPERTIES, ['id' => 'tab_' . self::TAB_PROPERTIES, 'title' => g_l('modules_voting', '[property]')]);

		$this->jsCmd->addCmd('setTab', self::TAB_PROPERTIES);
		$tabsHead = we_html_element::cssLink(CSS_DIR . 'we_tab.css') .
			we_html_element::jsScript(JS_DIR . 'initTabs.js') .
			we_html_element::jsScript(WE_JS_MODULES_DIR . '/category/category_top.js');


		$body = we_html_element::body([
				'onresize' => "weTabs.setFrameSize()",
				'onload' => "weTabs.setFrameSize();document.getElementById('tab_" . self::TAB_PROPERTIES . "').className='tabActive';",
				'id' => "eHeaderBody"
				], '<div id="main"><div id="headrow"><b>' . g_l('fileselector', '[catname]') . ':&nbsp;</b><span id="h_path" class="header_small"><b id="titlePath">' . str_replace(' ', '&nbsp;', $this->View->Model->Text) . '</b> (ID:' . $this->View->Model->ID . ')</span></div>' .
				$we_tabs->getHTML() .
				'</div>'
		);

		return $this->getHTMLDocument($body, $tabsHead);
	}

	protected function getHTMLEditorBody(): string{
		if(we_base_request::bool('home')){
			return $this->View->getHomeScreen();
		}

		return $this->getHTMLDocument(we_html_element::body(['class' => 'weEditorBody', 'onload' => "loaded=1;", 'onunload' => "doUnload()"], we_html_element::form([
						'name' => we_base_constants::WE_FORM, 'onsubmit' => "return false"], $this->View->getCommonHiddens(['cmd' => 'we_category_edit', 'pnt' => 'edbody']) . $this->getHTMLProperties())), $this->View->getJSProperty() . we_wysiwyg_editor::getHTMLHeader(false, true) . $this->jsCmd->getCmds());
	}

	function getHTMLTab1(): we_html_multiIconBoxContent{
		return (new we_html_multiIconBoxContent())
				->add(new we_html_multiIconBoxContentLine(g_l('thumbnails', '[name]'), $this->View->formName(), we_html_multiIconBox::PROP_PATH))
				->add(new we_html_multiIconBoxContentLine(g_l('weClass', '[defaults]'), $this->View->formSettings(), we_html_multiIconBox::PROP_ATTRIB));
	}

	function getHTMLProperties(): string{// TODO: move to View
		//$t = we_base_request::int('tabnr', 1);
		return we_html_element::jsScript(JS_DIR . 'utils/multi_editMulti.js') .
			we_html_element::div(['id' => 'tab1'], (new we_html_multiIconBox($this->getHTMLTab1()))->marginLeft()->get());
	}

	protected function getHTMLCmd(): string{
		if(($pid = we_base_request::int('pid', -1)) === -1){
			return $this->getHTMLDocument(we_html_element::body(), $this->jsCmd->getCmds());
		}

		$offset = we_base_request::int('offset', 0);
		$this->jsCmd->addCmd('loadTree', ['clear' => !$pid, 'items' => we_category_tree::getItems($pid, $offset, $this->Tree->default_segment)]);

		return $this->getHTMLDocument(
				we_html_element::body([], we_html_element::form(
						['name' => we_base_constants::WE_FORM], we_html_element::hiddens([
							'pnt' => 'cmd',
							'cmd' => 'no_cmd']
						)
					)
		));
	}

}
