<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: 39e7b777091a $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2019-12-20 15:43 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
class we_dialog_validation extends we_dialog_base{

	protected function __construct(bool $noInternals = true){
		parent::__construct($noInternals);
		$this->onOkJsOnly = true;
		$this->dialogTitle = g_l('validation', '[adjust_service]');
	}

	protected function getJs(): string{
		$this->jsCmd->addCmd('initDialog');
		return we_html_element::jsScript(JS_DIR . 'customizeValidation.js') . $this->jsCmd->getCmds();
	}

	protected function getDialogContentHTML(): string{
		$selServices = we_html_tools::select('validationService', [], 5, '', false, [
				'onchange' => 'we_cmd(\'customValidationService\',\'selectService\');'], true, 320);
		$btns = we_html_button::create_button('new_service', "javascript:we_cmd('customValidationService','newService');") .
			we_html_element::div(['style' => 'height:10px;']) .
			we_html_button::create_button(we_html_button::DELETE, "javascript:we_cmd('customValidationService','deleteService');", '', '', true);
		$table = (new we_html_table([], 1, 2))
			->setCol(0, 0, $selServices, ['style' => 'padding-right:10px;'])
			->setCol(0, 1, $btns, ['style' => 'vertical-align:top;']);
		$top = (new we_html_multiIconBoxContent())
			->add((new we_html_multiIconBoxContentLine(g_l('validation', '[available_services]'), $table->getHtml()))->space(we_html_multiIconBox::SPACE_MED2));

		$parts = (new we_html_multiIconBoxContent())
			->add((new we_html_multiIconBoxContentLine(g_l('validation', '[category]'), we_html_tools::select('category', we_validation_base::getAllCategories(), 1, '')))->space(we_html_multiIconBox::SPACE_MED2)->noline())
			->add((new we_html_multiIconBoxContentLine(g_l('validation', '[service_name]'), we_html_tools::textInput('name', '')))->space(we_html_multiIconBox::SPACE_MED2)->noline())
			->add((new we_html_multiIconBoxContentLine(g_l('validation', '[host]'), we_html_tools::textInput('host', '')))->space(we_html_multiIconBox::SPACE_MED2)->noline())
			->add((new we_html_multiIconBoxContentLine(g_l('validation', '[path]'), we_html_tools::textInput('path', '')))->space(we_html_multiIconBox::SPACE_MED2)->noline())
			->add((new we_html_multiIconBoxContentLine(g_l('validation', '[ctype]'), we_html_tools::textInput('ctype', '') . '<br/><span class="small">' . g_l('validation', '[desc][ctype]') . '</span>'))->space(we_html_multiIconBox::SPACE_MED2)->noline())
			->add((new we_html_multiIconBoxContentLine(g_l('validation', '[fileEndings]'), we_html_tools::textInput('fileEndings', '') . '<br/><span class="small">' . g_l('validation', '[desc][fileEndings]') . '</span>'))->space(we_html_multiIconBox::SPACE_MED2)->noline())
			->add((new we_html_multiIconBoxContentLine(g_l('validation', '[method]'), we_html_tools::select('s_method', ['post' => 'post', 'get' => 'get'], 1, '', false)))->space(we_html_multiIconBox::SPACE_MED2)->noline())
			->add((new we_html_multiIconBoxContentLine(g_l('validation', '[checkvia]'), we_html_tools::select('checkvia', ['url' => g_l('validation', '[checkvia_url]'), 'fileupload' => g_l('validation', '[checkvia_upload]')], 1, '', false)))->space(we_html_multiIconBox::SPACE_MED2)->noline())
			->add((new we_html_multiIconBoxContentLine(g_l('validation', '[varname]'), we_html_tools::textInput('varname', '') . '<br/><span class="small">' . g_l('validation', '[desc][varname]') . '</span>'))->space(we_html_multiIconBox::SPACE_MED2)->noline())
			->add((new we_html_multiIconBoxContentLine(g_l('validation', '[additionalVars]'), we_html_tools::textInput('additionalVars', '') . '<br/><span class="small">' . g_l('validation', '[desc][additionalVars]') . '</span>'))->space(we_html_multiIconBox::SPACE_MED2))
			->add((new we_html_multiIconBoxContentLine(g_l('validation', '[active]'), we_html_tools::select('active', [0 => 'false', 1 => 'true'], 1, '') . '<br/><span class="small">' . g_l('validation', '[desc][active]') . '</span>'))->space(we_html_multiIconBox::SPACE_MED2));

		return we_html_element::form(['name' => we_base_constants::WE_FORM], we_html_element::hiddens(['id' => '0', 'art' => '']) .
				we_html_element::div([], (new we_html_multiIconBox($top, 'weDocValidation'))->marginLeft()->get()) .
				we_html_element::div(['id' => 'dataTable', 'style' => 'display:none;'], (new we_html_multiIconBox($parts, 'weDocValidation'))->marginLeft()->get())
		);
	}

	protected function getDialogButtons(): string{
		$btnSave = we_html_button::create_button(we_html_button::SAVE, "javascript:we_cmd('customValidationService','saveService');");

		return $btnSave . we_html_button::create_button(we_html_button::CLOSE, 'javascript:WE().util.dialogTop(window).doClose();');
	}

}
