<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: b1df1125e20b $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2020-07-06 19:41 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
/*  a class for handling text-documents */
class we_document_text extends we_document_files{
	/* Constructor */

	/** initializes all data not static in class and not saved by persistent slots/elements.
	 * This is called from constructor and __wakeup on serialized documents
	 */
	protected function initStatic(): void{
		parent::initStatic();
		if(isWE()){
			$this->EditPageNrs = array_merge($this->EditPageNrs, [we_base_constants::WE_EDITPAGE_PROPERTIES, we_base_constants::WE_EDITPAGE_INFO, we_base_constants::WE_EDITPAGE_CONTENT,
				we_base_constants::WE_EDITPAGE_VALIDATION, we_base_constants::WE_EDITPAGE_VERSIONS]);
		}
	}

	/** must be called from the editor-script. Returns a filename which has to be included from the global-Script */
	function editor(we_base_jsCmd $jsCmd = null): we_editor_base{
		switch($this->EditPageNr){
			default:
				$_SESSION['weS']['EditPageNr'] = $this->EditPageNr = we_base_constants::WE_EDITPAGE_PROPERTIES;
			case we_base_constants::WE_EDITPAGE_PROPERTIES:
				return new we_editor_properties($this, $jsCmd);
			case we_base_constants::WE_EDITPAGE_INFO:
				return new we_editor_info($this, $jsCmd);
			case we_base_constants::WE_EDITPAGE_CONTENT:
				if($this->docHasChanged()){
					$jsCmd->addMsg(g_l('weClass', '[files][differ]'), we_base_util::WE_MESSAGE_WARNING);
				}
				$GLOBALS['we_editmode'] = we_base_WE::$editmode = true;
				return new we_editor_srcTmpl($this, $jsCmd);
			case we_base_constants::WE_EDITPAGE_PREVIEW:
				if(we_base_WE::$we_EDITOR){
					return (new we_editor_showTempDocumentData($this, $jsCmd))->setContent($this->getDocumentData());
				}
				$GLOBALS['we_editmode'] = we_base_WE::$editmode = false;
				return new we_editor_srcTmpl($this, $jsCmd);
			case we_base_constants::WE_EDITPAGE_VALIDATION:
				return new we_editor_validateDocument($this, $jsCmd);
			case we_base_constants::WE_EDITPAGE_VERSIONS:
				//we_base_WE::setWeDoc($this);
				return new we_editor_versions($this, $jsCmd);
		}
	}

	protected function docHasChanged(): bool{
		switch($this->ContentType){
			case we_base_ContentTypes::CSS:
			case we_base_ContentTypes::JS://we can't check since ids are replaced
				return false;
		}
		try{
			return strcmp(we_base_file::load(WE_DOCUMENT_ROOT . $this->Path), $this->getOldDocText()) !== 0;
		} catch (we_exception_file $e){
			return false;
		}
	}

	public function we_new(): void{
		parent::we_new();
		$this->Filename = $this->getDefaultFilename();
	}

	function isValidEditPage(int $editPageNr): bool{
		if($editPageNr == we_base_constants::WE_EDITPAGE_VALIDATION){
			return ($this->ContentType === we_base_ContentTypes::CSS);
		}

		return parent::isValidEditPage($editPageNr);
	}

	function registerMediaLinks(bool $temp = false, bool $linksReady = false): bool{
		if($linksReady){
			return parent::registerMediaLinks($temp);
		}
		switch($this->ContentType){
			case we_base_ContentTypes::CSS:
			case we_base_ContentTypes::JS:
				$this->replaceWEIDs('', true);
				return parent::registerMediaLinks(true);
		}
		return parent::registerMediaLinks($temp);
	}

	function replaceWEIDs(string $doc = '', bool $registerOnly = false): string{
		$doc = $doc ?: parent::getDocumentToSave();
		$matches = [];
		if(preg_match_all('|#WE:(\d+)#|', $doc, $matches)){
			$matches = array_unique(array_map('intval', $matches[1]), SORT_NUMERIC);
			$this->MediaLinks = $matches;
			if(!$registerOnly){
				if(!$matches){
					return $doc;
				}
				$urlReplace = we_folder_base::getUrlReplacements(DB_WE::$inst, true);
				$paths = id2pathA($matches, FILE_TABLE);
				foreach($paths as $match => $path){
					$doc = str_replace('#WE:' . $match . '#', ($urlReplace ? preg_replace($urlReplace, array_keys($urlReplace), $path) : $path), $doc);
				}
			}
		}
		return strtr($doc, [
			'#WE:ID#' => $this->ID,
			'#WE:PATH#' => $this->Path
		]);
	}

	private function restoreHtaccess(string $oldDoc): void{
		if($this->ID){
			we_base_file::save(WE_DOCUMENT_ROOT . $this->Path, $oldDoc);
		} else {
			we_base_file::delete(WE_DOCUMENT_ROOT . $this->Path);
		}
	}

	protected function getOldDocText(): string{
		return ($this->ID ? fS('SELECT Dat FROM ' . CONTENT_TABLE . ' c WHERE c.DID=' . $this->ID . ' AND c.DocumentTable="' . stripTblPrefix(FILE_TABLE) . '" AND c.nHash=x\'' . md5('data') . '\'') : '');
	}

	public function we_save(bool $resave = false, bool $skipHook = false): bool{
		if($this->ContentType === we_base_ContentTypes::HTACCESS && $this->ParentID == 0){
			//pretest new htaccess file
			$doc = parent::getDocumentToSave();
			$oldDoc = $this->getOldDocText();
			if(!we_base_file::save(WE_DOCUMENT_ROOT . $this->Path, $doc)){//generated error codes; since fopen is not capable of returning proper codes
				//restore old htaccess
				$this->restoreHtaccess($oldDoc);
				$this->errMsg = 'Error Write error';
				return false;
			}
			try{
				$data = we_http_util::getHTTP(we_http_util::getServerUrl(true), WEBEDITION_DIR . 'we_cmd.php?we_cmd[0]=OK');
			} catch (we_exception_http $e){
				$this->errMsg = $e->getMessage();
				$this->restoreHtaccess($oldDoc);
				return false;
			}
			if($data !== 'OK'){//generated error codes; since fopen is not capable of returning proper codes
//restore old htaccess
				$this->restoreHtaccess($oldDoc);
				$this->errMsg = 'Error 500, ' . ($data === 'OK' ? '' : $data);
				return false;
			}
		}

		if(($ret = parent::we_save($resave, $skipHook))){
			$ret = $this->registerMediaLinks(true, true);
		}

		return $ret;
	}

	protected function writeSiteDir(string $content): bool{
		switch($this->ContentType){
			case we_base_ContentTypes::HTACCESS:
				return true;
			default:
				return parent::writeSiteDir($content);
		}
	}

	function getText(int $renumber = 0): string{
		if($this->parseFile && $this->ContentType === we_base_ContentTypes::CSS){
			switch($this->Extension){
				case '.less':
				case '.scss':
					return ($this->Filename) . '.css';
			}
		}

		return parent::getText($renumber);
	}

	protected function getDocumentToSave(): string{
		$doc = parent::getDocumentToSave();
		if(defined('IMPORT_RUNNING')){
			return $doc;
		}

		switch($this->ContentType){
			case we_base_ContentTypes::CSS:
				if(!$this->parseFile){
					return $this->replaceWEIDs($doc);
				}
				switch($this->Extension){
					case '.sass':
					case '.css':
						break;
					case '.less':
						$less = new lessc();
						we_helpers_lessParser::$includedFiles = [];
						$less->setImportDir([WE_DOCUMENT_ROOT,
							WE_DOCUMENT_ROOT . $this->getParentPath(),
						]);
						$less->setFormatter('classic');
						try{
							//we prepend an extra / before #WE, to make parser believe this is an absolute path
							$doc = str_replace('/#WE:', '#WE:', $less->compile(preg_replace('|(#WE:\d+#)|', '/$1', $doc)));
							$this->DB_WE->query('DELETE FROM ' . FILELINK_TABLE . ' WHERE ID=' . intval($this->ID) . ' AND DocumentTable="' . stripTblPrefix(FILE_TABLE) . '" AND type="css"');
							$this->registerFileLinks(we_helpers_lessParser::$includedFiles, 'css');
						} catch (exception $e){
							$this->errMsg = str_replace(['\n', "\n"], ' ', $e->getMessage());
							throw new we_exception_base($e->getMessage());
						}

						break;
					case '.scss':
						$scss = new we_helpers_scss();
						we_helpers_scss::$includedFiles = []; //due to rebuild!
						$scss->setImportPaths(array_unique(['', WE_DOCUMENT_ROOT . $this->getParentPath(), WE_DOCUMENT_ROOT . '/']));
						try{
							$doc = $scss->compile($doc);
							$this->DB_WE->query('DELETE FROM ' . FILELINK_TABLE . ' WHERE ID=' . intval($this->ID) . ' AND DocumentTable="' . stripTblPrefix(FILE_TABLE) . '" AND type="css"');
							$this->registerFileLinks(we_helpers_scss::$includedFiles, 'css');
						} catch (exception $e){
							$this->errMsg = str_replace(['\n', "\n"], ' ', $e->getMessage());
							throw new we_exception_base($e->getMessage());
						}
				}
			//no break;
			case we_base_ContentTypes::JS:
				$doc = $this->replaceWEIDs($doc);
				//FIXME: write all dependend files to database link, this should be the same table, as used for media queries in 6.5
				break;
			default:
		}
		return $doc;
	}

	function formParseFile(): string{
		return we_html_forms::checkboxWithHidden((bool) $this->parseFile, 'we_' . $this->Name . '_parseFile', g_l('weClass', '[parseFile]'), false, '', '_EditorFrame.setEditorIsHot(true);');
	}

	public function getPropertyPage(we_base_jsCmd $jsCmd): string{
		return (new we_html_multiIconBox((new we_html_multiIconBoxContent())
					->add((new we_html_multiIconBoxContentLine(g_l('weClass', '[path]'), $this->formPath(), we_html_multiIconBox::PROP_PATH)))
					->add($this->ContentType === we_base_ContentTypes::CSS ? new we_html_multiIconBoxContentLine(g_l('weClass', '[document]'), $this->formParseFile(), we_html_multiIconBox::PROP_DOC) : null)
					->add((new we_html_multiIconBoxContentLine(g_l('weClass', '[owners]'), $this->formCreatorOwners($jsCmd), we_html_multiIconBox::PROP_USER))->fold())
					->add((new we_html_multiIconBoxContentLine(g_l('weClass', '[copy' . $this->ContentType . ']'), $this->formCopyDocument(), we_html_multiIconBox::PROP_COPY)))
					->add((in_array($this->ContentType, [we_base_ContentTypes::CSS, we_base_ContentTypes::JS]) ?
						(new we_html_multiIconBoxContentLine(g_l('modules_glossary', '[textreplacement]'), '<pre>
#WE:' . $this->ID . '# => ' . $this->Path . '
#WE:ID#   => ' . $this->ID . '
#WE:Path# => ' . $this->Path . '
</pre>', we_html_multiIconBox::PROP_HELP))->fold() : null)
					)
					, 'PropertyPage'))
				->marginLeft()
				->get();
	}

	public function mustCheckEditPageArray(): bool{
		return true;
	}

	public function getDocumentCode(): string{
		return $this->getElementS('data');
	}

	public function publInScheduleTable(): ?DateTime{
		return null;
	}

}
