<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: 0627c1498cde $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2020-07-02 12:07 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
abstract class we_editor_footer{

	private static function fileLocked(we_contents_root $we_doc, bool $allowNew): void{
		$dat = getHash('SELECT username,CONCAT(First," ",Second,"(",username,")") AS Name FROM ' . USER_TABLE . ' WHERE ID=' . $we_doc->isLockedByUser());

		echo we_html_tools::getHtmlTop('', '', we_html_element::body(['id' => 'footerBody'], we_html_element::div(['class' => 'footertable'], we_html_element::div([
						'class' => 'main'], '<span class="fa-stack fa-lg" style="color:#F2F200;margin-right:5px;"><i class="fa fa-exclamation-triangle fa-stack-2x" ></i><i style="color:black;" class="fa fa-exclamation fa-stack-1x"></i></span>' .
						sprintf(g_l('alert', '[file_locked_footer]'), $dat['Name']) .
						we_html_button::create_button(we_html_button::RELOAD, "javascript:WE().layout.weNavigationHistory.navigateReload();") .
						we_html_button::create_button(we_html_button::UNLOCK, "javascript:top.we_cmd('requestUnlock','" . $dat['username'] . "'," . $we_doc->ID . ",'" . $we_doc->Table . "');"))
					. ($allowNew ? we_html_element::div(['class' => 'optional'], self::addCopyButton($we_doc)) : '')
				)
		));
	}

	private static function fileInWorkspace(): void{
		echo we_html_tools::getHtmlTop('', '', we_html_element::body(['id' => 'footerBody'], we_html_element::div(['class' => 'footertable'], '<span class="fa-stack fa-lg" style="color:#F2F200;margin-right:5px;"><i class="fa fa-exclamation-triangle fa-stack-2x" ></i><i style="color:black;" class="fa fa-exclamation fa-stack-1x"></i></span>' .
					g_l('alert', '[' . FILE_TABLE . '][not_im_ws]'))));
	}

	private static function fileNoSave(): void{
		echo we_html_tools::getHtmlTop('', '', we_html_element::body(['id' => 'footerBody'], we_html_element::div(['class' => 'footertable'], '<span class="fa-stack fa-lg" style="color:#F2F200;margin-right:5px;"><i class="fa fa-exclamation-triangle fa-stack-2x" ></i><i style="color:black;" class="fa fa-exclamation fa-stack-1x"></i></span>' .
					g_l('alert', '[file_no_save_footer]'))));
	}

	private static function fileIsRestricted(we_contents_root $we_doc): void{
		echo we_html_tools::getHtmlTop('', '', we_html_element::body(['id' => 'footerBody'], we_html_element::div(['class' => 'footertable'], '<span class="fa-stack fa-lg" style="color:#F2F200;margin-right:5px;"><i class="fa fa-exclamation-triangle fa-stack-2x" ></i><i style="color:black;" class="fa fa-exclamation fa-stack-1x"></i></span>' .
					sprintf(g_l('alert', '[no_perms]', false, true), fS('SELECT CONCAT(First," ",Second," (",username,")") FROM ' . USER_TABLE . ' WHERE ID=' . intval($we_doc->CreatorID))))));
	}

	private static function workflow(we_contents_root $we_doc, bool $showPubl): string{
		if(we_workflow_utility::isUserInWorkflow($we_doc->ID, $we_doc->Table, we_users_user::getID()) || we_base_permission::hasPerm([we_base_perm::WORK_CAN_PUBLISH])){

			return we_html_element::body(['id' => 'footerBody'], we_html_element::form(['name' => we_base_constants::WE_FORM, 'method' => 'post'], we_workflow_view::getFooter($_SESSION['weS']['we_mode'], $we_doc, $showPubl)));
		}

		return we_html_element::body(['id' => 'footerBody'], we_html_element::div(['class' => 'footertable'], '<span class="fa-stack fa-lg" style="color:#F2F200;margin-right:16px;"><i class="fa fa-exclamation-triangle fa-stack-2x" ></i><i style="color:black;" class="fa fa-exclamation fa-stack-1x"></i></span>' .
					g_l('modules_workflow', '[doc_in_wf_warning]')));
	}

	private static function addDelButton(we_contents_root $we_doc): string{
		$ctrlElem = $we_doc->getControlElement('button', 'delete'); //	look tag we:controlElement for details
		if(!$ctrlElem || !$ctrlElem['hide']){
			$extra = '';
			if($we_doc instanceof we_folder_base){
				$extra .= '<li>' . we_html_element::b('&gt; ' . fI('SELECT COUNT(1) FROM ' . $we_doc->Table . ' WHERE ParentID=' . $we_doc->ID) . ' ' . g_l('fileselector', '[files]') . ' / ' . g_l('fileselector', '[folders]')) . '</li>';
			}
			return we_html_button::create_button(we_html_button::FOOTER_TRASH, "javascript:WE().util.showConfirm(window, '', WE().util.sprintf(WE().consts.g_l.main.delete_single_confirm_delete,WE().consts.g_l.contentTypes[doc.contentType])+'<ul><li>'+doc.Path+'</li>" . $extra . "</ul>',['delete_single_document','',doc.Table,1]);");
		}
		return '';
	}

	private static function addCopyButton(we_contents_root $we_doc): string{
		switch($we_doc->ContentType){
			case we_base_ContentTypes::FOLDER_DOCUMENT:
			case we_base_ContentTypes::FOLDER_OBJECTFILE:
			case we_base_ContentTypes::FOLDER_TEMPLATE:
			case we_base_ContentTypes::FOLDER_VFILE:
			case we_base_ContentTypes::OBJECT:
			case we_base_ContentTypes::TEMPLATE:
				return '';
			default:
				$ctrlElem = $we_doc->getControlElement('button', 'copy'); //	look tag we:controlElement for details
				if((!$ctrlElem || !$ctrlElem['hide']) && $we_doc->ID){
					return we_html_button::create_button(we_html_button::FOOTER_COPY, "javascript:top.we_cmd('cloneDocument');");
				}
		}
		return '';
	}

	/**
	 * @return string
	 * @desc Prints the footer for the normal mode
	 */
	private static function normalMode(we_contents_root $we_doc, string $we_transaction, bool $showPubl, bool $haspermNew, bool $hasPermDel): string{
		$main = $optional = [];
		if($we_doc->ID){
			//del
			if($hasPermDel){
				$optional[] = self::addDelButton($we_doc);
			}
			//copy
			if($haspermNew){
				$optional[] = self::addCopyButton($we_doc);
			}
			//new
			switch($we_doc->ContentType){
				case we_base_ContentTypes::TEMPLATE:
					if(we_base_permission::hasPerm([we_base_perm::WORK_DOCUMENT_WEBEDITION_NEW])){
						$optional[] = we_html_button::create_button(we_html_button::MAKE_NEW_DOC, "javascript:top.we_cmd('new',WE().consts.tables.FILE_TABLE,'',WE().consts.contentTypes.WEDOCUMENT,'','" . $we_doc->ID . "');WE().layout.weEditorFrameController.getActiveEditorFrame().setEditorMakeNewDoc(false);");
					}
					break;
				case we_base_ContentTypes::OBJECT:
					if(we_base_permission::hasPerm([we_base_perm::OBJECTFILE_NEW])){
						$optional[] = we_html_button::create_button(we_html_button::MAKE_NEW_OBJ, "javascript:top.we_cmd('new',WE().consts.tables.OBJECT_FILES_TABLE,'',WE().consts.contentTypes.OBJECT_FILE,'" . $we_doc->ID . "');WE().layout.weEditorFrameController.getActiveEditorFrame().setEditorMakeNewDoc(false);");
					}
					break;
			}

			if($showPubl && $we_doc instanceof we_document_base && $we_doc->Publish_Date){
				//	Park button
				$ctrlElem = $we_doc->getControlElement('button', 'unpublish'); //	look tag we:controlElement for details

				if(!$ctrlElem || !$ctrlElem['hide']){
					$main[] = we_html_button::create_button(we_html_button::FOOTER_UNPUBLISH, "javascript:we_cmd('unpublish', '" . $we_transaction . "');");
				}
			}
		}

		switch($we_doc->ContentType){
			case we_base_ContentTypes::WEDOCUMENT:
			case we_base_ContentTypes::OBJECT:
			case we_base_ContentTypes::OBJECT_FILE:
			case we_base_ContentTypes::FOLDER_DOCUMENT:
			case we_base_ContentTypes::FOLDER_OBJECTFILE:
			case we_base_ContentTypes::FOLDER_TEMPLATE:
			case we_base_ContentTypes::FOLDER_VFILE:
			case we_base_ContentTypes::COLLECTION:
				break;
			default:
				if(!empty($_SESSION['weS']['hasJava']) && we_modules_info::isActive(we_modules_info::EDITOR)){
					$optional[] = (stripos($we_doc->ContentType, 'text/') !== false ?
						we_html_button::create_button('fa:startEditor,fa-lg fa-clipboard', "javascript:editSource();") :
						we_html_button::create_button('fa:startEditor,fa-lg fa-clipboard', "javascript:editFile();")
						);
				}
		}

		//	Save Button
		$ctrlElem = $we_doc->getControlElement('button', 'save'); //	look tag we:controlElement for details
		if(!$ctrlElem || !$ctrlElem['hide']){
			$main[] = we_html_button::create_button(we_html_button::FOOTER_SAVE, "javascript:WE().layout.weEditorFrameController.getActiveEditorFrame().setEditorPublishWhenSave(false);we_save_document();");
		}

		if(we_modules_info::isActive(we_modules_info::WORKFLOW) && $we_doc->IsTextContentDoc && $we_doc->ID){
			//	Workflow button
			$ctrlElem = $we_doc->getControlElement('button', 'workflow'); //	look tag we:controlElement for details

			if(!$ctrlElem || !$ctrlElem['hide']){
				$main[] = we_html_button::create_button(we_html_button::FOOTER_SAVE_TO_WORKFLOW, "javascript:put_in_workflow('" . stripTblPrefix($we_doc->Table) . "');");
			}
		}

		switch($we_doc->ContentType){
			case we_base_ContentTypes::TEMPLATE:
				if(defined('VERSIONING_TEXT_WETMPL') && defined('VERSIONS_CREATE_TMPL') && VERSIONS_CREATE_TMPL && VERSIONING_TEXT_WETMPL){
					$main[] = we_html_button::create_button(we_html_button::FOOTER_SAVE_VERSION, "javascript:WE().layout.weEditorFrameController.getActiveEditorFrame().setEditorPublishWhenSave(true);we_save_document();");
				}

				$main[] = we_html_forms::checkbox('autoRebuild', false, 'autoRebuild', '', false, '', '', false, '', we_html_tools::TYPE_NONE, 0, we_html_button::create_button(we_html_button::FOOTER_REBUILD, "javascript:var chk=document.getElementById('autoRebuild');
						chk.checked=!chk.checked;
						WE().layout.weEditorFrameController.getActiveEditorFrame().setEditorAutoRebuild(chk.checked)", '', '', false, false, '', false, g_l('global', '[we_rebuild_at_save]'), 'switchButton'));
				break;
			default:
				if($showPubl){
					$ctrlElem = $we_doc->getControlElement('button', 'publish');
					if(!$ctrlElem || !$ctrlElem['hide']){
						$text = we_modules_info::isActive(we_modules_info::SCHEDULER) && we_schedule_scheduler::isSaveInSchedulerActive($we_doc) ? 'fat:saveInScheduler,fa-lg fa-clock-o' : we_html_button::FOOTER_PUBLISH;
						$main[] = we_html_element::span(['id' => 'publish_' . $we_doc->ID], we_html_button::create_button($text, "javascript:WE().layout.weEditorFrameController.getActiveEditorFrame().setEditorPublishWhenSave(true);we_save_document();"));
					}
				}
		}

		/* if(($we_doc->IsTextContentDoc) && $haspermNew){
		  $ctrlElem = $we_doc->getControlElement('checkbox', 'makeSameDoc');
		  if(!$ctrlElem || !$ctrlElem['hide']){
		  $main[] = we_html_element::span((($ctrlElem && $ctrlElem['hide'] ) ? ( ['style' => 'display:none'] ) : ['style' => 'display:inline-block']), we_html_forms::checkbox('makeSameDoc', ( $ctrlElem ? $ctrlElem['checked'] : false), 'makeSameDoc', '', false, '', '', ( $ctrlElem ? $ctrlElem['readonly'] : false), '', we_html_tools::TYPE_NONE, 0,
		  we_html_button::create_button(we_html_button::FOOTER_MAKE_SAME_DOC, "javascript:var chk=document.getElementById('makeSameDoc');chk.checked=!chk.checked;WE().layout.weEditorFrameController.getActiveEditorFrame().setEditorMakeSameDoc(chk.checked);", '', '', false, false, '', false, g_l('global', '[we_make_same][' . $we_doc->ContentType . ']'), 'switchButton')));
		  }
		  }

		 */

		return we_html_element::div(['class' => 'footertable'], we_html_element::div(['class' => 'main'], implode('', $main)) . we_html_element::div(['class' => 'optional'], implode('', array_reverse(array_filter($optional)))));
	}

	/**
	 * @return string
	 * @desc prints the footer for the See-Mode
	 */
	private static function SEEMode(we_contents_root $we_doc, string $we_transaction, bool $showPubl, bool $hasPermDel): string{
		$main = $optional = [];

		//First buttons which are always needed
		//	Always button preview
		if($we_doc->EditPageNr != we_base_constants::WE_EDITPAGE_PREVIEW && in_array(we_base_constants::WE_EDITPAGE_PREVIEW, $we_doc->EditPageNrs, true)){ // first button is always - preview, when exists
			$main[] = we_html_button::create_button(we_html_button::PREVIEW, "javascript:parent.editHeader.we_cmd('switch_edit_page', " . we_base_constants::WE_EDITPAGE_PREVIEW . ",'" . $we_transaction . "');");
		}

		// variants
		if($we_doc->EditPageNr == we_base_constants::WE_EDITPAGE_CONTENT && in_array(we_base_constants::WE_EDITPAGE_VARIANTS, $we_doc->EditPageNrs, true) && $we_doc->canHaveVariants(true) && $we_doc->EditPageNr != we_base_constants::WE_EDITPAGE_VARIANTS){ // first button is always - preview, when exists
			$main[] = we_html_button::create_button('shopVariants', "javascript:parent.editHeader.we_cmd('switch_edit_page', " . we_base_constants::WE_EDITPAGE_VARIANTS . ",'" . $we_transaction . "');");
		}

		//	image-documents have no preview but thumbnailview instead ...
		if($we_doc->EditPageNr != we_base_constants::WE_EDITPAGE_THUMBNAILS && in_array(we_base_constants::WE_EDITPAGE_THUMBNAILS, $we_doc->EditPageNrs, true)){
			$main[] = we_html_button::create_button('thumbnails', "javascript:parent.editHeader.we_cmd('switch_edit_page', " . we_base_constants::WE_EDITPAGE_THUMBNAILS . ",'" . $we_transaction . "');");
		}

		//	Button edit !!!
		if($we_doc->EditPageNr != we_base_constants::WE_EDITPAGE_CONTENT && in_array(we_base_constants::WE_EDITPAGE_CONTENT, $we_doc->EditPageNrs, true)){ // then button "edit"
			$main[] = we_html_button::create_button(we_html_button::OPEN_TO_EDIT, "javascript:parent.editHeader.we_cmd('switch_edit_page', " . we_base_constants::WE_EDITPAGE_CONTENT . ", '" . $we_transaction . "');");
		}
		//	Button properties
		if(($we_doc->EditPageNr == we_base_constants::WE_EDITPAGE_CONTENT || $we_doc->EditPageNr == we_base_constants::WE_EDITPAGE_SCHEDULER) && in_array(we_base_constants::WE_EDITPAGE_PROPERTIES, $we_doc->EditPageNrs, true)){
			if(we_base_permission::hasPerm([we_base_perm::WORK_PROPERTIES_CAN_SEE])){
				$main[] = we_html_button::create_button('properties', "javascript:parent.editHeader.we_cmd('switch_edit_page', " . we_base_constants::WE_EDITPAGE_PROPERTIES . ", '" . $we_transaction . "');");
			}
		}

		// Button workspace
		if(($we_doc->EditPageNr == we_base_constants::WE_EDITPAGE_CONTENT || $we_doc->EditPageNr == we_base_constants::WE_EDITPAGE_PROPERTIES) && in_array(we_base_constants::WE_EDITPAGE_WORKSPACE, $we_doc->EditPageNrs, true)){
			$main[] = we_html_button::create_button('workspace_button', "javascript:parent.editHeader.we_cmd('switch_edit_page', " . we_base_constants::WE_EDITPAGE_WORKSPACE . ", '" . $we_transaction . "');");
		}


		//	Button scheduler
		if(in_array(we_base_constants::WE_EDITPAGE_SCHEDULER, $we_doc->EditPageNrs, true) && ($we_doc->EditPageNr == we_base_constants::WE_EDITPAGE_CONTENT || $we_doc->EditPageNr == we_base_constants::WE_EDITPAGE_PROPERTIES) &&
			we_modules_info::isActive(we_modules_info::SCHEDULER) && we_base_permission::hasPerm([we_base_perm::SCHEDULER_CAN_SEE])){
			$main[] = we_html_button::create_button('fat:schedule_button,fa-lg fa-clock-o', "javascript:parent.editHeader.we_cmd('switch_edit_page', " . we_base_constants::WE_EDITPAGE_SCHEDULER . ", '" . $we_transaction . "');");
		}

		//	Button put in workflow
		if(/* $we_doc->EditPageNr != we_base_constants::WE_EDITPAGE_PROPERTIES && */ $we_doc->EditPageNr != we_base_constants::WE_EDITPAGE_SCHEDULER && // then button "workflow"
			we_modules_info::isActive(we_modules_info::WORKFLOW) && $we_doc->IsTextContentDoc && $we_doc->ID){

			$ctrlElem = $we_doc->getControlElement('button', 'workflow'); //	look tag we:controlElement for details

			if(!$ctrlElem || !$ctrlElem['hide']){
				$main[] = we_html_button::create_button(we_html_button::FOOTER_SAVE_TO_WORKFLOW, "javascript:put_in_workflow('" . stripTblPrefix($we_doc->Table) . "');");
			}
		}

		//Special buttons for special EDITPAGE
		//
		//	1. ONLY in PROPERTY page we need the button unpublish
		//
		if($we_doc->EditPageNr == we_base_constants::WE_EDITPAGE_PROPERTIES && $showPubl && $we_doc->ID && $we_doc instanceof we_document_base && $we_doc->Publish_Date){

			//	button unpublish
			$ctrlElem = $we_doc->getControlElement('button', 'unpublish'); //	look tag we:controlElement for details
			if(!$ctrlElem || !$ctrlElem['hide']){
				$main[] = we_html_button::create_button(we_html_button::FOOTER_UNPUBLISH, "javascript:we_cmd('unpublish', '" . $we_transaction . "');");
			}
		}

		//
		//	2. we always need the buttons -> save and publish
		//
		$ctrlElem = $we_doc->getControlElement('button', 'save'); //	look tag we:controlElement for details
		if(!$ctrlElem || !$ctrlElem['hide']){
			$main[] = we_html_button::create_button(we_html_button::FOOTER_SAVE, "javascript:WE().layout.weEditorFrameController.getActiveEditorFrame().setEditorPublishWhenSave(false);we_save_document();");
		}


		//
		// 3. publish when save and make same doc new.
		//
		if($showPubl){
			$ctrlElem = $we_doc->getControlElement('button', 'publish'); //	look tag we:controlElement for details

			if(!($ctrlElem && $ctrlElem['hide'])){

				$main[] = we_html_button::create_button(we_html_button::FOOTER_PUBLISH, "javascript:WE().layout.weEditorFrameController.getActiveEditorFrame().setEditorPublishWhenSave(true);we_save_document();");
			}
		}
		/*
		  if(($we_doc->IsTextContentDoc) && $haspermNew && ($we_doc->EditPageNr == we_base_constants::WE_EDITPAGE_CONTENT || $we_doc->EditPageNr == we_base_constants::WE_EDITPAGE_PREVIEW || $we_doc->EditPageNr == we_base_constants::WE_EDITPAGE_THUMBNAILS)){

		  //	makesamedoc only when not in edit_include-window
		  $ctrlElem = $we_doc->getControlElement('checkbox', 'makeSameDoc');

		  $showPubl_makeSamNew = ($ctrlElem && $ctrlElem['hide'] ? '<div style="display: hidden;">' : '') .
		  we_html_forms::checkbox('makeSameDoc', ( $ctrlElem ? $ctrlElem['checked'] : false), 'makeSameDoc', '', false, '', '', ($ctrlElem ? $ctrlElem['readonly'] : false), '', we_html_tools::TYPE_NONE, 0, we_html_button::create_button(we_html_button::FOOTER_MAKE_SAME_DOC, "javascript:var chk=document.getElementById('makeSameDoc');chk.checked=!chk.checked;WE().layout.weEditorFrameController.getActiveEditorFrame().setEditorMakeSameDoc(chk.checked);", '', '', false, false, '', false, g_l('global', '[we_make_same][' . $we_doc->ContentType . ']'), 'switchButton')) .
		  ($ctrlElem && $ctrlElem['hide'] ? '</div>' : '');

		  $main[] = $showPubl_makeSamNew;
		  } */

		//
		//	4. show delete button to delete this document, not in edit_include-window
		//
		if($hasPermDel && $we_doc->ID){
			$optional[] = self::addDelButton($we_doc);
		}
		return we_html_element::div(['class' => 'footertable'], we_html_element::div(['class' => 'main'], implode('', $main)) . we_html_element::div(['class' => 'optional'], implode('', array_reverse($optional))));
	}

	private static function hasNewPerm(we_contents_root $we_doc): bool{
		//	Check permissions for buttons
		switch($we_doc->ContentType){
			case we_base_ContentTypes::HTML:
				return we_base_permission::hasPerm([we_base_perm::WORK_HTML_NEW]);
			case we_base_ContentTypes::CSS:
				return we_base_permission::hasPerm([we_base_perm::WORK_CSS_NEW]);
			case we_base_ContentTypes::JS:
				return we_base_permission::hasPerm([we_base_perm::WORK_JS_NEW]);
			case we_base_ContentTypes::TEXT:
				return we_base_permission::hasPerm([we_base_perm::WORK_TEXT_NEW]);
			/* case we_base_ContentTypes::IMAGE:
			  return we_base_permission::hasPerm([we_base_perm::WORK_IMAGE_NEW]);
			 case we_base_ContentTypes::PDF:
			  return we_base_permission::hasPerm([we_base_perm::WORK_OTHER_NEW]); */
			case we_base_ContentTypes::WEDOCUMENT:
				return we_base_permission::hasPerm([we_base_perm::WORK_DOCUMENT_WEBEDITION_NEW]);
			case we_base_ContentTypes::OBJECT_FILE:
				return we_base_permission::hasPerm([we_base_perm::OBJECTFILE_NEW]);
			case we_base_ContentTypes::FOLDER_DOCUMENT:
				return we_base_permission::hasPerm([we_base_perm::WORK_DOCUMENT_FOLDER_NEW]);
			case we_base_ContentTypes::FOLDER_TEMPLATE:
				return we_base_permission::hasPerm([we_base_perm::WORK_TEMPLATES_FOLDER_NEW]);
			case we_base_ContentTypes::FOLDER_OBJECTFILE:
				return we_base_permission::hasPerm([we_base_perm::OBJECTFILE_NEW_FOLDER]);
		}
		return false;
	}

	private static function inWorkflow(we_contents_root $doc): bool{
		if(!we_modules_info::isActive(we_modules_info::WORKFLOW) || !$doc->IsTextContentDoc){
			return false;
		}
		return ($doc->ID ? we_workflow_utility::inWorkflow($doc->ID, $doc->Table) : false);
	}

	private static function checkUserAccess(we_contents_root $we_doc, bool $allowNew): void{
		switch($we_doc->userHasAccess()){
			case we_contents_root::USER_HASACCESS : //	all is allowed, creator or owner
				break;

			case we_contents_root::FILE_NOT_IN_USER_WORKSPACE : //	file is not in workspace of user
				self::fileInWorkspace();
				exit();

			case we_contents_root::USER_NO_PERM : //	access is restricted and user has no permission
				self::fileIsRestricted($we_doc);
				exit;

			case we_contents_root::FILE_LOCKED: //	file is locked by another user
				self::fileLocked($we_doc, $allowNew);
				exit;

			case we_contents_root::USER_NO_SAVE : //	user has not the right to save the file.
				self::fileNoSave();
				exit;
		}
	}

	private static function hasPermDel(we_contents_root $we_doc): bool{
		switch($we_doc->Table){
			case FILE_TABLE:
				return ($we_doc instanceof we_folder_base && we_base_permission::hasPerm([we_base_perm::WORK_DOCUMENT_FOLDER_DELETE])) ||
					(!$we_doc instanceof we_folder_base && we_base_permission::hasPerm([we_base_perm::WORK_DOCUMENT_DELETE]));
			case TEMPLATES_TABLE:
				return ($we_doc instanceof we_folder_base && we_base_permission::hasPerm([we_base_perm::WORK_TEMP_FOLDER_DELETE])) ||
					(!$we_doc instanceof we_folder_base && we_base_permission::hasPerm([we_base_perm::WORK_TEMPLATE_DELETE]));
			case OBJECT_FILES_TABLE:
				return ((
					($we_doc instanceof we_folder_objectFile && !empty($we_doc->isClassFolder())) ||
					$we_doc instanceof we_contents_objectFile) &&
					we_base_permission::hasPerm([
						we_base_perm::OBJECTFILE_DELETE]));

			case OBJECT_TABLE:
				return we_base_permission::hasPerm([we_base_perm::OBJECT_DELETE]);

			case VFILE_TABLE:
				return ($we_doc instanceof we_folder_base && we_base_permission::hasPerm([we_base_perm::WORK_COLLECTION_FOLDER_DELETE])) ||
					(!$we_doc instanceof we_folder_base && we_base_permission::hasPerm([we_base_perm::WORK_COLLECTION_DELETE]));
			default:
				return false;
		}
		}

	public static function show(): void{
		$we_transaction = we_base_request::_(we_base_request::TRANSACTION, 'we_cmd', we_base_request::_(we_base_request::TRANSACTION, 'we_transaction'), 1);
// init document
		try{
			$we_doc = we_contents_root::getSessionDocument($we_transaction);
		} catch (we_exception_base $e){
			//fix some error on delete
			echo 'no transaction';
			return;
		}
		$haspermNew = self::hasNewPerm($we_doc);

		self::checkUserAccess($we_doc, $haspermNew);

//	preparations of needed vars
		$showPubl = we_base_permission::hasPerm([we_base_perm::WORK_CAN_PUBLISH]) && $we_doc->userCanSave() && $we_doc->IsTextContentDoc;
		$reloadPage = (($showPubl || $we_doc->ContentType === we_base_ContentTypes::TEMPLATE) && (!$we_doc->ID));
		$hasPermDel = self::hasPermDel($we_doc);

		$showGlossaryCheck = 0; /* (!empty($_SESSION['prefs']['force_glossary_check']) &&
		  ( $we_doc->ContentType == we_base_ContentTypes::WEDOCUMENT || $we_doc->ContentType === we_base_ContentTypes::OBJECT_FILE ) ? 1 : 0);
		 */
//	added for we:controlElement type="button" name="save" hide="true"
		$ctrlElem = $we_doc->getControlElement('button', 'save');

		$canWeSave = $we_doc->userCanSave();

		if($canWeSave &&
			(($ctrlElem && $ctrlElem['hide']) ||
			(we_modules_info::isActive(we_modules_info::WORKFLOW) && self::inWorkflow($we_doc) && (!we_workflow_utility::canUserEditDoc($we_doc->ID, $we_doc->Table, we_users_user::getID())))
			)){
			$canWeSave = false;
		}

// publish for templates to save in version
		$pass_publish = $canWeSave && ($showPubl || ($we_doc->ContentType == we_base_ContentTypes::TEMPLATE && defined('VERSIONING_TEXT_WETMPL') && defined('VERSIONS_CREATE_TMPL') && VERSIONS_CREATE_TMPL && VERSIONING_TEXT_WETMPL)) ? [
			['publishWhenSave']] : [];
		['class' => $publClass] = we_editor_header::getDocumentClass($we_doc);
		$doc = [
			'we_transaction' => $we_transaction,
			'ID' => intval($we_doc->ID),
			'Path' => $we_doc->Path,
			'Text' => $we_doc->getText(),
			'Table' => $we_doc->Table,
			'contentType' => $we_doc->ContentType,
			'editFilename' => preg_replace('|/' . $we_doc->Filename . '.*$|', $we_doc->getText(), $we_doc->Path),
			'makeSameDocCheck' => intval(($we_doc->IsTextContentDoc) && $haspermNew && (!self::inWorkflow($we_doc))),
			'isTemplate' => $we_doc->Table === TEMPLATES_TABLE,
			'isFolder' => in_array($we_doc->ContentType, we_base_ContentTypes::FOLDER_TYPES),
			'isBinary' => we_base_ContentTypes::isBinaryFile($we_doc->ContentType),
			'classname' => $publClass,
			'_showGlossaryCheck' => $showGlossaryCheck,
			'weCanSave' => $canWeSave,
			'ctrlElem' => 0,
			'reloadOnSave' => $reloadPage,
			'pass_publish' => $pass_publish,
		];

		if(($we_doc->IsTextContentDoc) && $haspermNew && //	$js_permnew
			(we_editor_functions::isNormalMode() || $we_doc->EditPageNr == we_base_constants::WE_EDITPAGE_CONTENT)){ // not in SeeMode or in editmode
			$ctrlElem = $we_doc->getControlElement('checkbox', 'makeSameDoc');
			$doc['ctrlElem'] = ($ctrlElem ? ($ctrlElem['checked'] ? 1 : 2) : 3);
		}

//	Document is in workflow
		if(self::inWorkflow($we_doc)){
			$body = self::workflow($we_doc, $showPubl);
		} else {
			$_SESSION['weS']['seemForOpenDelSelector'] = [
				'ID' => $we_doc->ID,
				'Table' => $we_doc->Table
			];
			$canSave = $we_doc->userCanSave();
				switch($_SESSION['weS']['we_mode']){
					default:
					case we_base_constants::MODE_NORMAL: // open footer for NormalMode
					$content = $canSave ? self::normalMode($we_doc, $we_transaction, $showPubl, $haspermNew, $hasPermDel) : '';
						break;
					case we_base_constants::MODE_SEE: // open footer for SeeMode
					$content = $canSave ? self::SEEMode($we_doc, $we_transaction, $showPubl, $hasPermDel) :
						we_html_element::div(['class' => 'footertable'], '<span class="fa-stack fa-lg" style="color:#F2F200;margin-right:10px;"><i class="fa fa-exclamation-triangle fa-stack-2x" ></i><i style="color:black;" class="fa fa-exclamation fa-stack-1x"></i>' . g_l('SEEM', '[no_permission_to_edit_document]') . '</span>');
						break;
				}

			$body = we_html_element::body(array_merge([
					'id' => "footerBody",
					'onload' => "we_footerLoaded();",
						], $we_doc->getEditorBodyAttributes(we_contents_root::EDITOR_FOOTER))
					, we_html_element::form([
						'name' => we_base_constants::WE_FORM,
						'action' => '',
						'onsubmit' => ($we_doc instanceof we_folder_objectFile && $we_doc->isClassFolder() ? 'sub();return false;' : '')
						], we_html_element::hidden('sel', $we_doc->ID) .
						$content
			));
		}
		echo we_html_tools::getHtmlTop('', we_html_element::jsScript(JS_DIR . 'we_editor_footer.js', '', ['id' => 'loadVarEditor_footer', 'data-doc' => setDynamicVar($doc)]) . we_gui_progressBar::getJSCode(), $body);
	}

}
