<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: b1df1125e20b $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2020-07-06 19:41 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
if(!we_base_WE::$editmode){
	exit();
}

class we_editor_srcTmpl extends we_editor_base{
	/**
	 * @var null|we_document_html|we_document_text|we_contents_template
	 */
	protected $we_doc = null;

	private function we_getCodeMirror2Code(array &$options): string{
		$useCompletion = false;
		switch($this->we_doc->ContentType){ // Depending on content type we use different parsers and css files
			case we_base_ContentTypes::CSS:
				$mode = 'text/css';
				break;
			case we_base_ContentTypes::JS:
				$mode = 'text/javascript';
				break;
			case we_base_ContentTypes::TEMPLATE:
				$mode = we_base_ContentTypes::TEMPLATE;
				$useCompletion = true;
				break;
			case we_base_ContentTypes::HTML:
				$mode = 'application/x-httpd-php';
				$useCompletion = true;
				break;
			case we_base_ContentTypes::XML:
			case we_base_ContentTypes::HTACCESS:
			default://if we don't know, use XML Mode
				$mode = 'application/xml';
				break;
		}

		$tmp = we_unserialize($_SESSION['prefs']['editorCodecompletion']);
		$hasCompletion = is_array($tmp) ? array_sum($tmp) : false;
		$settings = http_build_query(['settings' => is_array($tmp) ? $tmp : []]);

		$options = [//these are the CodeMirror options
			'mode' => $mode,
			'electricChars' => false,
			'theme' => $_SESSION['prefs']['editorTheme'],
			'lineNumbers' => (bool) ($_SESSION['prefs']['editorLinenumbers']),
			'gutter' => true,
			'foldGutter' => true,
			'minFoldSize' => 5,
			'gutters' => ['CodeMirror-linenumbers', 'CodeMirror-foldgutter'],
			'indentWithTabs' => !$_SESSION['prefs']['editorIndentSpaces'],
			'tabSize' => intval($_SESSION['prefs']['editorTabSize']),
			'indentUnit' => intval($_SESSION['prefs']['editorTabSize']),
			'matchBrackets' => true,
			/* 	workTime: 300,
			  workDelay: 800, */
			'dragDrop' => false,
			'height' => intval(($_SESSION['prefs']['editorHeight'] != 0) ? $_SESSION['prefs']['editorHeight'] : 320),
			'lineWrapping' => (bool) ($_SESSION['weS']['we_wrapcheck']),
			'autoCloseTags' => (bool) ($_SESSION['prefs']['editorDocuintegration']), // use object with indentTags to indent these tags
			'autofocus' => true,
			'smartIndent' => (bool) ($_SESSION['prefs']['editorAutoIndent']),
			'closeCharacters' => '()[]{};>,',
			'hasCodeCompletion' => ($hasCompletion && $useCompletion),
			'showTrailingSpace' => (bool) ($_SESSION['prefs']['editorShowSpaces']),
			'showInvisibles' => (bool) ($_SESSION['prefs']['editorShowSpaces']),
		];

		$maineditor = we_html_element::cssLink(WE_LIB_DIR . 'CodeMirror/lib/codemirror.css') .
			we_html_element::cssLink(WE_LIB_DIR . 'CodeMirror/theme/' . $_SESSION['prefs']['editorTheme'] . '.css') .
			we_html_element::cssLink(WE_LIB_DIR . 'CodeMirror/addon/fold/foldgutter.css') .
			we_html_element::cssLink(WE_LIB_DIR . 'CodeMirror/addon/hint/show-hint.css') .
			we_html_element::cssLink(CSS_DIR . 'template/webEdition.css') .
			we_html_element::jsScript(WE_LIB_DIR . 'CodeMirror/lib/codemirror.js');

		foreach([
		'addon/edit/closetag.js',
		'addon/edit/matchbrackets.js',
		'addon/edit/trailingspace.js',
		'addon/fold/brace-fold.js',
		'addon/fold/comment-fold.js',
		'addon/fold/foldcode.js',
		'addon/fold/foldgutter.js',
		'addon/fold/markdown-fold.js',
		'addon/fold/xml-fold.js',
		'addon/hint/show-hint.js',
		'addon/hint/xml-hint.js',
		'addon/mode/overlay.js',
		'addon/search/searchcursor.js',
		'mode/clike/clike.js',
		'mode/css/css.js',
		'mode/htmlmixed/htmlmixed.js',
		'mode/javascript/javascript.js',
		'mode/php/php.js',
		'mode/sass/sass.js',
		'mode/xml/xml.js',
		] as $js){
			$maineditor .= we_html_element::jsScript(WE_LIB_DIR . 'CodeMirror/' . $js);
		}
		return
			$maineditor .
			we_html_element::jsScript(JS_DIR . 'codemirror/webEdition.js') .
			we_html_element::jsScript(JS_DIR . 'codemirror/show-invisibles.js') .
			we_html_element::jsScript(JS_DIR . 'codemirror/we-hint.js') .
			we_html_element::jsScript(WEBEDITION_DIR . 'we_cmd.php?we_cmd[0]=loadSourceEditorConsts&we_cmd[1]=tagJS&' . $settings) .
//($hasCompletion && $useCompletion ?		 :		''		) .
			($_SESSION['prefs']['editorTooltips'] ? we_html_element::cssLink(WEBEDITION_DIR . 'we_cmd.php?we_cmd[0]=loadSourceEditorConsts&we_cmd[1]=tagCSS&' . $settings) : '' ) .
			we_html_element::cssLink(WEBEDITION_DIR . 'we_cmd.php?we_cmd[0]=loadSourceEditorConsts&we_cmd[1]=CMCSS&' . http_build_query([
					'settings' => [
						'font' => [
							'size' => ($_SESSION['prefs']['editorFont'] && $_SESSION['prefs']['editorFontsize'] ? $_SESSION['prefs']['editorFontsize'] : '12'),
							'family' => ($_SESSION['prefs']['editorFont'] && $_SESSION['prefs']['editorFontname'] ? $_SESSION['prefs']['editorFontname'] : 'monospace')
						],
						'toolFont' => [
							'size' => ($_SESSION['prefs']['editorTooltipFont'] && $_SESSION['prefs']['editorTooltipFontsize'] ? $_SESSION['prefs']['editorTooltipFontsize'] : '12'),
							'family' => ($_SESSION['prefs']['editorTooltipFont'] && $_SESSION['prefs']['editorTooltipFontname'] ? $_SESSION['prefs']['editorTooltipFontname'] : 'sans-serif'),
						],
						'tab' => $_SESSION['prefs']['editorShowTab'],
						'Template' => [
							'tid' => ($this->we_doc->ContentType === we_base_ContentTypes::TEMPLATE && $_SESSION['prefs']['editorTooltipsIDs'] ? $this->we_doc->ID : 0),
							'moddate' => ($this->we_doc->ContentType === we_base_ContentTypes::TEMPLATE && $_SESSION['prefs']['editorTooltipsIDs'] ? $this->we_doc->Mod_Date : 0)
						],
		]]));
	}

	private function getInitEditor(): string{
		switch($_SESSION['prefs']['editorMode']){
			case 'java':
			case 'codemirror2':
				return'initCM();';
			default:
				return 'initDefaultEdior();';
		}
	}

	private function getTagWizzard(): array{
		// Code Wizard
		$allWeTags = we_wizard_tag::getExistingWeTags();

		$tagGroups = we_wizard_tag::getWeTagGroups();

		$selectedGroup = !empty($this->we_doc->TagWizardSelection) ? $this->we_doc->TagWizardSelection : "alltags";
		$groupselect = '<select class="weSelect" style="width: 250px;" id="weTagGroupSelect" name="we_' . $this->we_doc->Name . '_TagWizardSelection" onchange="selectTagGroup(this.value);">
<optgroup label="' . g_l('weCodeWizard', '[snippets]') . '">
<option value="snippet_standard" ' . ($selectedGroup === 'snippet_standard' ? 'selected="selected"' : '') . '>' . g_l('weCodeWizard', '[standard_snippets]') . '</option>
		<option value="snippet_custom" ' . ($selectedGroup === 'snippet_custom' ? 'selected="selected"' : '') . '>' . g_l('weCodeWizard', '[custom_snippets]') . '</option>
		</optgroup>
		<optgroup label="we:tags">';

		foreach(array_keys($tagGroups) as $tagGroupName){
			$tagGroupName = (string) $tagGroupName;
			switch($tagGroupName){
				case 'custom_tags':
					$tagGroupName = 'custom';
				case 'custom':
					$groupselect .= '<option value="-1" disabled="disabled">----------</option>';
			}
			$groupselect .= '<option value="' . $tagGroupName . '"' . ($tagGroupName == $selectedGroup ? ' selected="selected"' : '') . '">' . (we_modules_info::isActive($tagGroupName) ? g_l('menu_moduleInformation', '[' . $tagGroupName . '][text]') : g_l('weTagGroups', '[' . $tagGroupName . ']')) . '</option>';
			if($tagGroupName === 'alltags'){
				$groupselect .= '<option value="-1" disabled="disabled">----------</option>';
			}
		}
		$groupselect .= '</optgroup></select>';

		$tagselect = '<select onkeydown="return openTagWizWithReturn(event?event:window.event)" class="defaultfont" style="width: 250px; height: 100px;" size="7" ondblclick="edit_wetag(this.value);" name="tagSelection" id="tagSelection" onchange="WE().layout.button.enable(document, \'btn_direction_right_applyCode\')">';

		foreach($allWeTags as $tag){
			$tagselect .= '<option value="' . $tag . '">' . $tag . '</option>';
		}

		$tagselect .= '</select>';

		// buttons
		$editTagbut = we_html_button::create_button(we_html_button::DIRRIGHT, "javascript:executeEditButton();", '', '', false, false, "_applyCode");
		$tagWizardHtml = we_html_element::jsScript(JS_DIR . 'we_srcTmpl.js') . '
<table id="wizardTable" style="width:700px;" class="default defaultfont">
	<tr><td style="padding-bottom:5px;">' . $groupselect . '</td></tr>
	<tr>
		<td id="tagSelectCol" style="width: 250px;">' . $tagselect . we_wizard_code::getSelect() . we_wizard_code::getSelect('custom') . '</td>
		<td id="spacerCol" style="width: 50px;text-align:center">' . $editTagbut . '</td>
		<td id="tagAreaCol" style="width: 100%;text-align:right">' . we_html_element::textArea([
				'name' => 'we_' . $this->we_doc->Name . '_TagWizardCode',
				'id' => 'tag_edit_area',
				'style' => 'width:400px; height:100px;' . ($_SESSION['prefs']['editorFont'] ?
				' font-family: ' . $_SESSION['prefs']['editorFontname'] . '; font-size: ' . $_SESSION['prefs']['editorFontsize'] . 'px;' :
				''),
				'class' => 'defaultfont'
				], $this->we_doc->TagWizardCode) . '</td>
	</tr>
</table>';

		return ['group' => $selectedGroup, 'html' => $tagWizardHtml];
	}

	public function show(): string{
		if(!isset($_SESSION['weS']['we_wrapcheck'])){
			$_SESSION['weS']['we_wrapcheck'] = $_SESSION['prefs']['editorWrap'];
		}

		$code = $this->we_doc->getDocumentCode();

		$maineditor = '<textarea id="editarea" style="' . (($_SESSION['prefs']['editorFont'] == 1) ? ' font-family: ' . $_SESSION['prefs']['editorFontname'] . '; font-size: ' . $_SESSION['prefs']['editorFontsize'] . 'px;' : '') .
			'-moz-tab-size:' . $_SESSION['prefs']['editorTabSize'] . '; -o-tab-size:' . $_SESSION['prefs']['editorTabSize'] . '; -webkit-tab-size:' . $_SESSION['prefs']['editorTabSize'] . '; tab-size:' . $_SESSION['prefs']['editorTabSize'] . ';' .
			'" name="we_' . $this->we_doc->Name . '_txt[data]" wrap="' . ($_SESSION['weS']['we_wrapcheck'] ? 'virtual' : 'off') . '" ' .
			($_SESSION['prefs']['editorMode'] === 'codemirror2' ? '' : (we_base_browserDetect::isIE() || we_base_browserDetect::isOpera() ? 'onkeydown' : 'onkeypress') . '="editorChanged();return wedoKeyDown(this,event);"') . '>'
			. htmlspecialchars($code) . '</textarea>';
		$options = [];
		switch($_SESSION['prefs']['editorMode']){
			case 'java':
			case 'codemirror2': //Syntax-Highlighting
				$maineditor .= $this->we_getCodeMirror2Code($options);
				break;
		}

		$znr = -1;

		if($this->we_doc->ContentType === we_base_ContentTypes::TEMPLATE){
			['group' => $selectedGroup, 'html' => $twCode] = $this->getTagWizzard();
			$parts = (new we_html_multiIconBoxContent())
				->add(new we_html_multiIconBoxContentLine('', $twCode));
			$znr = 1;
		} else {
			$parts = null;
			$selectedGroup = '';
		}

		return $this->getPage(
				we_html_element::div(['id' => 'bodydiv', 'style' => 'display:none;position:absolute;top:10px;left:20px;right:20px;bottom:0;'], we_html_element::div(['id' => 'editorDiv',], $maineditor .
						we_html_element::div(['id' => 'srtable'], ($_SESSION['prefs']['editorMode'] === 'codemirror2' ?
							we_html_element::div([], '<input type="text" style="width: 10em;" id="query" placeholder="' . g_l('button', '[search][alt]') . '" onkeydown="cmSearch(event);"/>' . we_html_button::create_button(we_html_button::SEARCH, 'javascript:cmSearch(null);') .
								we_html_forms::checkbox('1', false, 'caseSens', g_l('weClass', '[caseSensitive]'))
							) .
							we_html_element::div([], '<input type="text" style="margin-left:2em;width: 10em;float:left;" id="replace" placeholder="' . g_l('button', '[replace][value]') . '" onkeydown="cmReplace(event);"/>' . we_html_button::create_button(we_html_button::REPLACE, 'javascript:cmReplace(null);')
							) : ''
							) .
							we_html_element::div([], we_html_forms::checkbox('1', ($_SESSION['weS']['we_wrapcheck'] == 1), 'we_wrapcheck_tmp', g_l('global', '[wrapcheck]'), false, '', ($_SESSION['prefs']['editorMode'] === 'codemirror2' ? 'editor.setOption(\'lineWrapping\',this.checked);' : "we_cmd('wrap_on_off',this.checked)"))
							) .
							($_SESSION['prefs']['editorMode'] === 'codemirror2' ?
							we_html_element::div(['id' => "reindentButton", 'style' => "margin-left:10px;"], we_html_button::create_button('fa:reindent,fa-lg fa-indent', 'javascript:reindent();')) : '')
						)
				)) .
				($parts ? (new we_html_multiIconBox($parts, 'weTMPLDocEdit'))
					->marginLeft()
					->fold($znr, false, g_l('weClass', '[showTagwizard]'), g_l('weClass', '[hideTagwizard]'), 'sizeEditor();')
					->get() : ''), we_html_element::jsScript(JS_DIR . 'we_srcTmpl.js', '', [
					'id' => 'loadVarSrcTmpl', 'data-doc' => setDynamicVar([
						'docName' => $this->we_doc->Name,
						'editorHighlightCurrentLine' => intval($_SESSION['prefs']['editorHighlightCurrentLine']),
						'CMOptions' => $options,
				])]), [
				'style' => "overflow:hidden;",
				'onload' => ($selectedGroup ? "selectTagGroup('" . $selectedGroup . "');" : '') . $this->getInitEditor(),
				'onunload' => "doUnload();parent.editorScrollPosTop = getScrollPosTop();parent.editorScrollPosLeft = getScrollPosLeft();",
				'onresize' => "sizeEditor();"
				]
		);
	}

}
