<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: 3ab2535a2e99 $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2020-03-31 01:46 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
class we_export_treeMain extends we_tree_base{

	protected function customJSFile(): string{
		return we_html_element::jsScript(WE_JS_MODULES_DIR . 'export/export_treeMain.js', 'initTree();');
	}

	/**
	 *
	 * @param int $ParentID
	 * @param int $offset
	 * @param int $segment
	 * @param bool $sort
	 * @return array<int,array{id:string,parentid:int,path:string,text:string,typ:string,open:bool,isfolder:bool,disabled:bool,tooltip:string,offset:int,contenttype:string,published:bool,table:string}>
	 */
	public static function getItems(int $ParentID, int $offset = 0, int $segment = 500, bool $sort = false): array{
		$db = new DB_WE();

		$items = [];

		$prevoffset = (int) max(0, $offset - $segment);
		if($offset && $segment){
			$items[] = [
				'id' => 'prev_' . $ParentID,
				'parentid' => $ParentID,
				'path' => '',
				'text' => 'display (' . $prevoffset . '-' . $offset . ')',
				'contenttype' => self::CT_UP,
				'table' => EXPORT_TABLE,
				'typ' => self::TYP_DOTS,
				'open' => false,
				'isfolder' => false,
				'published' => false,
				'disabled' => false,
				'tooltip' => '',
				'offset' => $prevoffset,
			];
		}

		$where = ' WHERE ParentID=' . $ParentID;
		if(($permittedExportTypes = we_export_frames::getPermittedExportTypes())){
			$where .= " AND ExportType IN ('" . implode("','", $permittedExportTypes) . "')";
		}

		$data = $db->getAllq('SELECT ID,ParentID,Path,Text,IsFolder FROM ' . EXPORT_TABLE . $where . ' ORDER BY IsFolder DESC,(text REGEXP "^[0-9]") DESC,abs(text),Text' . ($segment ? ' LIMIT ' . $offset . ',' . $segment : '' ));

		foreach($data as $cur){
			$items[] = [
				'id' => (string) $cur['ID'],
				'parentid' => (int) $cur['ParentID'],
				'path' => (string) $cur['Path'],
				'text' => (string) $cur['Text'],
				'table' => EXPORT_TABLE,
				'isfolder' => (bool) $cur['IsFolder'],
				'typ' => ($cur['IsFolder'] ? self::TYP_FOLDER : self::TYP_ITEM),
				'open' => false,
				'contenttype' => ($cur['IsFolder'] ? we_base_ContentTypes::FOLDER_COMMON : 'we/export'),
				'published' => true,
				'disabled' => false,
				'tooltip' => (string) $cur['ID'],
				'offset' => $offset,
			];
		}

		$total = fI('SELECT COUNT(1) FROM ' . EXPORT_TABLE . ' ' . $where);
		$nextoffset = $offset + $segment;
		if($segment && ($total > $nextoffset)){
			$items[] = [
				'id' => 'next_' . $ParentID,
				'parentid' => 0,
				'path' => '',
				'text' => 'display (' . $nextoffset . '-' . ($nextoffset + $segment) . ')',
				'contenttype' => self::CT_DOWN,
				'table' => EXPORT_TABLE,
				'typ' => self::TYP_DOTS,
				'open' => false,
				'isfolder' => false,
				'disabled' => false,
				'tooltip' => '',
				'offset' => $nextoffset,
				'published' => false,
			];
		}

		return $items;
	}

}
