<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: 2fcfd49ac317 $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2020-07-06 14:53 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
abstract class we_glossary_frameEditorItem extends we_glossary_frameEditor{

	public static function Header(we_glossary_frames $weGlossaryFrames, we_glossary_view $view): string{
		$we_tabs = new we_gui_tabs();
		$we_tabs->addTab('', we_gui_tabs::WE_ICON_PROPERTIES, true, '1', ['title' => g_l('modules_glossary', '[property]')]);

		return self::buildHeader($weGlossaryFrames, $we_tabs, $view->getTitle(), ($view->Model->ID ? htmlspecialchars($view->Model->Text) : g_l('modules_glossary', '[menu_new]')) . '<div id="mark"><i class="fa fa-asterisk modified"></i></div>');
	}

	public static function Body(we_glossary_frames $weGlossaryFrames, we_glossary_view $view): string{
		$tabNr = we_base_request::int('tabnr', 1);
		$tabNr = ($view->Model->IsFolder && $tabNr != 1) ? 1 : $tabNr;
		$glossary = $view->Model;
		$mode = $glossary->getAttribute('mode');
		$out = we_html_element::div(['id' => 'tab1', 'style' => ($tabNr == 1 ? '' : 'display: none')],
															(new we_html_multiIconBox(self::getHTMLTabProperties($glossary)))->marginLeft()->fold(2, false, g_l('navigation', '[more_attributes]'))->get());


		$head = we_html_element::jsScript(JS_DIR . 'multiIconBox.js') .
			we_html_element::jsScript(WE_JS_MODULES_DIR . 'glossary/we_glossary_frameEditorItem.js');

		$weGlossaryFrames->jsCmd->addCmd('loadHeaderFooter', $glossary->Type, ($glossary->Type === we_glossary_glossary::TYPE_LINK ? ($mode ?: we_glossary_glossary::MODE_LINK_INT) : ''), ($mode === we_glossary_glossary::MODE_LINK_CAT ? ($glossary->getAttribute('modeCategory') ?: we_glossary_glossary::MODE_CATEGORY_INT) : ''));

		return self::buildBody($weGlossaryFrames, $view, $out, $head, "showLinkMode('" . $mode . "');");
	}

	public static function Footer(we_glossary_frames $weGlossaryFrames, we_glossary_view $view): string{
		$ShowUnpublish = ($view->Model->ID == 0 || $view->Model->Publish_Date !== null);

		$main = we_html_element::div(
				['class' => 'main'],
		($ShowUnpublish ?
				we_html_button::create_button('deactivate', "javascript:top.content.editor.edbody.document.getElementById('Published').value=0;top.opener.top.we_cmd('save_glossary')", '', '', (!we_base_permission::hasPerm([
						we_base_perm::GLOSSARY_NEW, we_base_perm::GLOSSARY_EDIT]))) :
				'') .
				we_html_button::create_button(we_html_button::SAVE, "javascript:if(top.publishWhenSave==1){top.content.editor.edbody.document.getElementById('Publish_Date').value=1;};top.content.we_cmd('save_glossary');", '', '', (!we_base_permission::hasPerm([
						we_base_perm::GLOSSARY_NEW, we_base_perm::GLOSSARY_EDIT]))) .
				(!$ShowUnpublish ?
				we_html_forms::checkbox('1', false, 'publishWhenSave', g_l('modules_glossary', '[publish_when_saved]'), false, "defaultfont", "top.publishWhenSave = (this.checked) ? 1 : 0", false) :
				'') .
				we_html_forms::checkbox('1', false, 'makeNewEntry', g_l('modules_glossary', '[new_item_after_saving]'), false, "defaultfont", "top.makeNewEntryCheck = (this.checked) ? 1 : 0", false));

		$optional = we_html_element::div(['class' => 'optional'],
																	 ($view->Model->ID && we_base_permission::hasPerm([we_base_perm::GLOSSARY_DELETE]) ?
				we_html_button::create_button(we_html_button::TRASH, "javascript:top.we_cmd('delete_glossary');") : '')
		);
		$weGlossaryFrames->jsCmd->addCmd('checkSaveNew');

		return self::buildFooter($weGlossaryFrames, we_html_element::form([], we_html_element::div(['class' => 'footertable'], $main . $optional)));
	}

	private static function getHTMLTabProperties(we_glossary_glossary $glossary): we_html_multiIconBoxContent{
		$types = [we_glossary_glossary::TYPE_ACRONYM => g_l('modules_glossary', '[acronym]'),
			we_glossary_glossary::TYPE_ABBREVATION => g_l('modules_glossary', '[abbreviation]'),
			we_glossary_glossary::TYPE_FOREIGNWORD => g_l('modules_glossary', '[foreignword]'),
			we_glossary_glossary::TYPE_LINK => g_l('modules_glossary', '[link]'),
			we_glossary_glossary::TYPE_TEXTREPLACE => g_l('modules_glossary', '[textreplacement]'),
		];

		$hidden = we_html_element::hidden('newone', ($glossary->ID == 0 ? 1 : 0)) .
			we_html_element::hidden('Published', $glossary->ID == 0 ? 1 : ($glossary->Publish_Date === null ? 0 : 1), 'Published');


		$language = ($glossary->Language ?: we_base_WE::$Langs['DefaultFrontendLanguage']);

		$content = $hidden . '<table class="default defaultfont">
	<tr><td>' . g_l('modules_glossary', '[folder]') . '</td></tr>
	<tr><td>' . we_html_tools::select('Language', we_base_country::getWeFrontendLanguagesForBackend(), 1, $language, false, ['onchange' => "top.content.setHot();"], true, 520) . '</td></tr>
	<tr><td>' . g_l('modules_glossary', '[type]') . '</td></tr>
	<tr><td>' . we_html_tools::select('Type', $types, 1, $glossary->Type, false, ['onchange' => "top.content.setHot();showType(this.value);"], true, 520) . '</td></tr>
	<tr><td>' . we_html_forms::checkboxWithHidden((bool) $glossary->Fullword, 'Fullword', g_l('modules_glossary', '[Fullword]'), false, '', 'top.content.setHot();') . '</td></tr>
</table>';
		$parts = (new we_html_multiIconBoxContent())
			->add(new we_html_multiIconBoxContentLine(g_l('modules_glossary', '[path]'), $content, we_html_multiIconBox::PROP_PATH))
			->add((new we_html_multiIconBoxContentLine(g_l('modules_glossary', '[selection]'), self::getHTMLAbbreviation($glossary) .
				self::getHTMLAcronym($glossary) .
				self::getHTMLForeignWord($glossary) .
				self::getHTMLLink($glossary) .
				self::getHTMLTextReplacement($glossary), we_html_multiIconBox::PROP_LINK))
			->noline()
		);
		return $parts->append(self::getHTMLLinkAttributes($glossary));
	}

	private static function getHTMLAbbreviation(we_glossary_glossary $glossary): string{
		if($glossary->Type == we_glossary_glossary::TYPE_ABBREVATION){
			$text = html_entity_decode($glossary->Text);
			$title = html_entity_decode($glossary->Title);
			$language = $glossary->getAttribute('lang');
		} else {
			$text = $title = $language = '';
		}


		return we_html_element::div(['id' => 'type_abbreviation', 'style' => "display: block;"], '
<table class="default defaultfont">
	<tr><td>' . g_l('modules_glossary', '[abbreviation]') . '</td></tr>
	<tr><td>' . we_html_tools::textInput("abbreviation[Text]", $text, 255, ['onchange' => "setHot();"], 'text', '520px') . '</td></tr>
	<tr><td>' . g_l('modules_glossary', '[announced_word]') . '</td></tr>
	<tr><td>' . we_html_tools::textInput("abbreviation[Title]", $title, 255, ['onchange' => "setHot();"], 'text', '520px') . '</td></tr>
	<tr><td>' . self::getLangField("abbreviation[Attributes][lang]", $language, g_l('modules_glossary', '[language]')) . '</td></tr>
</table>');
	}

	private static function getHTMLAcronym(we_glossary_glossary $glossary): string{
		if($glossary->Type == we_glossary_glossary::TYPE_ACRONYM){
			$text = html_entity_decode($glossary->Text);
			$title = html_entity_decode($glossary->Title);
			$language = $glossary->getAttribute('lang');
		} else {
			$text = $title = $language = '';
		}

		return we_html_element::div(['id' => 'type_acronym', 'style' => "display: none;"],
															'<table class="default defaultfont">
	<tr><td>' . g_l('modules_glossary', '[acronym]') . '</td></tr>
	<tr><td>' . we_html_tools::textInput("acronym[Text]", $text, 255, ['onchange' => "setHot();"], 'text', '520px') . '</td></tr>
	<tr><td>' . g_l('modules_glossary', '[announced_word]') . '</td></tr>
	<tr><td>' . we_html_tools::textInput("acronym[Title]", $title, 255, ['onchange' => "setHot();"], 'text', '520px') . '</td></tr>
	<tr><td>' . self::getLangField('acronym[Attributes][lang]', $language, g_l('modules_glossary', '[language]')) . '</td></tr>
</table>');
	}

	private static function getHTMLForeignWord(we_glossary_glossary $glossary): string{
		if($glossary->Type == we_glossary_glossary::TYPE_FOREIGNWORD){
			$text = html_entity_decode($glossary->Text);
			$language = $glossary->getAttribute('lang');
		} else {
			$text = $language = '';
		}

		return we_html_element::div(['id' => 'type_foreignword', 'style' => "display: none;"],
															'<table class="default defaultfont">
	<tr><td>' . g_l('modules_glossary', '[foreignword]') . '</td></tr>
	<tr><td>' . we_html_tools::textInput("foreignword[Text]", $text, 255, ['onchange' => "setHot();"], 'text', '520px') . '</td></tr>
	<tr><td>' . self::getLangField("foreignword[Attributes][lang]", $language, g_l('modules_glossary', '[language]')) . '</td></tr>
</table>');
	}

	private static function getHTMLTextReplacement(we_glossary_glossary $glossary): string{
		if($glossary->Type == we_glossary_glossary::TYPE_TEXTREPLACE){
			$text = html_entity_decode($glossary->Text, ENT_COMPAT, 'UTF-8');
			$title = html_entity_decode($glossary->Title, ENT_COMPAT, 'UTF-8');
		} else {
			$title = $text = '';
		}

		return we_html_element::div(['id' => 'type_textreplacement', 'style' => "display: none;"], '<table class="default defaultfont">
<tr><td>' . g_l('modules_glossary', '[textreplacement]') . '</td></tr>
<tr><td>' . we_html_tools::textInput("textreplacement[Text]", $text, 255, ['onchange' => "setHot();"], 'text', '520px') . '</td></tr>
<tr><td>' . g_l('modules_glossary', '[textreplacement_Text]') . '</td></tr>
<tr><td>' . we_html_tools::textInput("textreplacement[Title]", $title, 255, ['onchange' => "setHot();"], 'text', '520px') . '</td></tr>
</table>');
	}

	private static function getHTMLLink(we_glossary_glossary $glossary): string{
		if($glossary->Type == we_glossary_glossary::TYPE_LINK){
			$text = html_entity_decode($glossary->Text);
			$mode = $glossary->getAttribute('mode');
		} else {
			$text = $mode = '';
		}

		return we_html_element::div(['id' => 'type_link', 'style' => "display: none;"],
															'<table class="default defaultfont">
	<tr><td>' . g_l('modules_glossary', '[link]') . '</td></tr>
	<tr><td>' . we_html_tools::textInput('link[Text]', $text, 255, ['onchange' => "setHot();"], 'text', '520px') . '</td></tr>
	<tr><td>' . we_html_tools::select('link[Attributes][mode]', [
					we_glossary_glossary::MODE_LINK_INT => g_l('modules_glossary', '[link_intern]'),
					we_glossary_glossary::MODE_LINK_EXT => g_l('modules_glossary', '[link_extern]'),
					we_glossary_glossary::MODE_LINK_OBJ => g_l('modules_glossary', '[link_object]'),
					we_glossary_glossary::MODE_LINK_CAT => g_l('modules_glossary', '[link_category]'),
					], 1, $mode, false, ['onchange' => "setHot();showLinkMode(this.value);"], true, 520) . '</td></tr>
</table>' .
				self::getHTMLIntern($glossary) .
				self::getHTMLExtern($glossary) .
				self::getHTMLObject($glossary) .
				self::getHTMLCategory($glossary)
		);
	}

	private static function getHTMLIntern(we_glossary_glossary $glossary): string{
		$cmd = "javascript:we_cmd('we_selector_document',{id:document.we_form.elements['link[Attributes][InternLinkID]'].value,table:WE().consts.tables.FILE_TABLE,targetID:'link[Attributes][InternLinkID]',targetName:'link[Attributes][InternLinkPath]',rootDirID:0})";

		if($glossary->Type === we_glossary_glossary::TYPE_LINK && $glossary->getAttribute('mode') === we_glossary_glossary::MODE_LINK_INT){
			//$linkPath = $glossary->getAttribute('InternLinkPath');
			$linkID = (int) $glossary->getAttribute('InternLinkID');
			$linkPath = id2path($linkID);
			$glossary->setAttribute('InternLinkPath', $linkPath);
			$internParameter = $glossary->getAttribute('InternParameter');
		} else {
			$linkPath = $linkID = $internParameter = '';
		}
		$weSuggest = we_gui_suggest::getInstance();
		$weSuggest->setAcId('docPath')
			->setContentType(we_base_ContentTypes::LINK_ABLE)
			->setInput('link[Attributes][InternLinkPath]', $linkPath)
			->setMaxResults(10)
			->setSelectButton(we_html_button::create_button(we_html_button::SELECT, $cmd))
			->setResult('link[Attributes][InternLinkID]', (string) $linkID)
			->setSelector(we_gui_suggest::DocSelector)
			->setTable(FILE_TABLE)
			->setWidth(400);

		return we_html_element::div(['id' => 'mode_intern', 'style' => "display: none;"],
															'<table class="default defaultfont">
	<tr><td>' . $weSuggest->getHTML() . '</td></tr>
	<tr><td>' . g_l('modules_glossary', '[parameter]') . '</td></tr>
	<tr><td>' . we_html_tools::textInput('link[Attributes][InternParameter]', $internParameter, 0, ['onchange' => "setHot();"], 'text', '520px') . '</td></tr>
</table>');
	}

	private static function getHTMLExtern(we_glossary_glossary $glossary): string{
		if($glossary->Type == we_glossary_glossary::TYPE_LINK && $glossary->getAttribute('mode') === we_glossary_glossary::MODE_LINK_EXT){
			$url = $glossary->getAttribute('ExternUrl');
			$parameter = $glossary->getAttribute('ExternParameter');
		} else {
			$url = we_base_link::EMPTY_EXT;
			$parameter = '';
		}

		return we_html_element::div(['id' => 'mode_extern', 'style' => "display: none;"],
															'<table class="default defaultfont">
		<tr><td>' . we_html_tools::textInput('link[Attributes][ExternUrl]', $url, 0, ['onchange' => "setHot();"], 'text', '520px') . '</td></tr>
		<tr><td>' . g_l('modules_glossary', '[parameter]') . '</td></tr>
		<tr><td>' . we_html_tools::textInput('link[Attributes][ExternParameter]', $parameter, 0, ['onchange' => "setHot();"], 'text', '520px') . '</td></tr>
	</table>');
	}

	private static function getHTMLObject(we_glossary_glossary $glossary): string{
		if($glossary->Type == we_glossary_glossary::TYPE_LINK && $glossary->getAttribute('mode') === we_glossary_glossary::MODE_LINK_OBJ){
			$linkPath = $glossary->getAttribute('ObjectLinkPath');
			$linkID = $glossary->getAttribute('ObjectLinkID');
			$workspaceID = $glossary->getAttribute('ObjectWorkspaceID');
			$parameter = $glossary->getAttribute('ObjectParameter');
		} else {
			$linkPath = $workspaceID = $parameter = '';
			$linkID = 0;
		}

		$cmd = we_modules_info::isActive(we_modules_info::OBJECT) ? "javascript:we_cmd('we_selector_document',{id:document.we_form.elements['link[Attributes][ObjectLinkID]'].value,table:WE().consts.tables.OBJECT_FILES_TABLE,targetID:'link[Attributes][ObjectLinkID]',targetName:'link[Attributes][ObjectLinkPath]',jsCmd:'populateWorkspaces',rootDirID:0,filter:WE().consts.contentTypes.OBJECT_FILE,open_doc:" . (we_base_permission::hasPerm([
				we_base_perm::OBJECTS_CAN_SELECT_OTHER_USERS]) ? 0 : 1) . "})" : '';

		$weSuggest = we_gui_suggest::getInstance();
		$weSuggest->setAcId('objPathLink')
			->setContentType([we_base_ContentTypes::FOLDER_OBJECTFILE, we_base_ContentTypes::OBJECT_FILE])
			->setInput('link[Attributes][ObjectLinkPath]', $linkPath)
			->setMaxResults(10)
			->setSelectButton(we_html_button::create_button(we_html_button::SELECT, $cmd))
			->setResult('link[Attributes][ObjectLinkID]', (string) $linkID)
			->setSelector(we_gui_suggest::DocSelector)
			->setTable(OBJECT_FILES_TABLE)
			->setWidth(400);

		$wsid = ($glossary->getAttribute('ObjectLinkID') ? we_navigation_dynList::getWorkspacesForObject(intval($glossary->getAttribute('ObjectLinkID'))) : []);

		return we_html_element::div(['id' => 'mode_object', 'style' => "display: none;"], '
	<table class="default">
			<tr><td>' . $weSuggest->getHTML() . '</td></tr>
</table>' .
				we_html_element::div(['id' => 'ObjectWorkspaceID', 'style' => "display: block;"], '
		<table class="default defaultfont">
			<tr><td>' . g_l('modules_glossary', '[workspace]') . '</td></tr>
			<tr><td>' . we_html_tools::select('link[Attributes][ObjectWorkspaceID]', $wsid, 0, $workspaceID, false, ['style' => "width:520px; border: #AAAAAA solid 1px;",
						'onchange' => "setHot();"]) . '</td></tr>
</table>') . '
	<table class="default defaultfont">
		<tr><td>' . g_l('modules_glossary', '[parameter]') . '</td></tr>
		<tr><td>' . we_html_tools::textInput('link[Attributes][ObjectParameter]', $parameter, 0, ['onchange' => "setHot();"], 'text', '520px') . '</td></tr>
	</table>');
	}

	private static function getHTMLCategory(we_glossary_glossary $glossary): string{
		if($glossary->Type == we_glossary_glossary::TYPE_LINK && $glossary->getAttribute('mode') === we_glossary_glossary::MODE_LINK_CAT){
			$linkPath = $glossary->getAttribute('CategoryLinkPath');
			$linkID = $glossary->getAttribute('CategoryLinkID');
			$internLinkPath = $glossary->getAttribute('CategoryInternLinkPath');
			$internLinkID = $glossary->getAttribute('CategoryInternLinkID');
			$modeCategory = $glossary->getAttribute('modeCategory');
			$url = $glossary->getAttribute('CategoryUrl');
			$catParameter = $glossary->getAttribute('CategoryCatParameter');
			$parameter = $glossary->getAttribute('CategoryParameter');
		} else {
			$linkPath = '/';
			$linkID = 0;
			$internLinkPath = '';
			$internLinkID = '';
			$modeCategory = 0;
			$url = we_base_link::EMPTY_EXT;
			$catParameter = '';
			$parameter = '';
		}

		$selector1 = we_html_tools::formElementTable('', we_html_tools::textInput('link[Attributes][CategoryLinkPath]', $linkPath, 0, ['onchange' => "setHot();", 'readonly' => 'readonly'], 'text', '400px'), we_html_element::hidden('link[Attributes][CategoryLinkID]', $linkID), we_html_button::create_button(we_html_button::SELECT, "javascript:we_cmd('we_selector_document',{id:document.we_form.elements['link[Attributes][CategoryLinkID]'].value,table:WE().consts.tables.CATEGORY_TABLE,targetID:'link[Attributes][CategoryLinkID]',targetName:'link[Attributes][CategoryLinkPath]',jsCmd:'setHot',rootDirID:0})"));


		$selector2 = we_html_tools::formElementTable('', we_html_tools::textInput('link[Attributes][CategoryInternLinkPath]', $internLinkPath, 0, ['onchange' => "setHot();",
					'readonly' => 'readonly'], 'text', '400px'), we_html_element::hidden('link[Attributes][CategoryInternLinkID]', $internLinkID), we_html_button::create_button(we_html_button::SELECT, "javascript:we_cmd('we_selector_document',{id:document.we_form.elements['link[Attributes][CategoryInternLinkID]'].value,table:WE().consts.tables.FILE_TABLE,targetID:'link[Attributes][CategoryInternLinkID]',targetName:'link[Attributes][CategoryInternLinkPath]',rootDirID:0})")
		);

		return we_html_element::div(['id' => 'mode_category', 'style' => "display: none;"],
															'<table class="default defaultfont">
		<tr><td>' . $selector1 . '</td></tr>
		<tr><td>' . g_l('modules_glossary', '[link_selection]') . '</td></tr>
		<tr><td>' . we_html_tools::select('link[Attributes][modeCategory]', [we_glossary_glossary::MODE_CATEGORY_INT => g_l('modules_glossary', '[link_intern]'),
					we_glossary_glossary::MODE_CATEGORY_EXT => g_l('modules_glossary', '[link_extern]'),
					], 1, $modeCategory, false, ['onchange' => "setHot();showLinkModeCategory(this.value);"], true, 520) . '</td></tr>
	</table>
	<div id="mode_category_intern" style="display: none;">
	<table class="default">
		<tr><td>' . $selector2 . '</td></tr>
	</table>
	</div>
	<div id="mode_category_extern" style="display: none;">
		<table class="default">
			<tr><td>' . we_html_tools::textInput('link[Attributes][CategoryUrl]', $url, 0, ['onchange' => "setHot();"], 'text', '520px') . '</td></tr>
		</table>
	</div>
	<table class="default defaultfont">
		<tr><td>' . g_l('modules_glossary', '[parameter_name]') . '</td></tr>
		<tr><td>' . we_html_tools::textInput('link[Attributes][CategoryCatParameter]', $catParameter, 0, ['onchange' => "setHot();"], 'text', '520px') . '</td></tr>
		<tr><td>' . g_l('modules_glossary', '[parameter]') . '</td></tr>
		<tr><td>' . we_html_tools::textInput('link[Attributes][CategoryParameter]', $parameter, 0, ['onchange' => "setHot();"], 'text', '520px') . '</td></tr>
	</table>');
	}

	// ---> Helper Methods

	private static function getLangField(string $name, string $value, string $title): string{
		$name = md5($name);
		$langs = array_keys(we_base_country::getWELangs());
		array_unshift($langs, '');
		$options = array_combine($langs, $langs);

		$input = we_html_tools::textInput($name, $value, 0, [], 'text');

		$select = we_html_tools::select($name . '_sel', $options, 1, "", false, ['onchange' => "setHot();this.form.elements['" . $name . "'].value=this.options[this.selectedIndex].value;this.selectedIndex=-1;"]);

		return we_html_tools::formElementTable($title, $input, $select);
	}

	private static function getHTMLLinkAttributes(we_glossary_glossary $glossary): we_html_multiIconBoxContent{
		return (new we_html_multiIconBoxContent())
				->add((new we_html_multiIconBoxContentLine('', we_html_tools::alertAttentionBox(g_l('modules_glossary', '[linkprops_desc]'), we_html_tools::TYPE_INFO, 520)))
					->space(we_html_multiIconBox::SPACE_ICON)
					->noline()
				)
				->add((new we_html_multiIconBoxContentLine(g_l('modules_glossary', '[attributes]'), we_html_tools::formElementTable(g_l('modules_glossary', '[title]'), we_html_tools::textInput('link[Title]', $glossary->Title, 0, [
								'onchange' => "setHot();"], 'text', '520px')) .
						we_html_tools::formElementTable(g_l('modules_glossary', '[anchor]'), we_html_tools::textInput('link[Attributes][anchor]', $glossary->getAttribute('anchor'), 0, [
								'onchange' => "setHot();", 'pattern' => '^#?[a-zA-Z]+[a-zA-Z0-9_:.-=]*$',
								'onblur' => "if(this.value&&!new RegExp(\'#?[a-z]+[a-z0-9_:.-=]*$\',\'i\').test(this.value)){WE().util.showMessage('" . g_l('linklistEdit', '[anchor_invalid]') . "', WE().consts.message.WE_MESSAGE_ERROR);this.focus();}"], 'text', '520px')) .
						we_html_tools::formElementTable(g_l('modules_glossary', '[link_attribute]'), we_html_tools::textInput('link[Attributes][attribute]', $glossary->getAttribute('attribute'), 0, [
								'onchange' => "setHot();"], 'text', '520px')) .
						we_html_tools::formElementTable(g_l('modules_glossary', '[target]'), we_html_tools::targetBox('link[Attributes][target]', 0, '', $glossary->getAttribute('target'), 'setHot();', 100)), we_html_multiIconBox::PROP_ATTRIB))
					->noline()
				)
				->add((new we_html_multiIconBoxContentLine(g_l('modules_glossary', '[language]'), self::getLangField('link[Attributes][lang]', $glossary->getAttribute('lang'), g_l('modules_glossary', '[link_language]')) .
						self::getLangField('link[Attributes][hreflang]', $glossary->getAttribute('hreflang'), g_l('modules_glossary', '[href_language]')), we_html_multiIconBox::PROP_LANG))
					->noline()
				)
				->add((new we_html_multiIconBoxContentLine(g_l('modules_glossary', '[keyboard]'), we_html_tools::formElementTable(g_l('modules_glossary', '[accesskey]'), we_html_tools::textInput('link[Attributes][accesskey]', $glossary->getAttribute('accesskey'), 0, [
								'onchange' => "setHot();"], 'text', '520px')) .
						we_html_tools::formElementTable(g_l('modules_glossary', '[tabindex]'), we_html_tools::textInput('link[Attributes][tabindex]', $glossary->getAttribute('tabindex'), 0, [
								'onchange' => "setHot();"], 'text', '520px')), we_html_multiIconBox::PROP_KEYBOARD))
					->noline())
				->add((new we_html_multiIconBoxContentLine(g_l('modules_glossary', '[relation]'), we_html_tools::getRelSelect(md5('link[Attributes][rel]'), $glossary->getAttribute('rel'), 'rel', 'setHot();'), we_html_multiIconBox::PROP_REVREL))
					->noline()
				)

		/* ['headline' => g_l('modules_glossary', '[popup]'),
		  'html' => $popup->getHTML(),
		  'noline' => true,
		  'icon' => we_html_multiIconBox::PROP_WINDOW,
		  ] */
		;
	}

}
