<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: be2cc06777cf $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2020-01-28 16:08 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */

/**
 * Filter model class for navigation tool
 *
 */
class we_navigation_customerFilter extends we_customer_abstractFilter{
	/** @var bool */
	private $useDocumentFilter = true;

	public function __construct(int $modelId = 0, int $mode = self::OFF, array $specificCustomers = [], array $blackList = [], array $whiteList = [], array $filter = [
		]){
		parent::__construct(NAVIGATION_TABLE, $modelId, $mode, $specificCustomers, $blackList, $whiteList, $filter);
	}

	public static function getFilterByDbHash(array $hash): self{
		return new self(intval($hash['modelId']), intval($hash['mode']), array_filter(explode(',', $hash['specificCustomers'])), array_filter(explode(',', $hash['whiteList'])), array_filter(explode(',', $hash['blackList'])), we_unserialize($hash['filter']));
	}

	public function getCustomerFilterFromRequest(int $id): void{
		parent::getCustomerFilterFromRequest($id);
		$this->useDocumentFilter = self::getUseDocumentFilterFromRequest();
	}

	function getUseDocumentFilter(): bool{
		return $this->useDocumentFilter;
	}

	public function setUseDocumentFilter(bool $useDocumentFilter): void{
		$this->useDocumentFilter = $useDocumentFilter;
	}

	static function getUseDocumentFilterFromRequest(): bool{
		return we_base_request::bool('wecf_useDocumentFilter');
	}

	/**
	 * initializes and returns filter object
	 *
	 * @param integer $id
	 * @return we_navigation_customerFilter
	 */
	public static function getFilterById(int $id): we_navigation_customerFilter{
		$hash = parent::getFilterHashFromDb($id, NAVIGATION_TABLE);
		return $hash ? self::getFilterByDbHash($hash) : new self($id);
	}

}
