<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: 2fcfd49ac317 $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2020-07-06 14:53 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
class we_navigation_frames extends we_modules_frame{
	/** @var array<string,array<string,mixed>> */
	private $jsVars = [];

	/** @var we_navigation_navigation */
	protected $Model;

	/** @var we_navigation_tree */
	protected $Tree;

	/** @var we_navigation_view */
	protected $View;

	const TAB_PROPERTIES = '1'; //make sure to keep 1
	const TAB_CONTENT = '2';
	const TAB_CUSTOMER = '3';
	const TAB_PREVIEW = 'preview';

	function __construct(){
		$this->module = we_modules_info::NAVIGATION;
		parent::__construct();
		$this->showTreeFooter = true;

		$this->Tree = new we_navigation_tree($this->jsCmd);
		$this->View = new we_navigation_view();
		$this->Model = &$this->View->Model;
	}

	public function getHTML(string $what = '', int $mode = 0, int $step = 0): string{
		switch($what){
			case 'preview' :
				return we_base_permission::hasPerm([we_base_perm::ADMINISTRATOR]) ? $this->getHTMLEditorBody() : '';
			case 'previewIframe' :
				return we_base_permission::hasPerm([we_base_perm::ADMINISTRATOR]) ? $this->getHTMLEditorPreviewIframe() : '';
			case 'fields' :
				return $this->getHTMLFieldSelector();
			case 'dyn_preview' :
				return $this->getHTMLDynPreview();
			case 'frameset':
				if(isset($_SESSION['weS']['module_sessions']['navigation'])){
					unset($_SESSION['weS']['module_sessions']['navigation']);
				}
				$tab = we_base_request::string('tab', '');
				$sid = we_base_request::int('sid', -1);
				return $this->getHTMLFrameset($this->Tree->getJSTreeCode(), ($tab !== '' ? '&tab=' . $tab : '' ) . ($sid !== -1 ? '&sid=' . $sid : ''));
			default:
				return parent::getHTML($what, $mode, $step);
		}
	}

	function getHTMLSearch(): string{
		$keyword = we_base_request::_(we_base_request::RAW, 'keyword', '');
		$arr = array_map('escape_sql_query', explode(' ', strToLower($keyword)));
		$DB_WE = DB_WE::$inst;

		$condition = [we_users_util::get_ws_paths(NAVIGATION_TABLE, true)];
		$condition[] = '(Path REGEXP "' . implode('|', $arr) . '" OR Text REGEXP "' . implode('|', $arr) . '" OR Display REGEXP "' . implode('|', $arr) . '")';

		$entries = $DB_WE->getAllFirstq('SELECT ID,Path FROM ' . NAVIGATION_TABLE . ' WHERE ' . implode(' AND ', $condition) . ' ORDER BY Path', false);

		$select = we_html_element::div(['style' => 'background-color:white;width:520px;height:180px;']);
		if($entries){
			$select = '<select name="search_results" size="20" style="width:520px;height:180px;" ondblclick="we_cmd(\'openSelectionToEdit\');">';
			foreach($entries as $id => $path){
				$select .= '<option value="' . $id . '">' . $path . '</option>';
			}
			$select .= '</select>';
		}

		$buttons = we_html_button::position_yes_no_cancel(
				we_html_button::create_button(we_html_button::OPEN_TO_EDIT, "javascript:we_cmd('openSelectionToEdit');"), '', we_html_button::create_button(we_html_button::CANCEL, "javascript:WE().util.dialogTop(window).doClose();")
		);

		$content = we_html_tools::formElementTable(g_l('modules_users', '[search_for]'), we_html_tools::textInput('keyword', $keyword, 0, [], 'text', '470px'), we_html_button::create_button(we_html_button::SEARCH, "javascript:document.we_form.submit();")
			) . we_html_element::div(['style' => 'height:20px;']) .
			we_html_tools::formElementTable(g_l('modules_users', '[search_result]'), $select);

		return $this->getHTMLDocument(we_html_element::body(['class' => 'weEditorBody'], we_html_element::form(['name' => we_base_constants::WE_FORM,
						'method' => 'post'], we_html_element::hiddens([
							'mod' => 'navigation',
							'pnt' => 'search']) .
						we_html_tools::dialogLayout($content, g_l('modules_users', '[search]'), $buttons))
				), we_html_element::jsScript(WE_JS_MODULES_DIR . 'navigation/navigationSearch.js'));
	}

	protected function getHTMLTreeFooter(): string{
		return we_html_element::div(['id' => 'infoField', 'style' => 'display:none;']) . $this->getHTMLSearchTreeFooter();
	}

	protected function getHTMLCmd(): string{
		if(($pid = we_base_request::int('pid', -1)) < 0){
			return $this->getHTMLDocument(we_html_element::body(), $this->jsCmd->getCmds());
		}

		$openFolders = we_base_request::intList('open', []);
		$offset = we_base_request::int("offset", 0);
		$this->jsCmd->addCmd('loadTree', ['clear' => !$pid, 'items' => we_navigation_tree::getItems($pid, $offset, $this->Tree->default_segment, false, $openFolders)]);

		return $this->getHTMLDocument(we_html_element::body([], we_html_element::form(['name' => we_base_constants::WE_FORM], we_html_element::hiddens([
							'pnt' => 'cmd',
							'cmd' => 'no_cmd'])
					)
		));
	}

	/**
	 * Frame for tabs
	 *
	 * @return string
	 */
	protected function getHTMLEditorHeader(): string{
		if(we_base_request::bool('home')){
			return parent::getHTMLEditorHeader();
		}

		$we_tabs = new we_gui_tabs();

		$we_tabs->addTab('', we_gui_tabs::WE_ICON_PROPERTIES, false, self::TAB_PROPERTIES, ['id' => 'tab_' . self::TAB_PROPERTIES, 'title' => g_l('navigation', '[property]')]);
		if($this->Model->IsFolder && we_base_permission::hasPerm([we_base_perm::NAVIGATION_EDIT_DYNAMIC])){
			$we_tabs->addTab('', we_gui_tabs::WE_ICON_CONTENT, false, self::TAB_CONTENT, ['id' => 'tab_' . self::TAB_CONTENT, 'title' => g_l('navigation', '[content]')]);
		}

		if(we_modules_info::isActive(we_modules_info::CUSTOMER) && we_base_permission::hasPerm([we_base_perm::CUSTOMER_CAN_EDIT_CUSTOMERFILTER])){
			$we_tabs->addTab('', we_gui_tabs::WE_ICON_CUSTOMER_FILTER, false, self::TAB_CUSTOMER, ['id' => 'tab_' . self::TAB_CUSTOMER, 'title' => g_l('navigation', '[customers]')]);
		}

		if($this->Model->IsFolder && we_base_permission::hasPerm([we_base_perm::ADMINISTRATOR])){
			$we_tabs->addTab('', we_gui_tabs::WE_ICON_PREVIEW, false, "'" . self::TAB_PREVIEW . "'", ['id' => 'tab_' . self::TAB_PREVIEW, 'title' => g_l('navigation', '[preview]')]);
		}

		return $this->getHTMLDocument(we_html_element::body([
					'id' => 'eHeaderBody',
					'onload' => "initNavHeader();",
					'onresize' => "weTabs.setFrameSize()"
					], we_html_element::div(['id' => 'main'], we_html_element::div(['id' => 'headrow'], '&nbsp;' .
							we_html_element::b(g_l('navigation', ($this->Model->IsFolder ? '[group]' : '[entry]'))) . ':&nbsp;' .
							str_replace('&amp;', '&', $this->Model->Text) . ' (ID: ' . we_html_element::span(['id' => 'h_id'], (string) ($this->Model->ID)) . ') ' .
							we_html_element::div(['id' => 'mark'], '<i class="fa fa-asterisk modified"></i>')) .
						$we_tabs->getHTML())
				), we_html_element::cssLink(CSS_DIR . 'we_tab.css') .
				we_html_element::jsScript(JS_DIR . 'initTabs.js') .
				we_html_element::jsScript(WE_JS_MODULES_DIR . 'navigation/navigation_view_prop.js') .
				$this->jsCmd->getCmds()
		);
	}

	protected function getHTMLEditorBody(): string{
		if(we_base_request::bool('home')){
			return $this->View->getHomeScreen();
		}

		return $this->getHTMLDocument(we_html_element::body(['class' => 'weEditorBody', 'onload' => 'loaded=1;'], we_html_element::form(['name' => we_base_constants::WE_FORM,
						'onsubmit' => 'return false'], $this->getHTMLProperties())), we_html_element::jsScript(JS_DIR . 'multiIconBox.js') . $this->View->getJSProperty($this->jsVars) .
				$this->jsCmd->getCmds());
	}

	private function getHTMLGeneral(): we_html_multiIconBoxContent{
		$table = new we_html_table(['class' => 'default', 'style' => 'margin-top: 5px;'], 1, 2);

		$parentid = (!empty($this->Model->Text) && !empty($this->Model->ID) ?
			fI('SELECT ParentID FROM ' . NAVIGATION_TABLE . ' WHERE ID=' . intval($this->Model->ID)) :
			(we_base_request::string('presetFolder') ?
			$this->Model->ParentID :
			(($wq = we_navigation_navigation::getWSQuery()) ?
			fI('SELECT ID FROM ' . NAVIGATION_TABLE . ' WHERE IsFolder=1 ' . $wq . ' ORDER BY Path LIMIT 1') :
			0)
			));

		if($this->Model->ID){
			$num = fI('SELECT COUNT(ID) FROM ' . NAVIGATION_TABLE . ' WHERE ParentID=' . intval($parentid));
			$table
				->setCol(0, 0, we_html_element::hidden('Ordn', ($this->Model->Ordn)) .
					we_html_tools::select('Position', $this->View->getEditNaviPosition(), 1, $this->Model->Ordn, false, ['onchange' => "we_cmd('move_abs',this.value);"])
				)
				->setCol(0, 1, we_html_element::span(['style' => 'margin-left: 15px'], we_html_button::create_button(we_html_button::DIRUP, 'javascript:we_cmd("move_up");', '', '', !($this->Model->Ordn > 0), false) .
						we_html_button::create_button(we_html_button::DIRDOWN, 'javascript:we_cmd("move_down");', '', '', !($this->Model->Ordn < ($num - 1)), false)));
		} else {
			$table->setCol(0, 0, we_html_element::hiddens(['name' => 'Ordn', 'value' => '-1']));
		}

		return (new we_html_multiIconBoxContent())
				->add((new we_html_multiIconBoxContentLine(g_l('navigation', '[general]'), we_html_element::hidden('newone', ($this->Model->ID == 0 ? 1 : 0)) .
						we_html_tools::formElementTable(g_l('navigation', '[name]'), we_html_tools::textInput('Text', strtr($this->Model->Text, array_flip(get_html_translation_table(HTML_SPECIALCHARS))), 0, [
								'style' => "width: 520px;", 'onchange' => "we_cmd('setHot');"])) .
						we_html_tools::formElementTable(g_l('navigation', '[display]'), we_html_tools::textInput('Display', $this->Model->Display, 0, ['style' => "width: 520px;",
								'onchange' => "we_cmd('setHot');"])) .
						$this->getHTMLChooser(g_l('navigation', '[group]'), NAVIGATION_TABLE, 0, 'ParentID', $parentid, 'ParentPath', 'setHot', [we_base_ContentTypes::FOLDER_COMMON], ($this->Model->IsFolder == 0 && $this->Model->Depended == 1)), we_html_multiIconBox::PROP_PATH))
					->noline())
				->add((new we_html_multiIconBoxContentLine(g_l('navigation', '[order]'), $table->getHtml(), we_html_multiIconBox::PROP_SORT))
					->noline()
		);
	}

	private function getHTMLImage(): we_html_multiIconBoxContent{
		return (new we_html_multiIconBoxContent())
				->add((new we_html_multiIconBoxContentLine(g_l('navigation', '[icon]'), $this->getHTMLChooser('', FILE_TABLE, 0, 'IconID', $this->Model->IconID, 'IconPath', 'setHot', [
							we_base_ContentTypes::IMAGE], false, true, ['folder',
							we_base_ContentTypes::IMAGE]), we_html_multiIconBox::PROP_IMAGE))
					->noline()
				)
				->add((new we_html_multiIconBoxContentLine(g_l('navigation', '[icon_properties]'), $this->getHTMLImageAttributes()))
					->fold()
					->space(we_html_multiIconBox::SPACE_ICON)
					->rightHead()
					->noline()
		);
	}

	private function getHTMLPropertiesItem(): we_html_multiIconBoxContent{
		$seltypeStat = [
			we_navigation_navigation::STYPE_DOCLINK => g_l('navigation', '[docLink]'),
			we_navigation_navigation::STYPE_URLLINK => g_l('navigation', '[urlLink]')
		];

		$seltypeDyn = [we_navigation_navigation::DYN_DOCTYPE => g_l('navigation', '[documents]')];

		if(we_modules_info::isActive(we_modules_info::OBJECT)){
			$seltypeDyn[we_navigation_navigation::DYN_CLASS] = g_l('navigation', '[objects]');
			$seltypeStat[we_navigation_navigation::STYPE_OBJLINK] = g_l('navigation', '[objLink]');
		}
		if(we_modules_info::isActive(we_modules_info::CATEGORY)){
			$seltypeDyn[we_navigation_navigation::DYN_CATEGORY] = g_l('navigation', '[categories]');
			$seltypeStat[we_navigation_navigation::STYPE_CATLINK] = g_l('navigation', '[catLink]');
		}

		switch($this->Model->Selection){
			case we_navigation_navigation::SELECTION_DYNAMIC:
				$seltype = $seltypeDyn;
				break;
			default:
				$seltype = $seltypeStat;
		}

		$selection_block = we_html_tools::select('SelectionTypeDyn', $seltypeDyn, 1, 0, false, ['style' => 'display:none']) .
			we_html_tools::select('SelectionTypeStat', $seltypeStat, 1, 0, false, ['style' => 'display:none']) .
			($this->Model->Depended == 1 ? $this->getHTMLDependedProfile() :
			we_html_element::hiddens([
				'CategoriesControl' => we_base_request::string('CategoriesControl'),
				'SortControl' => we_base_request::string('SortControl'),
			]) .
			(we_base_permission::hasPerm([we_base_perm::NAVIGATION_EDIT_DYNAMIC]) ?
			we_html_tools::select('Selection', [
				we_navigation_navigation::SELECTION_DYNAMIC => g_l('navigation', '[dyn_selection]'),
				we_navigation_navigation::SELECTION_STATIC => g_l('navigation', '[stat_selection]')
				], 1, $this->Model->Selection, false, ['onchange' => "we_cmd('setSelectionMode', this);"], true, 520) . '<br/>' :
			we_html_element::hiddens([
				'Selection' => $this->Model->Selection,
			]) ) .
			we_html_tools::select('SelectionType', $seltype, 1, $this->Model->SelectionType, false, ['onchange' => "we_cmd('setSelectionType', this.value);",
				'style' => 'width: 520px; margin-top: 5px;'], true, 520) .
			we_html_element::div(['id' => 'dynamicDiv', 'style' => ($this->Model->Selection === we_navigation_navigation::SELECTION_DYNAMIC ? 'display: block;' : 'display: none;')], $this->getHTMLDynamic()) .
			we_html_element::div(['id' => 'staticDiv', 'style' => ($this->Model->Selection === we_navigation_navigation::SELECTION_STATIC ? 'display: block;' : 'display: none;')], we_html_element::div([
					'id' => 'staticSelect', 'style' => ($this->Model->SelectionType !== we_navigation_navigation::STYPE_URLLINK ? 'display: block;' : 'display: none;')], $this->getHTMLStatic()) .
				we_html_element::div(['id' => 'staticUrl', 'style' => (($this->Model->SelectionType === we_navigation_navigation::STYPE_CATLINK || $this->Model->SelectionType === we_navigation_navigation::STYPE_URLLINK) ? 'display: block;' : 'display: none;') . ';margin-top:5px;'], $this->getHTMLLink()) .
				we_html_element::div(['style' => 'margin-top:5px;'], we_html_tools::formElementTable(g_l('navigation', '[parameter]'), we_html_tools::textInput('Parameter', $this->Model->Parameter, 0, [
							'onchange' => "we_cmd('setHot');"], 'text', '520px')))
			)
			);

		return (new we_html_multiIconBoxContent())
				->add((new we_html_multiIconBoxContentLine(g_l('navigation', '[selection]'), $selection_block, we_html_multiIconBox::PROP_LINK))
					->noline());
	}

	private function getHTMLPropertiesGroup(): we_html_multiIconBoxContent{
		$weSuggest = we_gui_suggest::getInstance();

		$cmd1 = "document.we_form.elements.LinkID.value";

		$cmd_obj = we_modules_info::isActive(we_modules_info::OBJECT) ? "javascript:we_cmd('we_selector_document',{id:document.we_form.elements.LinkID.value,table:WE().consts.tables.OBJECT_FILES_TABLE,targetID:'LinkID',targetName:'LinkPath',jsCmd:'populateFolderWs',rootDirID:0,filter:WE().consts.contentTypes.OBJECT_FILE,open_doc:false})" : '';

		$button_doc = we_html_button::create_button(we_html_button::SELECT, "javascript:we_cmd('we_selector_document',{id:document.we_form.elements.LinkID.value,table:WE().consts.tables.FILE_TABLE,targetID:'LinkID',targetName:'LinkPath',rootDirID:0,open_doc:false})") .
			we_html_button::create_button(we_html_button::OPEN_TO_EDIT, 'javascript:WE().layout.openToEdit("' . FILE_TABLE . '",' . $cmd1 . ',"")', '', '', false);
		$button_obj = we_html_button::create_button(we_html_button::SELECT, $cmd_obj) .
			(we_modules_info::isActive(we_modules_info::OBJECT) ? we_html_button::create_button(we_html_button::OPEN_TO_EDIT, 'javascript:WE().layout.openToEdit("' . OBJECT_FILES_TABLE . '",' . $cmd1 . ',"")', '', '', false) : '');

		$buttons = we_html_element::div(['id' => "docFolderLink", 'style' => 'display: ' . ($this->Model->SelectionType === we_navigation_navigation::STYPE_DOCLINK ? 'inline' : 'none')], $button_doc) .
			we_html_element::div(['id' => "objFolderLink", 'style' => 'display: ' . ($this->Model->SelectionType === we_navigation_navigation::STYPE_OBJLINK ? 'inline' : 'none')], $button_obj);
		$path = ($this->Model->LinkID == 0 ?
			'' :
			id2path($this->Model->LinkID, ($this->Model->SelectionType === we_navigation_navigation::STYPE_DOCLINK ? FILE_TABLE : (we_modules_info::isActive(we_modules_info::OBJECT) && $this->Model->SelectionType === we_navigation_navigation::STYPE_OBJLINK ? OBJECT_FILES_TABLE : FILE_TABLE))));

		$seltype = [
			we_navigation_navigation::STYPE_DOCLINK => g_l('navigation', '[docLink]'),
			we_navigation_navigation::STYPE_URLLINK => g_l('navigation', '[urlLink]')
		];
		if(we_modules_info::isActive(we_modules_info::OBJECT)){
			$seltype[we_navigation_navigation::STYPE_OBJLINK] = g_l('navigation', '[objLink]');
		}

		$weSuggest->setAcId('LinkPath')
			->setContentType(
				$this->Model->SelectionType === we_navigation_navigation::STYPE_DOCLINK ?
					array_merge(we_base_ContentTypes::LINK_ABLE, we_base_ContentTypes::FOLDER_TYPES) :
					(we_modules_info::isActive(we_modules_info::OBJECT) && $this->Model->SelectionType === we_navigation_navigation::STYPE_OBJLINK ?
						[we_base_ContentTypes::FOLDER_OBJECTFILE, we_base_ContentTypes::OBJECT_FILE] :
						array_merge(we_base_ContentTypes::LINK_ABLE, we_base_ContentTypes::FOLDER_TYPES))
			)
			->setInput('LinkPath', $path, [], false, true)
			->setMaxResults(50)
			->setResult('LinkID', (string) ($this->Model->LinkID))
			->setSelector(we_gui_suggest::DocSelector)
			->setTable($this->Model->SelectionType === we_navigation_navigation::STYPE_DOCLINK ? FILE_TABLE : ($this->Model->SelectionType === we_navigation_navigation::STYPE_OBJLINK ? OBJECT_FILES_TABLE : FILE_TABLE))
			->setWidth(330)
			->setSelectButton($buttons)
			->setTrashButton(we_html_button::create_button(we_html_button::TRASH, 'javascript:document.we_form.elements.LinkID.value=0;document.we_form.elements.LinkPath.value="";', ''));

		$weAcSelector = $weSuggest->getHTML();

		$selection = we_html_tools::select('SelectionType', $seltype, 1, $this->Model->SelectionType, false, ['onchange' => "onFolderSelectionChangeJS(this.value);setFolderSelection(this.value); we_cmd('setHot');",
				'style' => 'width: 520px; margin-top: 5px;'], true, 520) .
			we_html_element::div(['id' => "folderSelectionDiv", 'style' => 'display: ' . ($this->Model->SelectionType != we_navigation_navigation::STYPE_URLLINK ? 'block' : 'none') . ';margin-top: 5px;'], $weAcSelector) .
			we_html_element::div(['id' => "folderUrlDiv", 'style' => 'display: ' . ($this->Model->SelectionType === we_navigation_navigation::STYPE_URLLINK ? 'block' : 'none') . ';margin-top: 5px;'], we_html_tools::textInput('Url', $this->Model->Url, 0, [
					'onchange' => "we_cmd('setHot');"], 'text', '520px')) .
			$this->getHTMLWorkspace('object', 0, 'WorkspaceID') . we_html_tools::formElementTable(g_l('navigation', '[parameter]'), we_html_tools::textInput('Parameter', $this->Model->Parameter, 0, [
					'onchange' => "we_cmd('setHot');"], 'text', '520px') . '<br/>' .
				we_html_forms::checkboxWithHidden($this->Model->CurrentOnUrlPar, 'CurrentOnUrlPar', g_l('navigation', '[current_on_urlpar]'), false, "defaultfont", "we_cmd('setHot');"));

		return (new we_html_multiIconBoxContent())
				->add((new we_html_multiIconBoxContentLine(g_l('navigation', '[selection]'), $selection, we_html_multiIconBox::PROP_LINK))
					->noline()
		);
	}

	private function getHTMLPropertiesGroupMore(): we_html_multiIconBoxContent{
		// COPY FOLDER
		$disabled = ($this->Model->isnew ? ' ' . g_l('weClass', '[availableAfterSave]') : '');

		return (new we_html_multiIconBoxContent())
				->add((new we_html_multiIconBoxContentLine(g_l('weClass', '[copyFolder]'), ($disabled ? we_html_tools::alertAttentionBox($disabled, we_html_tools::TYPE_INFO, 400, true, 0) : we_html_button::create_button(we_html_button::SELECT, "javascript:we_cmd('we_navigation_dirSelector',{id:document.we_form.CopyFolderID.value,targetID:'CopyFolderID',targetName:'CopyFolderPath',jsCmd:'copyNaviFolder'})")) .
						we_html_element::hiddens([
							'CopyFolderID' => '',
							'CopyFolderPath' => ''
						]), we_html_multiIconBox::PROP_COPY))
					->noline()
		);
	}

	private function getHTMLDependedProfile(): string{
		/* switch($this->Model->Selection){
		  case we_navigation_navigation::SELECTION_DYNAMIC:
		  $seltype = [we_navigation_navigation::DYN_DOCTYPE => g_l('navigation', '[documents]')];
		  if(we_modules_info::isActive(we_modules_info::OBJECT)){
		  $seltype[we_navigation_navigation::DYN_CLASS] = g_l('navigation', '[objects]');
		  }
		  break;
		  default:
		  $seltype = [we_navigation_navigation::STYPE_DOCLINK => g_l('navigation', '[docLink]')];
		  if(we_modules_info::isActive(we_modules_info::OBJECT)){
		  $seltype[we_navigation_navigation::STYPE_OBJLINK] = g_l('navigation', '[objLink]');
		  }
		  } */

		$table = (new we_html_table(['width' => '520', 'class' => 'default defaultfont'], 5, 2))
			->setCol(0, 0, g_l('navigation', '[stat_selection]'))
			->setCol(1, 0, g_l('navigation', ($this->Model->SelectionType === we_navigation_navigation::STYPE_CATLINK ? '[catLink]' : ($this->Model->SelectionType === we_navigation_navigation::STYPE_DOCLINK ? '[docLink]' : '[objLink]'))) . ':')
			->setCol(1, 1, id2path($this->Model->LinkID, ($this->Model->SelectionType === we_navigation_navigation::STYPE_CATLINK ? CATEGORY_TABLE : ($this->Model->SelectionType === we_navigation_navigation::STYPE_DOCLINK ? FILE_TABLE : OBJECT_FILES_TABLE))));

		if(!empty($this->Model->Url) && $this->Model->Url != we_base_link::EMPTY_EXT){
			$table
				->setCol(2, 0, g_l('navigation', '[linkSelection]') . ':')
				->setCol(2, 1, $this->Model->Url);
		} elseif(!empty($this->Model->UrlID) && $this->Model->UrlID){
			$table
				->setCol(2, 0, g_l('navigation', '[linkSelection]') . ':')
				->setCol(2, 1, id2path($this->Model->UrlID));
		}

		if($this->Model->SelectionType === we_navigation_navigation::STYPE_CATLINK){
			$table
				->setCol(3, 0, g_l('navigation', '[catParameter]') . ':')
				->setCol(3, 1, $this->Model->CatParameter);
		}

		if(!empty($this->Model->Parameter)){
			$table
				->setCol(4, 0, g_l('navigation', '[parameter]') . ':')
				->setCol(4, 1, $this->Model->Parameter);
		}

		return $table->getHtml();
	}

	private function getHTMLDynamic(): string{
		$dtq = we_doctype_docTypes::getDoctypeQuery();
		$docTypes = [g_l('navigation', '[no_entry]')] + $this->db->getAllFirstq('SELECT dt.ID,dt.Text FROM ' . DOC_TYPES_TABLE . ' dt LEFT JOIN ' . FILE_TABLE . ' dtf ON dt.ParentFileID=dtf.ID ' . $dtq['join'] . ' WHERE ' . $dtq['where'], false);

		$classID2Name = $classDirs = [];
		$this->jsVars = [
			'classDirs' => [],
			'classPaths' => [],
			'hasClassSubDirs' => []
		];
		$allowedClasses = we_users_util::getAllowedClasses();

		$firstClass = 0;
		if(we_modules_info::isActive(we_modules_info::OBJECT) && $allowedClasses){
			$all = $this->db->getAllq('SELECT
	o.ID,
	o.Filename,
	o.Path,
	of.ID AS classDirID
FROM ' .
				OBJECT_TABLE . ' o JOIN ' .
				OBJECT_FILES_TABLE . ' `of` ON (o.ID=of.TableID)
WHERE
	of.IsClassFolder=1 AND
	o.ID IN(' . implode(',', $allowedClasses) . ')
GROUP BY o.ID');
			foreach($all as $rec){
				$classID = (int) $rec['ID'];
				$classDirID = (int) $rec['classDirID'];
				if(!$firstClass){
					$firstClass = $classID;
				}
				$classID2Name[$classID] = $rec['Filename'];
				$classDirs[] = $classDirID;

				$this->jsVars['hasClassSubDirs'][$classID] = false;
				$this->jsVars['classDirs'][$classID] = $classDirID;
				$this->jsVars['classPaths'][$classID] = $rec['Path'];
			}
			asort($classID2Name);
			if($classDirs){
				$classes = $this->db->getAllq('SELECT TableID FROM ' . OBJECT_FILES_TABLE . ' WHERE TableID IN (' . implode(',', $allowedClasses) . ') AND IsFolder=1 GROUP BY TableID', true);
				foreach($classes as $id){
					$this->jsVars['hasClassSubDirs'][(int) $id] = true;
				}
			}
		}

		/* $wsid = ($this->Model->SelectionType === we_navigation_navigation::STYPE_OBJLINK && $this->Model->LinkID ?
		  we_navigation_dynList::getWorkspacesForObject($this->Model->LinkID) :
		  []);
		 */

		$sortVal = $this->Model->Sort[0]['field'] ?? '';
		$sortOrder = $this->Model->Sort[0]['order'] ?? 'ASC';

		$sortSelect = we_html_tools::select('SortOrder', ['ASC' => g_l('navigation', '[ascending]'), 'DESC' => g_l('navigation', '[descending]')], 1, $sortOrder, false, [
				'onchange' => "we_cmd('setHot');"], true, 120);

		return we_html_element::div(['id' => 'doctype', 'style' => ($this->Model->Selection === we_navigation_navigation::SELECTION_DYNAMIC && $this->Model->DynamicSelection === we_navigation_navigation::DYN_DOCTYPE ? 'display: block' : 'display: none') . ';width: 520px;margin-top:5px;'], we_html_tools::formElementTable(g_l('navigation', '[doctype]'), we_html_tools::select('DocTypeID', $docTypes, 1, $this->Model->DocTypeID, false, [
						'onchange' => "clearFields(); we_cmd('setHot');"], true, 520))) .
			we_html_element::div(['id' => 'classname', 'style' => ($this->Model->Selection === we_navigation_navigation::SELECTION_DYNAMIC && $this->Model->DynamicSelection === we_navigation_navigation::DYN_CLASS ? 'display: block' : 'display: none') . '; width: 520px;margin-top:5px;'], (we_modules_info::isActive(we_modules_info::OBJECT) ? we_html_tools::formElementTable(g_l('navigation', '[class]'), we_html_tools::select('ClassID', $classID2Name, 1, $this->Model->ClassID, false, [
						'onchange' => "clearFields();onSelectionClassChangeJS(this.value);"], true, 520)) . $this->getHTMLWorkspace('class', $firstClass) : '')) . we_html_element::div([
				'id' => 'fieldChooser', 'style' => ($this->Model->DynamicSelection != we_navigation_navigation::DYN_CATEGORY ? 'display: block' : 'display: none') . '; width: 520px;margin-top: 5px;'], $this->getHTMLFieldChooser(g_l('navigation', '[title_field]'), 'TitleField', $this->Model->TitleField, 'putTitleField', $this->Model->DynamicSelection)) .
			$this->getHTMLDirSelector() .
			(we_modules_info::isActive(we_modules_info::CATEGORY) ?
			we_html_element::div(['id' => 'catSort', 'style' => ($this->Model->DynamicSelection != we_navigation_navigation::DYN_CATEGORY ? 'display: block' : 'display: none') . '; width: 520px;'], $this->getHTMLCategory()) : '') .
			$this->getHTMLFieldChooser(g_l('navigation', '[sort]'), 'SortField', $sortVal, 'putSortField', $this->Model->DynamicSelection, $sortSelect, 120) .
			we_html_element::div(['id' => 'dynUrl', 'style' => ($this->Model->DynamicSelection == we_navigation_navigation::DYN_CATEGORY ? 'display: block' : 'display: none') . '; width: 520px;'], $this->getHTMLLink('dynamic_')) .
			we_html_element::div(['style' => 'width: 520px;margin-top: 5px;'], we_html_tools::formElementTable(g_l('navigation', '[parameter]'), we_html_tools::textInput('dynamic_Parameter', $this->Model->Parameter, 0, [
						'onchange' => "we_cmd('setHot');"], 'text', '520px'))) .
			$this->getHTMLCount() .
			we_html_element::span(['style' => 'margin-top:20px;'], we_html_button::create_button(we_html_button::PREVIEW, 'javascript:top.content.we_cmd("dyn_preview");') . ($this->Model->hasDynChilds() ? we_html_button::create_button(we_html_button::DELETE_ALL, 'javascript:top.content.we_cmd("depopulate");') : ''));
	}

	private function getHTMLStatic(bool $disabled = false): string{
		/* $seltype = [we_navigation_navigation::STYPE_DOCLINK => g_l('navigation', '[docLink]')];
		  if(we_modules_info::isActive(we_modules_info::OBJECT)){
		  $seltype[we_navigation_navigation::STYPE_OBJLINK] = g_l('navigation', '[objLink]');
		  } */

		$cmd_doc = "javascript:we_cmd('we_selector_document',{id:document.we_form.elements.LinkID.value,table:WE().consts.tables.FILE_TABLE,targetID:'LinkID',targetName:'LinkPath',jsCmd:'enable_open_navigation_doc',rootDirID:0,open_doc:" . (we_base_permission::hasPerm([
				we_base_perm::WORK_WORKSPACE_CAN_SELECT_OTHER_FILES]) ? 0 : 1) . "})";

		$cmd_obj = we_modules_info::isActive(we_modules_info::OBJECT) ? "javascript:we_cmd('we_selector_document',{id:document.we_form.elements.LinkID.value,table:WE().consts.tables.OBJECT_FILES_TABLE,targetID:'LinkID',targetName:'LinkPath',jsCmd:'populateWorkspaces',rootDirID:0,open_doc:" . (we_base_permission::hasPerm([
				we_base_perm::OBJECTS_CAN_SELECT_OTHER_USERS]) ? 0 : 1) . "})" : '';
		$cmd_cat = "javascript:we_cmd('we_selector_document',{id:document.we_form.elements.LinkID.value,table:WE().consts.tables.CATEGORY_TABLE,targetID:'LinkID',targetName:'LinkPath',jsCmd:'populateText',rootDirID:0})";

		$button_doc = we_html_button::create_button(we_html_button::SELECT, $cmd_doc, '', '', $disabled) .
			we_html_button::create_button(we_html_button::OPEN_TO_EDIT, 'javascript:WE().layout.openToEdit(WE().consts.tables.FILE_TABLE,document.we_form.elements.LinkID.value,"")', '', '', ($this->Model->LinkID ? false : true), false, '_navigation_doc');

		$button_obj = we_html_button::create_button(we_html_button::SELECT, $cmd_obj, '', '', $disabled) .
			(we_modules_info::isActive(we_modules_info::OBJECT) ? we_html_button::create_button(we_html_button::OPEN_TO_EDIT, 'javascript:WE().layout.openToEdit(WE().consts.tables.OBJECT_FILES_TABLE,document.we_form.elements.LinkID.value,"")', '', '', $disabled, false, '_navigation_obj') : '');

		$buttons = we_html_element::div(['id' => "docLink", 'style' => 'display: ' . ($this->Model->SelectionType === we_navigation_navigation::STYPE_DOCLINK ? 'inline' : 'none')], $button_doc) .
			we_html_element::div(['id' => "objLink", 'style' => 'display: ' . ($this->Model->SelectionType === we_navigation_navigation::STYPE_OBJLINK ? 'inline' : 'none')], $button_obj) .
			(we_modules_info::isActive(we_modules_info::CATEGORY) ?
			we_html_element::div(['id' => "catLink", 'style' => 'display: ' . ($this->Model->SelectionType === we_navigation_navigation::STYPE_CATLINK ? 'inline' : 'none')], we_html_button::create_button(we_html_button::SELECT, $cmd_cat, '', '', $disabled)) :
			'');

		switch($this->Model->SelectionType){
			default:
			case we_navigation_navigation::STYPE_DOCLINK:
				$path = ($this->Model->LinkID ? id2path($this->Model->LinkID) : '');
				break;
			case we_navigation_navigation::STYPE_CATLINK:
				$path = (we_modules_info::isActive(we_modules_info::CATEGORY) && $this->Model->LinkID ? id2path($this->Model->LinkID, CATEGORY_TABLE) : '');
				break;
			case we_navigation_navigation::STYPE_OBJLINK:
				$path = (we_modules_info::isActive(we_modules_info::OBJECT) && $this->Model->LinkID ? id2path($this->Model->LinkID, OBJECT_FILES_TABLE) : '');
				break;
		}

		$weSuggest = we_gui_suggest::getInstance();
		$weSuggest->setAcId('LinkPath', '')
			->setContentType(
				$this->Model->SelectionType === we_navigation_navigation::STYPE_DOCLINK ?
					array_merge(we_base_ContentTypes::LINK_ABLE, [we_base_ContentTypes::FOLDER_DOCUMENT]) :
					($this->Model->SelectionType === we_navigation_navigation::STYPE_OBJLINK ? [we_base_ContentTypes::OBJECT_FILE] : []))
			->setInput('LinkPath', $path, [], false, true)
			->setMaxResults(50)
			->setResult('LinkID', (string) $this->Model->LinkID)
			->setSelector(we_gui_suggest::DocSelector)
			->setTable($this->Model->SelectionType === we_navigation_navigation::STYPE_DOCLINK ? FILE_TABLE : ($this->Model->SelectionType === we_navigation_navigation::STYPE_OBJLINK ? OBJECT_FILES_TABLE : CATEGORY_TABLE))
			->setWidth(370)
			->setSelectButton($buttons);

		return we_html_element::div(['style' => "margin-top:5px"], $weSuggest->getHTML()) . $this->getHTMLWorkspace();
	}

	private function getHTMLTab2(): we_html_multiIconBoxContent{
		$seltype = [we_navigation_navigation::DYN_DOCTYPE => g_l('navigation', '[documents]')];

		if(we_modules_info::isActive(we_modules_info::OBJECT)){
			$seltype[we_navigation_navigation::DYN_CLASS] = g_l('navigation', '[objects]');
		}
		if(we_modules_info::isActive(we_modules_info::CATEGORY)){
			$seltype[we_navigation_navigation::DYN_CATEGORY] = g_l('navigation', '[categories]');
		}
		$selection_block = we_html_element::hiddens([
				'CategoriesControl' => we_base_request::string('CategoriesControl'),
				'SortControl' => we_base_request::string('SortControl'),
			]) .
			we_html_tools::select('Selection', [
				we_navigation_navigation::SELECTION_STATIC => g_l('navigation', '[no_dyn_content]'),
				we_navigation_navigation::SELECTION_DYNAMIC => g_l('navigation', '[dyn_content]')
				], 1, $this->Model->Selection, false, ['style' => 'width: 520px;', 'onchange' => "we_cmd('setSelectionMode', this, true);"], true, 520) .
			we_html_element::div(['id' => we_navigation_navigation::SELECTION_DYNAMIC, 'style' => ($this->Model->Selection === we_navigation_navigation::SELECTION_DYNAMIC ? 'display: block;' : 'display: none;') . ';margin-top:5px'], we_html_tools::select('DynamicSelection', $seltype, 1, $this->Model->DynamicSelection, false, [
					'onchange' => "we_cmd('setSelectionType', this.value);"], true, 520) .
				$this->getHTMLDynamic());

		return (new we_html_multiIconBoxContent())
				->add((new we_html_multiIconBoxContentLine(g_l('navigation', '[content]'), $selection_block))
					->space(we_html_multiIconBox::SPACE_MED)
		);
	}

	private function getHTMLEditorPreview(): string{
		// build the page

		return $this->View->getCommonHiddens(['pnt' => 'preview', 'tabnr' => 'preview']) . we_html_tools::dialogLayout('<table class="defaultfont" class="default">
<tr>
	<td><iframe name="preview" style="background: white; border: 1px solid black; width: 640px; height: 150px" src="' . WEBEDITION_DIR . 'we_cmd.php?we_cmd[0]=we_showMod&mod=navigation&pnt=previewIframe"></iframe></td>
</tr>
<tr>
	<td height="30"><label for="previewCode">' . g_l('navigation', '[preview_code]') . '</label><br/></td>
<tr>
	<td>' . we_html_element::textArea(
					[
						'name' => 'previewCode',
						'id' => 'previewCode',
						'style' => 'width: 640px; height: 200px;',
						'class' => 'defaultfont'
					], $this->Model->previewCode) . '</td>
</tr>
<tr>
	<td height="10"></td>
<tr>
<tr>
	<td style="text-align:right">' .
				we_html_button::create_button(we_html_button::REFRESH, 'javascript: showPreview();') .
				we_html_button::create_button(we_html_button::RESET, 'javascript:document.getElementById("previewCode").value="' . str_replace(["\r\n", "\n"], '\n', addslashes(
							strtr(we_navigation_navigation::defaultPreviewCode, ['@###PARENTID###@' => $this->Model->ID]))) . '"; showPreview();')//@phan-suppress-current-line PhanParamSuspiciousOrder
				. '</td>
</tr>
</table>', g_l('navigation', '[preview]'));
	}

	private function getHTMLEditorPreviewIframe(): string{
		$templateCode = $this->Model->previewCode;

		// initialize a document (only for caching needed)
		//we_base_WE::$we_doc = new we_webEditionDocument();

		$tp = new we_weTag_parser($templateCode);
		try{
			$tp->parseTags($templateCode);
		} catch (we_exception_parser $e){
			return htmlspecialchars($e->getMessage());
		}
		ob_start();
		if(($file = we_base_file::saveTemp($templateCode))){
			include($file);
		}
		return ob_get_clean() ?: '';
	}

	function getHTMLProperties(): string{
		$tabNr = we_base_request::string('tabnr', self::TAB_PROPERTIES); //FIXME: due to preview - fix this as a better tab-name; replace 1-3 with consts

		if($this->Model->IsFolder == 0 && $tabNr != self::TAB_PROPERTIES && $tabNr != self::TAB_CUSTOMER){
			$tabNr = self::TAB_PROPERTIES;
		}

		$hiddens = [
			'cmd' => '',
			'pnt' => 'edbody',
			'tabnr' => $tabNr,
		];

		//$weSuggest = & weSuggest::getInstance();
		switch($tabNr){
			case self::TAB_PREVIEW:
				return $this->getHTMLEditorPreview();
			default:
				// Property tab content
				return we_html_element::div(
						['id' => 'tab1',
							'style' => ($tabNr == self::TAB_PROPERTIES ? 'display: block;' : 'display: none')
						], $this->View->getCommonHiddens($hiddens) .
						we_html_element::hiddens([
							'IsFolder' => (string) ($this->Model->IsFolder),
							'presetFolder' => we_base_request::string('presetFolder', '')
						]) .
							(new we_html_multiIconBox($this->getHTMLGeneral()->append(($this->Model->IsFolder ?
									$this->getHTMLPropertiesGroup() :
									$this->getHTMLPropertiesItem()
								))
								->add(($this->Model->Selection == we_navigation_navigation::SELECTION_STATIC || $this->Model->IsFolder) ? (new we_html_multiIconBoxContentLine(g_l('navigation', '[more_attributes]'), (new we_html_multiIconBox($this->getHTMLAttributes(), 'weNaviAttrib'))->get()))
									->rightHead()
									->space(we_html_multiIconBox::SPACE_ICON)
									->fold()
									->noline() : null)
								->append($this->Model->IsFolder ? $this->getHTMLPropertiesGroupMore() : new we_html_multiIconBoxContent())
								->append($this->getHTMLImage())
							))
							->marginLeft()
							->get()
					) .
					($this->Model->IsFolder && we_base_permission::hasPerm([we_base_perm::NAVIGATION_EDIT_DYNAMIC]) ?
					we_html_element::div(['id' => 'tab' . self::TAB_CONTENT, 'style' => ($tabNr == self::TAB_CONTENT ? 'display: block;' : 'display: none')],
													(new we_html_multiIconBox($this->getHTMLTab2()))->marginLeft()->get()) :
					''
					) . ((we_modules_info::isActive(we_modules_info::CUSTOMER)) ?
					we_html_element::div(['id' => 'tab' . self::TAB_CUSTOMER, 'style' => ($tabNr == self::TAB_CUSTOMER ? 'display: block;' : 'display: none')],
													(new we_html_multiIconBox($this->getHTMLTab3()))->marginLeft()->get()) :
					''
					);
		}
	}

	private function getHTMLFieldChooser(string $title, string $name, string $value, string $cmd, string $type, string $extraField = '', int $extraFieldWidth = 0): string{
		$disabled = !(($this->Model->DynamicSelection == we_navigation_navigation::DYN_CLASS && $this->Model->ClassID != 0) || ($this->Model->DynamicSelection == we_navigation_navigation::DYN_DOCTYPE && $this->Model->DocTypeID != 0));
		$button = we_html_button::create_button(we_html_button::SELECT, "javascript:fieldChooserBut('" . $cmd . "');", '', '', $disabled, false, '_' . $name);
		$tmp = explode('_', $value, 2);
		if(!$extraField){
			$showValue = ($type === we_navigation_navigation::DYN_CLASS && !empty($tmp[1])) ? $tmp[1] : $value;
			return we_html_tools::formElementTable($title, we_html_element::hidden($name, $value) . we_html_tools::textInput('__' . $name, $showValue, 0, ['onchange' => "setFieldValue('" . $name . "',this); we_cmd('setHot');"], 'text', '400px'), '', $button);
		}
		$showValue = !empty($tmp[1]) ? $tmp[1] : $value;
		return we_html_tools::formElementTable($title, we_html_element::hidden($name, $value) .
				we_html_tools::textInput('__' . $name, $showValue, 0, ['onchange' => "setFieldValue('" . $name . "',this); we_cmd('setHot');"], 'text', (400 - abs($extraFieldWidth) - 8) . 'px'), '', $extraField, $button);
	}

	private function getHTMLChooser(string $title, string $table = FILE_TABLE, int $rootDirID = 0, string $IDName = 'ID', int $IDValue = 0, string $PathName = 'Path', string $cmd = '', array $filter = [
			we_base_ContentTypes::WEDOCUMENT], bool $disabled = false, bool $showtrash = false, array $acCTypes = []): string{
		if($IDValue == 0){
			$path = '/';
		} elseif(isset($this->Model->$IDName) && !empty($this->Model->$IDName)){
			$path = id2path($this->Model->$IDName, $table);
		} else {
			$path = id2path($IDValue, $table);
		}

		if($table === NAVIGATION_TABLE){
			$cmd = "javascript:we_cmd('we_navigation_dirSelector',{id:document.we_form.$IDName.value,targetID:'" . $IDName . "',targetName:'" . $PathName . "',jsCmd:'" . $cmd . "'})";
			$selector = we_gui_suggest::DirSelector;
		} else if(in_array(we_base_ContentTypes::FOLDER_DOCUMENT, $filter) || in_array(we_base_ContentTypes::FOLDER_OBJECTFILE, $filter)){
			$cmd = "javascript:we_cmd('we_selector_file',{id:document.we_form.$IDName.value,table:'" . $table . "',targetID:'" . $IDName . "',targetName:'" . $PathName . "',jsCmd:'" . $cmd . "',rootDirID:'" . $rootDirID . "'})";
			$selector = we_gui_suggest::DirSelector;
		} else {
			$cmd = "javascript:we_cmd('" . (in_array(we_base_ContentTypes::IMAGE, $filter, true) ? 'we_selector_image' : 'we_selector_document') . "',{id:document.we_form." . $IDName . ".value,table:'" . $table . "',targetID:'" . $IDName . "',targetName:'" . $PathName . "',jsCmd:'" . $cmd . "',rootDirID:'" . $rootDirID . "',filter:'" . implode('\',\'', $filter) . "',open_doc:" . (we_base_permission::hasPerm([
					we_base_perm::WORK_WORKSPACE_CAN_SELECT_OTHER_FILES]) ? 0 : 1) . "})";
			$selector = we_gui_suggest::DocSelector;
		}

		if($selector == we_gui_suggest::DocSelector){
			$path = $path === '/' ? '' : $path;
			$required = false;
		} else {
			$required = true;
		}

		if($showtrash){
			$button = we_html_button::create_button(we_html_button::SELECT, $cmd, '', '', $disabled) .
				we_html_button::create_button(we_html_button::TRASH, 'javascript:document.we_form.elements["' . $IDName . '"].value=0;document.we_form.elements["' . $PathName . '"].value="/";');
			$width = 157;
		} else {
			$width = 120;
			$button = we_html_button::create_button(we_html_button::SELECT, $cmd, '', '', $disabled);
		}

		$weSuggest = we_gui_suggest::getInstance();
		$weSuggest->setAcId($PathName)
			->setContentType($acCTypes ?: $filter)
			->setInput($PathName, $path, [], false, true)
			->setLabel($title)
			->setMaxResults(50)
			->setRequired($required)
			->setResult($IDName, (string) ($IDValue))
			->setSelector($selector)
			->setTable($table)
			->setWidth(520 - $width)
			->setSelectButton($button);

		$weAcSelector = $weSuggest->getHTML();
		return $weAcSelector /* ?:
		  we_html_tools::formElementTable($title, we_html_tools::textInput($PathName, $path, 0, ['readonly' => 'readonly'], 'text', (520 - $width) . 'px'), we_html_element::hidden($IDName, $IDValue), $button)
		  ) */;
	}

	private function getHTMLCategory(): string{
		$addbut = we_html_button::create_button(we_html_button::ADD, "javascript:we_cmd('we_selector_document',{table:WE().consts.tables.CATEGORY_TABLE,jsCmd:'addCat'});we_cmd('setHot');");
		$del_but = we_html_button::create_button(we_html_button::TRASH, 'javascript:' . we_base_constants::PLACEHOLDER . ";we_cmd('setHot');");

		$this->jsCmd->addCmd('categoriesEdit', [510, $this->Model->Categories, $del_but]);

		$table = (new we_html_table([
				'id' => 'CategoriesBlock',
				'style' => 'display: block;',
				'class' => 'default withSpace'
				], 3, 2
			))
			->setCol(0, 0, g_l('navigation', '[categories]'), ['colspan' => '2', 'class' => 'defaultfont'])
			->setCol(1, 0, we_html_element::div([
					'id' => 'categories',
					'class' => 'blockWrapper',
					'style' => 'width: 520px; height: 60px; border: #AAAAAA solid 1px;'
					]
				), ['colspan' => '2']
			)
			->setCol(2, 0, we_html_forms::checkboxWithHidden($this->Model->CatAnd, 'CatAnd', g_l('navigation', '[catAnd]')), ['style' => 'text-align:left'])
			->setCol(2, 1, we_html_button::create_button(we_html_button::DELETE_ALL, "javascript:removeAllCats()") . $addbut, ['style' => 'text-align:right']);

		return $table->getHtml();
	}

	private function getHTMLFieldSelector(): string{
		$type = we_base_request::string('type'); // doctype || class
		$selection = we_base_request::int('selection'); // doctype or classid
		$cmd = we_base_request::string('cmd'); // js command
		$multi = we_base_request::bool('multi'); // js command

		$fields = [];
		switch($type){
			case we_navigation_navigation::DYN_DOCTYPE://FIXME: is this called?
			case we_navigation_navigation::STYPE_DOCLINK://this is correct
				$ids = array_filter(we_base_request::filterIntList(fS('SELECT Templates FROM ' . DOC_TYPES_TABLE . ' WHERE ID=' . $selection)));
				foreach($ids as $templateID){
					$template = new we_contents_template((int) $templateID);
					$fields = array_merge($fields, array_keys($template->readAllVariantFields(true)));
				}
				$f = array_unique($fields);
				$fields = array_combine($f, $f);
				break;
			default:
				if(we_modules_info::isActive(we_modules_info::OBJECT)){
					$class = new we_contents_object($selection);
					$found = array_keys($class->getAllVariantFields());
					foreach($found as $key){
						[, $tmp] = explode('_', $key, 2);
						$fields[$key] = $tmp;
					}
				}
		}
		asort($fields);
		$parts = (new we_html_multiIconBoxContent())
			->add(new we_html_multiIconBoxContentLine('', we_html_tools::select('fields', $fields, 20, '', $multi, ['style' => "width: 300px; height: 200px; margin: 5px 0 5px 0;",
					'onclick' => "setTimeout(selectItem,100);"])));

		$body = we_html_element::body(
				['class' => 'weDialogBody', 'onload' => "loaded=1;"], we_html_element::form(['name' => we_base_constants::WE_FORM, 'onsubmit' => "return false"],
																																								(new we_html_multiIconBox($parts))
						->marginLeft()
						->buttons(we_html_button::position_yes_no_cancel(we_html_button::create_button(we_html_button::OK, "javascript:setFields('" . $cmd . "');", '', '', true, false), '', we_html_button::create_button(we_html_button::CLOSE, 'javascript:WE().util.dialogTop(window).doClose();')))
						->headline(g_l('navigation', '[select_field_txt]'))
						->get()
		));

		return $this->getHTMLDocument($body, we_html_element::jsScript(WE_JS_MODULES_DIR . 'navigation/navigation_view_prop.js'));
	}

	private function getHTMLCount(): string{
		return we_html_element::div(['style' => "width: 520px; margin-top: 5px"], we_html_tools::formElementTable(g_l('navigation', '[show_count]'), we_html_tools::textInput('ShowCount', (string) ($this->Model->ShowCount ?: 5)
						, 0, ['onblur' => "var r=parseInt(this.value);if(isNaN(r)){this.value=" . ($this->Model->ShowCount ?: 5) . ";}else{ this.value=r; we_cmd('setHot');}"], 'number', '520px')));
	}

	private function getHTMLDirSelector(): string{
		$table = $this->Model->DynamicSelection === we_navigation_navigation::DYN_DOCTYPE ? FILE_TABLE :
			($this->Model->DynamicSelection === we_navigation_navigation::DYN_CLASS ? OBJECT_FILES_TABLE :
			($this->Model->DynamicSelection === we_navigation_navigation::DYN_CATEGORY ? CATEGORY_TABLE :
			FILE_TABLE));

		$rootDirID = intval(($ws = we_users_util::get_ws($table)) ? reset($ws) : 0);

		$button_doc = we_html_button::create_button(we_html_button::SELECT, "javascript:we_cmd('we_selector_directory',{id:document.we_form.elements.FolderID.value,table:WE().consts.tables.FILE_TABLE,targetID:'FolderID',targetName:'FolderPath',jsCmd:'setHot',rootDirID:" . $rootDirID . "})");
		$countSubDirs = 1;
		if(we_modules_info::isActive(we_modules_info::OBJECT) && ($this->Model->DynamicSelection === we_navigation_navigation::DYN_CLASS || $this->Model->SelectionType === we_navigation_navigation::STYPE_OBJLINK)){
			$classDirID = fI('SELECT of.ID FROM ' . OBJECT_FILES_TABLE . ' `of` WHERE of.IsClassFolder=1 AND of.TableID=' . $this->Model->ClassID);
			$countSubDirs = fI('SELECT COUNT(1) FROM ' . OBJECT_FILES_TABLE . ' WHERE ParentID=' . $classDirID . ' AND IsFolder=1');
		}

		$button_obj = we_modules_info::isActive(we_modules_info::OBJECT) ? we_html_button::create_button(we_html_button::SELECT, "javascript:we_cmd('we_selector_directory',{id:document.we_form.elements.FolderID.value,table:WE().consts.tables.OBJECT_FILES_TABLE,targetID:'FolderID',targetName:'FolderPath',jsCmd:'setHot',rootDirID:classDirs[document.we_form.elements.ClassID.options[document.we_form.elements.ClassID.selectedIndex].value]})", '', "", ($countSubDirs ? false : true), false, "_XFolder") : '';
		$button_cat = we_html_button::create_button(we_html_button::SELECT, "javascript:we_cmd('we_selector_document',{id:document.we_form.elements.FolderID.value,table:WE().consts.tables.CATEGORY_TABLE,targetID:'FolderID',targetName:'FolderPath',rootDirID:'" . $rootDirID . "'})");
		$buttons = we_html_element::div(['id' => 'docFolder', 'style' => 'display: ' . (($table === FILE_TABLE) ? 'inline' : 'none')], $button_doc) .
			we_html_element::div(['id' => "objFolder", 'style' => 'display: ' . ($table === OBJECT_FILES_TABLE ? 'inline' : 'none')], $button_obj) .
			we_html_element::div(['id' => "catFolder", 'style' => 'display: ' . ($table === CATEGORY_TABLE ? 'inline' : 'none')], $button_cat);


		$path = id2path($this->Model->FolderID ?: $rootDirID, $table);
		$attribs = ['onchange' => "we_cmd('setHot');"];
		if(!$countSubDirs){
			$attribs['disabled'] = "disabled";
		}

		$weSuggest = we_gui_suggest::getInstance();
		$weSuggest->setAcId('FolderPath', $table === OBJECT_FILES_TABLE ? id2path($this->Model->ClassID, OBJECT_FILES_TABLE) : '')
			->setContentType(we_base_ContentTypes::FOLDER_TYPES)
			->setInput('FolderPath', $path, $attribs)
			->setMaxResults(50)
			->setResult('FolderID', (string) ($this->Model->FolderID ?: $rootDirID))
			->setSelector(we_gui_suggest::DirSelector)
			->setLabel(g_l('navigation', '[dir]'))
			->setTable($table)
			->setWidth(400)
			->setSelectButton($buttons);

		return we_html_element::div(['style' => 'margin-top:5px;'], $weSuggest->getHTML());
	}

	private function getHTMLDynPreview(): string{
		$select = new we_html_select(['size' => 20, 'style' => 'width: 420px; height: 250px; margin: 5px 0 5px 0;']);

		$items = $this->Model->getDynamicEntries();

		foreach($items as $k => $item){
			$txt = id2path(
				$item['id'], ($this->Model->DynamicSelection == we_navigation_navigation::DYN_DOCTYPE) ? FILE_TABLE : ($this->Model->DynamicSelection == we_navigation_navigation::DYN_CATEGORY ? CATEGORY_TABLE : OBJECT_FILES_TABLE));
			if($item['field']){
				$opt = we_html_select::getNewOptionGroup([
						'class' => 'bold',
						'style' => 'font-style: normal; color: darkblue;',
						'label' => $item['field']
				]);
				$opt->addChild(we_html_select::getNewOption((string) $k, $txt));
				$select->addChild($opt);
			} else {
				$select->addOption((string) $k, $txt);
			}
		}

		return $this->getHTMLDocument(
				we_html_element::body(['class' => 'weDialogBody'], we_html_element::form([
						'name' => we_base_constants::WE_FORM, 'onsubmit' => 'return false'
						], (new we_html_multiIconBox((new we_html_multiIconBoxContent())
								->add(new we_html_multiIconBoxContentLine(g_l('navigation', ($this->Model->DynamicSelection === we_navigation_navigation::DYN_CATEGORY ? '[categories]' : ($this->Model->DynamicSelection === we_navigation_navigation::DYN_CLASS ? '[objects]' : '[documents]'))), $select->getHtml()))))
							->marginLeft()
							->buttons(we_html_element::div(['style' => "float:right;"], we_html_button::create_button(we_html_button::CLOSE, 'javascript:WE().util.dialogTop(window).doClose();')))
							->headline(g_l('navigation', '[dyn_selection]'))
							->get()
				)), '');
	}

	private function getHTMLWorkspace(string $type = 'object', int $defClassID = 0, string $field = 'WorkspaceID'): string{
		if($type === 'class'){
			$wsid = ($this->Model->DynamicSelection == we_navigation_navigation::DYN_CLASS && $this->Model->ClassID ?
				we_navigation_dynList::getWorkspacesForClass($this->Model->ClassID) :
				($defClassID ?
				we_navigation_dynList::getWorkspacesForClass($defClassID) :
				[]
				));

			return we_html_element::div(['id' => "objLinkWorkspaceClass", 'style' => 'display: ' . (($this->Model->Selection == we_navigation_navigation::SELECTION_DYNAMIC) ? 'block' : 'none') . ';margin-top: 5px;'], we_html_tools::formElementTable(g_l('navigation', '[workspace]'), we_html_tools::select('WorkspaceIDClass', $wsid, 1, $this->Model->WorkspaceID, false, [
							'style' => 'width: 520px;', 'onchange' => "we_cmd('setHot');"])
			));
		}

		$wsid = ($this->Model->SelectionType === we_navigation_navigation::STYPE_OBJLINK && $this->Model->LinkID ?
			we_navigation_dynList::getWorkspacesForObject($this->Model->LinkID) : []);

		if($field === 'WorkspaceID'){
			return we_html_element::div(['id' => "objLinkWorkspace", 'style' => 'display: ' . (($this->Model->SelectionType === we_navigation_navigation::STYPE_OBJLINK && ($this->Model->WorkspaceID > -1)) ? 'block' : 'none') . ';margin-top: 5px;'], we_html_tools::formElementTable(g_l('navigation', '[workspace]'), we_html_tools::select('WorkspaceID', $wsid, 1, $this->Model->WorkspaceID, false, [
							'style' => 'width: 520px;', 'onchange' => "we_cmd('setHot');"])));
		}

		return we_html_element::div(['id' => "objLinkFolderWorkspace", 'style' => 'display: ' . (($this->Model->SelectionType === we_navigation_navigation::STYPE_OBJLINK && ($this->Model->WorkspaceID > -1)) ? 'block' : 'none') . ';margin-top: 5px;'], we_html_tools::formElementTable(g_l('navigation', '[workspace]'), we_html_tools::select('WorkspaceID', $wsid, 1, $this->Model->WorkspaceID, false, [
						'style' => 'width: 520px;', 'onchange' => "we_cmd('setHot');"])));
	}

	private function getHTMLLink(string $prefix = ''): string{
		$cmd = "javascript:we_cmd('we_selector_document',{id:document.we_form.elements." . $prefix . "UrlID.value,table:WE().consts.tables.FILE_TABLE,targetID:'" . $prefix . "UrlID',targetName:'" . $prefix . "UrlIDPath',jsCmd:'setHot',rootDirID:0,filter:WE().consts.contentTypes.WEDOCUMENT,open_doc:" . (we_base_permission::hasPerm([
				we_base_perm::WORK_WORKSPACE_CAN_SELECT_OTHER_FILES]) ? 0 : 1) . "})";

		$path = id2path($this->Model->UrlID);

		$weSuggest = we_gui_suggest::getInstance();
		$weSuggest->setAcId($prefix . 'UrlIDPath')
			->setContentType(array_merge(we_base_ContentTypes::LINK_ABLE, [we_base_ContentTypes::FOLDER_DOCUMENT]))
			->setInput($prefix . 'UrlIDPath', $path, [], false, true)
			->setMaxResults(50)
			->setResult($prefix . 'UrlID', (string) ($this->Model->UrlID))
			->setSelector(we_gui_suggest::DocSelector)
			->setWidth(400)
			->setSelectButton(we_html_button::create_button(we_html_button::SELECT, $cmd));

		$weAcSelector = $weSuggest->getHTML();

		return we_html_element::div(['id' => $prefix . 'LinkSelectionDiv', 'style' => 'display: ' . (($this->Model->SelectionType === we_navigation_navigation::STYPE_CATLINK || $this->Model->DynamicSelection == we_navigation_navigation::DYN_CATEGORY) ? 'block' : 'none') . ';margin-top: 5px;'], we_html_tools::formElementTable(g_l('navigation', '[linkSelection]'), we_html_tools::select($prefix . 'LinkSelection', [
						we_navigation_navigation::LSELECTION_INTERN => g_l('navigation', '[intern]'),
						we_navigation_navigation::LSELECTION_EXTERN => g_l('navigation', '[extern]')
						], 1, $this->Model->LinkSelection, false, ['style' => 'width: 520px;', 'onchange' => "setLinkSelection('" . $prefix . "',this.value); we_cmd('setHot');"]))) .
			we_html_element::div(['id' => $prefix . 'intern', 'style' => 'display: ' . (($this->Model->LinkSelection === we_navigation_navigation::LSELECTION_INTERN && $this->Model->SelectionType != we_navigation_navigation::STYPE_URLLINK) ? 'block' : 'none') . ';margin-top: 5px;'], $weAcSelector) .
			we_html_element::div(['id' => $prefix . 'extern', 'style' => 'display: ' . (($this->Model->LinkSelection === we_navigation_navigation::LSELECTION_EXTERN || $this->Model->SelectionType === we_navigation_navigation::STYPE_URLLINK) ? 'block' : 'none') . ';margin-top: 5px;'], we_html_tools::textInput($prefix . 'Url', $this->Model->Url, 0, [
					'onchange' => "we_cmd('setHot');", 'placeholder' => 'http://www.example.com'], 'text', '520px')) .
			we_html_element::div(['style' => "margin-top: 5px;"], we_html_tools::formElementTable(g_l('navigation', '[catParameter]'), we_html_tools::textInput($prefix . 'CatParameter', $this->Model->CatParameter, 0, [
						'onchange' => "we_cmd('setHot');"], 'text', '520px')));
	}

	private function getHTMLAttributes(): we_html_multiIconBoxContent{
		$title = we_html_tools::formElementTable(g_l('navigation', '[title]'), we_html_tools::textInput('Attributes[title]', $this->Model->getAttribute('title'), 0, [
					'onchange' => "we_cmd('setHot')"], 'text', '520px'));

		$anchor = we_html_tools::formElementTable(g_l('navigation', '[anchor]'), we_html_tools::textInput('Attributes[anchor]', $this->Model->getAttribute('anchor'), 0, [
					'onchange' => "we_cmd('setHot');", 'pattern' => '^#?[a-zA-Z]+[a-zA-Z0-9_:.-=]*$',
					'onblur' => "if(this.value&&!new RegExp(\'#?[a-z]+[a-z0-9_:.-=]*$\',\'i\').test(this.value)){WE().util.showMessage('" . g_l('linklistEdit', '[anchor_invalid]') . "', WE().consts.message.WE_MESSAGE_ERROR);}"], 'text', '520px') . '<br/>' .
				we_html_forms::checkboxWithHidden($this->Model->CurrentOnAnker, 'CurrentOnAnker', g_l('navigation', '[current_on_anker]'), false, "defaultfont", "we_cmd('setHot');"));

		$target = we_html_tools::formElementTable(g_l('navigation', '[target]'), we_html_tools::targetBox('Attributes[target]', 400, '', $this->Model->getAttribute('target'), "we_cmd('setHot');", 100));

		$link = we_html_tools::formElementTable(g_l('navigation', '[link_attribute]'), we_html_tools::textInput('Attributes[link_attribute]', $this->Model->getAttribute('link_attribute'), 0, [
					'onchange' => "we_cmd('setHot');"], 'text', '520px'));

		$lang = we_html_tools::getLangField('Attributes[lang]', $this->Model->getAttribute('lang'), g_l('navigation', '[link_language]'), 520);
		$hreflang = we_html_tools::getLangField('Attributes[hreflang]', $this->Model->getAttribute('hreflang'), g_l('navigation', '[href_language]'), 520);

		$parts = (new we_html_multiIconBoxContent())
			->add((new we_html_multiIconBoxContentLine('', we_html_tools::alertAttentionBox(g_l('navigation', '[linkprops_desc]'), we_html_tools::TYPE_INFO, 520)))
				->space(we_html_multiIconBox::SPACE_ICON)
				->noline())
			->add((new we_html_multiIconBoxContentLine(g_l('navigation', '[attributes]'), $title . $anchor . $link . $target, we_html_multiIconBox::PROP_ATTRIB))
				->noline())
			->add((new we_html_multiIconBoxContentLine(g_l('navigation', '[language]'), $lang . $hreflang, we_html_multiIconBox::PROP_LANG))
			->noline());

		$accesskey = we_html_tools::formElementTable(g_l('navigation', '[accesskey]'), we_html_tools::textInput('Attributes[accesskey]', $this->Model->getAttribute('accesskey'), 0, [
					'onchange' => "we_cmd('setHot');"], 'text', '520px'));

		$tabindex = we_html_tools::formElementTable(g_l('navigation', '[tabindex]'), we_html_tools::textInput('Attributes[tabindex]', $this->Model->getAttribute('tabindex'), 0, [
					'onchange' => "we_cmd('setHot');"], 'text', '520px'));

		$parts->add((new we_html_multiIconBoxContentLine(g_l('navigation', '[keyboard]'), $accesskey . $tabindex, we_html_multiIconBox::PROP_KEYBOARD))
				->noline());

		$relfield = we_html_tools::getRelSelect('Attributes[rel]', $this->Model->getAttribute('rel'), 'rel', "we_cmd('setHot');", 500);

		$parts->add((new we_html_multiIconBoxContentLine(g_l('navigation', '[relation]'), $relfield, we_html_multiIconBox::PROP_REVREL))
				->noline());

		return $parts;
	}

	private function getHTMLImageAttributes(): string{
		$input_width = '70px';
		$img_props = (new we_html_table(['class' => 'withSpace'], 4, 5))
			->setCol(0, 0, we_html_tools::formElementTable(g_l('navigation', '[icon_width]'), we_html_tools::textInput('Attributes[icon_width]', $this->Model->getAttribute('icon_width'), 0, [
						'onchange' => "we_cmd('setHot');"], 'text', $input_width)))
			->setCol(0, 1, we_html_tools::formElementTable(g_l('navigation', '[icon_height]'), we_html_tools::textInput('Attributes[icon_height]', $this->Model->getAttribute('icon_height'), 0, [
						'onchange' => "we_cmd('setHot');"], 'text', $input_width)))
			->setCol(0, 2, we_html_tools::formElementTable(g_l('navigation', '[icon_border]'), we_html_tools::textInput('Attributes[icon_border]', $this->Model->getAttribute('icon_border'), 0, [
						'onchange' => "we_cmd('setHot');"], 'text', $input_width)))
			->setCol(0, 3, we_html_tools::formElementTable(g_l('navigation', '[icon_hspace]'), we_html_tools::textInput('Attributes[icon_hspace]', $this->Model->getAttribute('icon_hspace'), 0, [
						'onchange' => "we_cmd('setHot');"], 'text', $input_width)))
			->setCol(0, 4, we_html_tools::formElementTable(g_l('navigation', '[icon_vspace]'), we_html_tools::textInput('Attributes[icon_vspace]', $this->Model->getAttribute('icon_vspace'), 0, [
						'onchange' => "we_cmd('setHot');"], 'text', $input_width)))
			->setCol(1, 0, we_html_tools::formElementTable(g_l('navigation', '[icon_align]'), we_html_tools::select(
						'Attributes[icon_align]', [
						'' => 'Default',
						'top' => 'Top',
						'middle' => 'Middle',
						'bottom' => 'Bottom',
						'left' => 'left',
						'right' => 'Right',
						'texttop' => 'Text Top',
						'absmiddle' => 'Abs Middle',
						'baseline' => 'Baseline',
						'absbottom' => 'Abs Bottom'
						], 1, $this->Model->getAttribute('icon_align'), false, ['style' => 'width: 470px;'])), ['colspan' => '5'])
			->setCol(2, 0, we_html_tools::formElementTable(g_l('navigation', '[icon_alt]'), we_html_tools::textInput('Attributes[icon_alt]', $this->Model->getAttribute('icon_alt'), 0, [
						'onchange' => "we_cmd('setHot');"], 'text', '470px')), ['colspan' => '5'])
			->setCol(3, 0, we_html_tools::formElementTable(g_l('navigation', '[icon_title]'), we_html_tools::textInput('Attributes[icon_title]', $this->Model->getAttribute('icon_title'), 0, [
					'onchange' => "we_cmd('setHot');"], 'text', '470px')), ['colspan' => '5']);

		return $img_props->getHTML();
	}

	private function getHTMLTab3(): we_html_multiIconBoxContent{
		$view = new we_navigation_customerFilterView($this->Model->customerFilter, 520);
		return (new we_html_multiIconBoxContent())
				->add((new we_html_multiIconBoxContentLine('', $view->getFilterHTML($this->Model->IsFolder == 0 && $this->Model->Selection == we_navigation_navigation::SELECTION_DYNAMIC)))->noline()->space(we_html_multiIconBox::SPACE_MED));
	}

	protected function getHTMLEditorFooter(array $btn_cmd = []): string{
		if(we_base_request::bool('home')){
			return parent::getHTMLEditorFooter([]);
		}

		$buttons = we_html_element::div(['class' => 'main'],
																	we_html_button::create_button(we_html_button::FOOTER_SAVE, "javascript:top.content.makeNewDoc=document.we_form.makeNewDoc.checked;top.content.we_cmd('we_navigation_save');", '', '', (!we_base_permission::hasPerm([
						we_base_perm::NAVIGATION_EDIT]))) .
				we_html_forms::checkbox('makeNewDoc', false, 'makeNewDoc', g_l('global', ($this->View->Model->IsFolder ? '[we_new_folder_after_save]' : '[we_new_entry_after_save]')))
		);
		if($this->Model->ID && we_base_permission::hasPerm([we_base_perm::NAVIGATION_DELETE, we_base_perm::NAVIGATION_EDIT])){
			$buttons .= we_html_element::div(['class' => 'optional'],
																		we_html_button::create_button(we_html_button::FOOTER_TRASH, "javascript:top.content.we_cmd('we_navigation_delete');"));
		}

		return $this->getHTMLDocument(
				we_html_element::body([
					'id' => 'footerBody',
					'onload' => 'document.we_form.makeNewDoc.checked=top.content.makeNewDoc;'
					], we_html_element::form([], we_html_element::div(['class' => 'footertable'], $buttons))));
	}

}
