<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: bb9e63f458ee $
 * $Author: Lukas Imhof $
 * $Date: 2020-07-09 21:55 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
class we_newsletter_dirSelector extends we_selector_directory{
	/** @var string */
	var $fields = 'ID,ParentID,Text,Path,IsFolder';

	function __construct(int $id, string $JSIDName = '', string $JSTextName = '', string $JSCommand = '', string $order = '', string $FolderText = '', int $rootDirID = 0, bool $multiple = false){
		$this->fields = 'ID,ParentID,Text,Path,IsFolder,IF(IsFolder,"' . we_base_ContentTypes::FOLDER_COMMON . '","' . we_base_ContentTypes::NEWSLETTER . '") AS ContentType';
		$table = NEWSLETTER_TABLE;
		parent::__construct($id, $table, $JSIDName, $JSTextName, $JSCommand, $order, $FolderText, $rootDirID, $multiple);
	}

	protected function printCreateFolderHTML(): void{
		$this->FolderText = rawurldecode($this->FolderText);
		$txt = $this->FolderText;

		$folder = new we_newsletter_newsletter();
		$folder->Text = $txt;
		$folder->ParentID = $this->dir;t_e('this', $this, id2path($this->dir, NEWSLETTER_TABLE));
		$folder->Path = ($this->dir ? id2path($this->dir, NEWSLETTER_TABLE) : '') . '/' . $folder->Text;
		$folder->IsFolder = true;
		$folder->save();

		if($this->canSelectDir){
			$this->jsCmd->addCmd('updateSelectData', [
				'currentPath' => $folder->Path,
				'currentID' => $folder->ID,
				'currentText' => $folder->getText(),
			]);
		}

		$this->printCmdAddEntriesHTML();
		$this->jsCmd->addCmd('setButtons', [['NewFolderBut', $this->userCanMakeNewDir()]]);

		//FIXME: the following cmd does not enter the new entry at the alphabetically correct position
		$this->jsCmd->addCmd('loadTree', ['clear' => false, 'items' => we_tree_newsletter::getItems($folder->ParentID)]);

		$this->setWriteSelectorData();
		echo we_html_tools::getHtmlTop('', $this->jsCmd->getCmds(), we_html_element::body());
	}

	protected function query(): array{
		return $this->db->getAllq('SELECT ' . $this->fields . ' FROM ' . $this->db->escape($this->table) . ' WHERE IsFolder=1 AND ParentID=' . intval($this->dir) .
				self::getWsQuery(NEWSLETTER_TABLE) .
				($this->order ? (' ORDER BY IsFolder DESC,' . $this->order) : '')
		);
	}

	protected function printCmdAddEntriesHTML(): void{
		$entries = [];
		foreach($this->query() as $item){
			$entries[] = [
				(int) $item['ID'],
				$item['Text'],
				(int) $item['IsFolder'],
				$item['Path']
			];
		}
		if($entries){
			$this->jsCmd->addCmd('addEntries', $entries);
		}
		$this->jsCmd->addCmd('setButtons', [['NewFolderBut', $this->userCanMakeNewDir()]]);
		$this->jsCmd->addCmd('writeBody');
	}

	protected function getFramsetJSFile(): string{
		return parent::getFramsetJSFile() .
			we_html_element::jsScript(JS_DIR . 'selectors/newsletterdir_selector.js');
	}

	protected function printHeaderHeadlines(): string{
		return '<th class="selector selectoricon"></th>
		<th class="selector"colspan="2">' . g_l('fileselector', '[filename]') . '</th>';
	}

	protected function userCanSeeDir(bool $showAll = false): bool{
		return true;
	}

	protected function userCanMakeNewDir(): bool{
		return we_base_permission::hasPerm([we_base_perm::NEWSLETTER_NEW]);
	}

	protected function userHasFolderPerms(): bool{
		return we_base_permission::hasPerm([we_base_perm::NEWSLETTER_NEW]);
	}

	public function printHTML(string $what = we_selector_file::FRAMESET, bool $withPreview = true): void{
		parent::printHTML($what, false);
	}

}
