<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: c7679e8cd076 $
 * $Author: Marc Krämer <marc.kraemer@aldente-it.de> $
 * $Date: 2019-11-02 01:19 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
class we_rpcCmd_collectionGetValidByID extends we_rpc_cmd{
	/** @var ?we_contents_collection */
	protected $collection = null;

	/** @var int */
	protected $collectionID = 0;

	/** @var int[] */
	protected $IDs = [];

	/** @var string */
	protected $transaction = '';

	/** @var string */
	protected $table = '';

	/** @var bool */
	protected $full = false;

	/** @var bool */
	protected $recursive = true;

	/** @var bool */
	protected $initSessDat = false;

	function __construct(){
		parent::__construct();
		$this->initByRequest();
	}

	protected function initByRequest(): void{
		if(!($this->IDs = we_base_request::intList('we_cmd', [], 'ids'))){
			$this->resp->setData('error', ['Missing field id']);
			$this->resp->setStatus(false);
			return;
		}

		$this->resp->setData('index', we_base_request::int('we_cmd', 0, 'index'));
		$this->resp->setData('notReplace', we_base_request::bool('we_cmd', true, 'notReplace'));
		$this->resp->setData('message', we_base_request::string('we_cmd', '', 'message'));

		// TODO: only save as prop when really needed!
		$this->transaction = we_base_request::_(we_base_request::TRANSACTION, 'we_cmd', '', 'transaction');
		$this->collectionID = we_base_request::int('we_cmd', 0, 'collection');
		$this->full = we_base_request::bool('we_cmd', false, 'full');
		$this->recursive = we_base_request::bool('we_cmd', true, 'recursive');
		$this->table = we_base_request::_(we_base_request::TABLE, 'we_cmd', FILE_TABLE, 'table');
		try{
			$this->collection = we_contents_root::getSessionDocument($this->transaction);
			$this->initSessDat = true;
		} catch (Exception $ex){
			if($this->collectionID){
				$this->collection = new we_contents_collection();
				$this->collection->initByID($this->collectionID);
			} else {
				$this->resp->setData('error', ['no collection error']);
				$this->resp->setStatus(false);
				return;
			}
		}
	}

	protected function getValidItems(): array{
		return $this->collection->getValidItemsFromIDs($this->IDs, $this->full, $this->recursive, $this->table);
	}

	function execute(): we_rpc_response{
		if(!$this->resp->getStatus()){
			return $this->resp;
		}

		$this->resp->setData('items', $this->getValidItems());

		return $this->resp;
	}

}
