<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: 1296fd17b7f6 $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2020-04-19 12:52 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
class we_rpcCmd_objectfileFolderAction extends we_rpc_cmd{

	function execute(): we_rpc_response{
		$transaction = we_base_request::_(we_base_request::TRANSACTION, 'we_cmd', '', 'transaction');
		$we_doc = we_contents_root::getSessionDocument($transaction);
		assert($we_doc instanceof we_folder_objectFile);
		
		if(!$we_doc->ID){
			return $this->resp;
		}

		$ids = [];
		$arr = we_base_request::string('we_cmd', '', 1);
		if($arr){
			$allDocs = explode(',', $arr);
			foreach($allDocs as $v){
				$parts = explode('_', $v, 2);
				$ids[] = $parts[1] === OBJECT_FILES_TABLE ? intval($parts[0]) : 0;
			}
			$ids[] = array_filter($ids);
		}

		// FIXME: send deletedItems for ACTION_DELETE and some message
		switch(we_base_request::string('we_cmd', '', 0)){
			case we_folder_base::ACTION_DELETE:
				$we_doc->deleteObjects($ids);
				break;
			case we_folder_base::ACTION_UNPUBLISH:
				$we_doc->publishObjects($ids, false);
				break;
			case we_folder_base::ACTION_PUBLISH:
				$we_doc->publishObjects($ids);
				break;
			case we_folder_base::ACTION_CATEGORY_ADD:
				$cats = we_base_request::intList('we_cmd', [], 2);
				$we_doc->changeCats($ids, $cats, true);
				break;
			case we_folder_base::ACTION_CATEGORY_DELETE:
				$cats = we_base_request::intList('we_cmd', [], 2);
				$we_doc->changeCats($ids, $cats, false);
				break;
			case we_folder_base::ACTION_UNSEARCHABLE:
				$we_doc->setObjectProperty($ids, 'IsSearchable', false);
				break;
			case we_folder_base::ACTION_SEARCHABLE:
				$we_doc->setObjectProperty($ids, 'IsSearchable', true);
				break;
			case we_folder_base::ACTION_COPY_WS:
				$we_doc->setObjectProperty($ids, 'Workspaces');
				break;
			case we_folder_base::ACTION_COPY_TID:
				$we_doc->setObjectProperty($ids, 'TriggerID');
				break;
		}

		return $this->resp;
	}

}
