<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: a4498495f5d5 $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2020-07-07 22:00 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
class we_selector_directory extends we_selector_file{
	protected $userCanMakeNewFolder = true;
	protected $FolderText = '';

	function __construct(int $id, string $table = '', string $JSIDName = '', string $JSTextName = '', string $JSCommand = '', string $order = '', string $FolderText = '', int $rootDirID = 0, bool $multiple = false, string $filter = '', int $startID = 0){
		parent::__construct($id, $table, $JSIDName, $JSTextName, $JSCommand, $order, $rootDirID, $multiple, $filter, $startID);
		switch($this->table){
			case FILE_TABLE:
			case TEMPLATES_TABLE:
			case OBJECT_FILES_TABLE :
			case OBJECT_TABLE :
				$this->fields .= ',DATE_FORMAT(Mod_Date,"' . g_l('weEditorInfo', '[mysql_date_format]') . '") AS Mod_Date,RestrictOwners,Owners,OwnersReadOnly,CreatorID';
				break;
			default:
		}
		$this->title = g_l('fileselector', '[dirSelector][title]');
		$this->userCanMakeNewFolder = $this->userCanMakeNewDir();
		$this->FolderText = $FolderText;
	}

	public function printHTML(string $what = we_selector_file::FRAMESET, bool $withPreview = true): void{
		switch($what){
			case self::SETDIR:
				$this->printSetDirHTML();
				break;
			case self::CREATEFOLDER:
				$this->printCreateFolderHTML();
				break;
			case self::PREVIEW:
				$this->printPreviewHTML();
				break;
			default:
				parent::printHTML($what, $withPreview);
		}
	}

	protected function printCmdHTML(): void{
		$this->jsCmd->addCmd('setButtons', [['NewFolderBut', $this->userCanMakeNewFolder]]);

		parent::printCmdHTML();
	}

	protected function query(): array{
		return $this->db->getAllq('SELECT ' . $this->fields . ' FROM ' . $this->db->escape($this->table) . ' WHERE IsFolder=1 AND ParentID=' . intval($this->dir) . ' AND((1' . we_users_util::makeOwnersSql() . ') ' .
				self::getWsQuery($this->table) . ')' . ($this->order ? (' ORDER BY IsFolder DESC,' . $this->order) : ''));
	}

	protected function setDefaultDirAndID(bool $setLastDir): void{
		$ws = we_users_util::get_ws($this->table);
		$rootDirID = (int) ($ws ? reset($ws) : 0);

		$this->dir = $this->startID ?: ($setLastDir ? (isset($_SESSION['weS']['we_fs_lastDir'][$this->table]) ? intval($_SESSION['weS']['we_fs_lastDir'][$this->table]) : $rootDirID ) : $rootDirID);
		if($ws && in_array($this->dir, $ws)){
			$this->dir = 0;
		}
		$this->id = $this->dir;
		if($this->rootDirID){
			if(!in_parentID($this->dir, $this->rootDirID, $this->table)){
				$this->dir = $this->rootDirID;
				$this->id = $this->rootDirID;
			}
		}
		$this->path = '';

		$this->values = [
			'ParentID' => 0,
			'Text' => '/',
			'Path' => '/',
			'IsFolder' => 1,
			'Mod_Date' => '',
			'RestrictOwners' => 0,
			'Owners' => '',
			'OwnersReadOnly' => '',
			'CreatorID' => 0
		];
	}

	protected function getFsQueryString(string $what): string{
		return WEBEDITION_DIR . 'we_cmd.php?we_cmd[0]=' . get_class($this) . '&we_cmd[what]=' . $what . '&we_cmd[rootDirID]=' . $this->rootDirID . "&we_cmd[table]=" . $this->table . "&we_cmd[id]=" . $this->id . "&we_cmd[startID]=" . $this->startID . "&we_cmd[order]=" . $this->order . "&we_cmd[open_doc]=" . intval($this->open_doc);
	}

	protected function getFramsetJSFile(): string{
		return parent::getFramsetJSFile() .
			we_html_element::jsScript(JS_DIR . 'selectors/directory_selector.js');
	}

	protected function printCmdAddEntriesHTML(): void{
		$entries = [];

		foreach($this->query() as $item){
			$entries[] = [
				(int) $item['ID'],
				$item['Text'],
				(int) $item['IsFolder'],
				$item['Path'],
				$item['Mod_Date'],
				we_base_ContentTypes::FOLDER_DOCUMENT
			];
		}
		if($entries){
			$this->jsCmd->addCmd('addEntries', $entries);
		}
		$this->jsCmd->addCmd('setButtons', [['NewFolderBut', $this->userCanMakeNewDir()]]);
		$this->jsCmd->addCmd('writeBody');
	}

	protected function printHeaderHeadlines(): string{
		return '<th class="selector treeIcon"></th>
		<th class="selector directory">' . g_l('fileselector', '[filename]') . '</th>
		<th class="selector moddate">' . g_l('fileselector', '[modified]') . '</th>
		<th class="selector remain"></th>';
	}

	protected function userCanSeeDir(bool $showAll = false): bool{
		if(we_base_permission::hasPerm([we_base_perm::ADMINISTRATOR])){
			return true;
		}
		if(!$showAll && !we_users_util::in_workspace($this->dir, we_users_util::get_ws($this->table), $this->table)){
			return false;
		}
		return we_users_util::userIsOwnerCreatorOfParentDir($this->dir, $this->table);
	}

	protected function userCanMakeNewDir(): bool{
		if(($this->table === OBJECT_FILES_TABLE) && (!$this->dir)){
			return false;
		}
		if(we_base_permission::hasPerm([we_base_perm::ADMINISTRATOR])){
			return true;
		}
		if(!$this->userCanSeeDir() || !$this->userHasFolderPerms()){
			return false;
		}
		return true;
	}

	protected function userHasFolderPerms(): bool{
		switch($this->table){
			case FILE_TABLE:
				if(!we_base_permission::hasPerm([we_base_perm::WORK_DOCUMENT_FOLDER_NEW])){
					return false;
				}
				break;
			case TEMPLATES_TABLE:
				if(!we_base_permission::hasPerm([we_base_perm::WORK_TEMPLATES_FOLDER_NEW])){
					return false;
				}
				break;
			case OBJECT_FILES_TABLE :
				if(!we_base_permission::hasPerm([we_base_perm::OBJECTFILE_NEW_FOLDER])){
					return false;
				}
				break;
		}
		return true;
	}

	protected function setWriteSelectorData(bool $withWrite = true): void{
		$pid = $this->dir;
		$options = [];
		$c = 0;
		while($pid != 0){
			$c++;
			$item = getHash('SELECT ID,Text,ParentID FROM ' . $this->db->escape($this->table) . ' WHERE ID=' . intval($pid));
			if($item){
				$options[] = [$item['Text'], $item['ID']];
			}
			$pid = $item['ParentID'];
			if($c > 500 || ($this->rootDirID && $item['ID'] == $this->rootDirID)){
				$pid = 0;
			}
		}
		if(!$this->rootDirID){
			$options[] = ['/', 0];
		}
		$this->jsCmd->addCmd('writeOptions', array_reverse($options));
		if($withWrite){
			$this->jsCmd->addCmd('writeBody');
		}
	}

	protected function printHeaderTable(string $extra = '', bool $append = false): string{
		return '
<table class="selectorHeaderTable">
	<tr style="vertical-align:middle">
		<td class="defaultfont lookinText">' . g_l('fileselector', '[lookin]') . '</td>
		<td class="lookin"><select name="lookin" id="lookin" class="weSelect" onchange="WE().util.dialogTop(window).setDir(this.options[this.selectedIndex].value);" class="defaultfont" style="width:100%"></select>' .
			($this->table !== OBJECT_TABLE ? '
		</td>
		<td>' . we_html_button::create_button(we_html_button::ROOT_DIR, "javascript:if(WE().util.dialogTop(window).fileSelect.data.rootDirButsState){WE().util.dialogTop(window).setRootDir();}", '', "", $this->dir == intval($this->rootDirID), false) . '</td>
		<td>' . we_html_button::create_button(we_html_button::LEVEL_UP, "javascript:if(WE().util.dialogTop(window).fileSelect.data.rootDirButsState){WE().util.dialogTop(window).goBackDir();}", '', "", $this->dir == intval($this->rootDirID), false) . '</td>' .
			($this->table !== CATEGORY_TABLE && ($append || !$extra) ? '<td>' . we_html_button::create_button(we_html_button::NEW_DIR, "javascript:WE().util.dialogTop(window).drawNewFolder();", '', '', !$this->userCanMakeNewDir(), false, '', false, '', '', 'btn_new_dir') . '</td>' : '') .
			($extra ?: '') :
			''
			) . '
	</tr>
</table>';
	}

	protected function printSetDirHTML(): void{
		$isWS = we_users_util::in_workspace($this->dir, we_users_util::get_ws($this->table), $this->table);
		if($isWS && $this->id == 0){
			$this->path = '/';
			$this->dir = 0;
		}
		$this->jsCmd->addCmd('clearEntries');
		if($isWS){
			$this->jsCmd->addCmd('updateSelectData', [
				'currentDir' => $this->dir,
				'parentID' => $this->values['ParentID'],
				'currentPath' => $this->path,
				'currentID' => $this->id,
			]);
		}
		$this->printCmdAddEntriesHTML();
		$this->jsCmd->addCmd('setButtons', [
			['NewFolderBut', $this->userCanMakeNewDir() && $this->userCanMakeNewFolder],
			['RootDirButs', /* $isWS && */ (intval($this->dir) !== intval($this->rootDirID))]
		]);

		if($isWS){
			$this->jsCmd->addCmd('unselectAllFiles');
		}
		$this->setWriteSelectorData();

		echo we_html_tools::getHtmlTop('', $this->jsCmd->getCmds(), we_html_element::body());
		$_SESSION['weS']['we_fs_lastDir'][$this->table] = $this->dir;
	}

	protected function printCreateFolderHTML(): void{
		$this->FolderText = rawurldecode($this->FolderText);
		$txt = $this->FolderText;
		if($this->table === OBJECT_FILES_TABLE){
			$folder = new we_folder_objectFile();
			$folder->initByPath($this->path . '/' . $txt, OBJECT_FILES_TABLE);
		} else {
			$folder = we_folder_base::instance($this->table);
			$folder->we_new($this->table, $this->dir, $txt);
		}

		if(!($msg = $folder->checkFieldsOnSave())){
			$folder->we_save();
		}
		$this->jsCmd->addCmd('clearEntries');
		if($msg){
			$this->jsCmd->addMsg($msg, we_base_util::WE_MESSAGE_ERROR);
		} else {
			$this->jsCmd->addCmd('makeNewTreeEntry', [
				'id' => $folder->ID,
				'parentid' => $folder->ParentID,
				'text' => $txt,
				'open' => 1,
				'contenttype' => $folder->ContentType,
				'table' => $this->table
			]);
			if($this->canSelectDir){
				$this->jsCmd->addCmd('updateSelectData', [
					'currentPath' => $folder->Path,
					'currentID' => $folder->ID,
					'currentText' => $folder->getText(),
				]);
			}
		}

		$this->printCmdAddEntriesHTML();
		$this->jsCmd->addCmd('setButtons', [['NewFolderBut', $this->userCanMakeNewDir()]]);
		$this->jsCmd->addCmd('loadIfActive', $this->table);

		$this->setWriteSelectorData();
		echo we_html_tools::getHtmlTop('', $this->jsCmd->getCmds(), we_html_element::body());
	}

	protected function getFrameset(bool $withPreview = true): string{
		return '<body class="selector" onload="startFrameset();' . ($withPreview ? 'showPreview(' . $this->id . ');' : '') . '">' .
			we_html_element::div(['id' => 'fsheader'], $this->printHeaderHTML()) .
			we_html_element::iframe('fsbody', $this->getFsQueryString(we_selector_file::BODY), '', '', '', $withPreview ? 'preview' : '') .
			($withPreview ? we_html_element::div(['id' => 'fspreviewDiv']) : '') .
			we_html_element::div(['id' => 'fsfooter'], $this->printFooterTable()) .
			we_html_element::div(['id' => 'fspath', 'class' => 'radient']) .
			we_html_element::iframe('fscmd', 'about:blank') .
			'</body>';
	}

	protected function setFramesetJavaScriptOptions(): void{
		parent::setFramesetJavaScriptOptions();
		$this->jsoptions['options']['userCanMakeNewFolder'] = $this->userCanMakeNewFolder;
		$this->jsoptions['data']['makefolderState'] = $this->userCanMakeNewFolder;
	}

	protected function getPreviewContentRepresentation(string $ct, array $metainfos, array &$previewFields, array $folderFolders, array $folderFiles, array $data): void{
		switch($ct){
			case we_base_ContentTypes::IMAGE:
				$Title = ($metainfos['title'] ?? ((isset($metainfos['Title']) && !empty($metainfos['useMetaTitle'])) ? $metainfos['Title'] : ''));
				$name = ($metainfos['name'] ?? '');
				$alt = ($metainfos['alt'] ?? '');
				if($Title !== ''){
					$previewFields['attributes']['data'][] = [
						'caption' => g_l('weClass', '[Title]'),
						'content' => htmlspecialchars($Title)
					];
				}
				if($name){
					$previewFields['attributes']['data'][] = [
						'caption' => g_l('weClass', '[name]'),
						'content' => $name
					];
				}
				if($alt){
					$previewFields['attributes']['data'][] = [
						'caption' => g_l('weClass', '[alt]'),
						'content' => htmlspecialchars($alt)
					];
				}

			case we_base_ContentTypes::APPLICATION:
			case we_base_ContentTypes::PDF:

				$metaDataFields = we_metadata_metaData::getDefinedMetaDataFields();
				foreach($metaDataFields as $md){
					switch($md['tag']){
						case we_contents_root::TITLE_FIELD:
						case we_contents_root::DESCRIPTION_FIELD:
						case we_contents_root::KEYWORD_FIELD:
							break;
						default:
							if(isset($metainfos[$md['tag']])){
								$previewFields['metainfos']['data'][] = [
									'caption' => $md['tag'],
									'content' => $metainfos[$md['tag']]
								];
							}
					}
				}
				return;
//we need to set it explicitly
			case we_base_ContentTypes::FOLDER_COMMON:
			case we_base_ContentTypes::FOLDER_DOCUMENT:
			case we_base_ContentTypes::FOLDER_TEMPLATE:
			case we_base_ContentTypes::FOLDER_OBJECTFILE:
			case we_base_ContentTypes::FOLDER_VFILE:
				if(!empty($folderFolders)){
					foreach($folderFolders as $fId => $fxVal){
						$previewFields['folders']['data'][] = [
							'content' => '<a href="javascript:WE().util.dialogTop(window).selectorOnDblClick(' . $fId . ');">' . $fxVal . '</a><a href="javascript:WE().layout.openToEdit(\'' . $this->table . '\',\'' . $fId . '\')" class="edit"><div><i class="fa fa-lg fa-edit"></i></div></a>'
						];
					}
				}
				foreach($folderFiles as $fId => $fxVal){
					$previewFields['files']['data'][] = [
//we need to set it explicitly
						'content' => '<a href="javascript:WE().util.dialogTop(window).selectorOnDblClick(' . $fId . ');">' . $fxVal . '</a><a href="javascript:WE().layout.openToEdit(\'' . $this->table . '\',\'' . $fId . '\')" class="edit"><div><i class="fa fa-lg fa-edit"></i></div></a>'
					];
				}
				return;

			case we_base_ContentTypes::TEMPLATE:
				if(isset($data['MasterTemplateID']) && !empty($data['MasterTemplateID'])){
					$mastertemppath = fS('SELECT Path FROM ' . $this->db->escape($this->table) . ' WHERE ID=' . intval($data['MasterTemplateID']));
					$previewFields['masterTemplate']["data"][] = [
						'caption' => 'ID',
						'content' => '<a href="javascript:WE().layout.openToEdit(\'' . $this->table . '\',\'' . $data['MasterTemplateID'] . '\')" class="edit"><div><i class="fa fa-lg fa-edit"></i></div></a>'
					];
					$previewFields['masterTemplate']['data'][] = [
						'caption' => g_l('weClass', '[path]'),
						'content' => $mastertemppath
					];
				}
				return;
		}
	}

	protected function showPreviewFolderFiles(array $data): bool{
		return false;
	}

//remove old thumb
	protected function printPreviewHTML(): void{
		$resp = new we_rpc_response();
		$rpcView = new we_rpc_genericJSONView('json');
		if(!$this->id){
			$resp->setStatus(false);
			echo $rpcView->getResponse($resp);
			return;
		}
		$data = getHash('SELECT * FROM ' . $this->table . ' WHERE ID=' . intval($this->id));
		$path = $data ? $data['Path'] : '';
		$metainfos = [];
		if($this->showPreviewFolderFiles($data)){
			$wsQuery = we_users_util::get_ws_paths($this->table);
			$owner = we_users_util::makeOwnersSql(false);
			$folderFolders = $this->db->getAllFirstq('SELECT ID,Text FROM ' . $this->db->escape($this->table) . ' WHERE IsFolder=1 AND ParentID=' . intval($this->id) . ($wsQuery ? ' AND (' . $wsQuery . ')' : '') . $owner . ' ORDER BY (Text REGEXP "^[0-9]") DESC', false);
			$folderFiles = $this->db->getAllFirstq('SELECT ID,Text FROM ' . $this->db->escape($this->table) . ' WHERE IsFolder=0 AND ParentID=' . intval($this->id) . ($wsQuery ? ' AND (' . $wsQuery . ')' : '') . $owner . ' ORDER BY (Text REGEXP "^[0-9]") DESC', false);
		} else {
			$folderFolders = $folderFiles = [];
			switch($this->table){
				case VFILE_TABLE:
					$folderFiles = $this->db->getAllFirstq('SELECT f.ID,f.Text FROM ' . FILELINK_TABLE . ' fl JOIN ' . FILE_TABLE . ' f ON (f.ID=fl.remObj AND fl.remTable="tblFile") WHERE fl.DocumentTable="' . stripTblPrefix(VFILE_TABLE) . '" AND fl.type="collection" AND fl.ID=' . intval($this->id) . ' ORDER BY IsFolder,(Text REGEXP "^[0-9]") DESC', false);
					$data['ContentType'] = we_base_ContentTypes::COLLECTION;
					break;
				case FILE_TABLE:
					$metainfos = $this->db->getAllFirstq('SELECT c.Name, c.Dat FROM ' . CONTENT_TABLE . ' c WHERE c.DID=' . intval($this->id) . ' AND c.DocumentTable="' . stripTblPrefix(FILE_TABLE) . '"', false);
					break;
				case OBJECT_FILES_TABLE :
					$fieldnames = getHash('SELECT DefaultDesc,DefaultTitle,DefaultKeywords FROM ' . OBJECT_TABLE . ' WHERE ID=' . intval($data['TableID']));
					$selFields = [];
					foreach($fieldnames as $key => $val){
						if(!$val || $val === '_'){ // bug #4657
							continue;
						}
						switch($key){
							case 'DefaultDesc':
								$selFields[] = '`' . $val . '` AS Description';
								break;
							case 'DefaultTitle':
								$selFields[] = '`' . $val . '` AS Title';
								break;
							case 'DefaultKeywords':
								$selFields[] = '`' . $val . '` AS Keywords';
								break;
						}
					}
					if($selFields){
						$metainfos = getHash('SELECT ' . implode(',', $selFields) . ' FROM ' . OBJECT_X_TABLE . intval($data['TableID']) . ' obx WHERE obx.OF_ID=' . intval($data['ID']));
					}
			}
		}

		$fs = file_exists(WE_DOCUMENT_ROOT . $path) ? filesize(WE_DOCUMENT_ROOT . $path) ?: 0 : 0;
		$filesize = we_base_file::getHumanFileSize($fs);

		$showPreview = false;
		$imagesize = [];
		$imagepreview = '';
		switch($data['ContentType']){
			case we_base_ContentTypes::IMAGE:
				if($fs == 0){
					$imagesize = [0, 0];
					$imagepreview = '<img src="' . ICON_DIR . 'no_image.gif' . '" id="previewpic"><p>' . g_l('fileselector', '[image_not_uploaded]') . '</p>';
				} else {
					$imagesize = getimagesize(WE_DOCUMENT_ROOT . $data['Path']);
					$imagepreview = "<a href='" . $data['Path'] . "' target='_blank'><img src='" . WEBEDITION_DIR . 'we_cmd.php?we_cmd[0]=we_base_showThumbnail&' . http_build_query([
							'id' => $this->id,
							'size' => ['width' => '150', 'height' => '200'],
							'path' => str_replace(WE_DOCUMENT_ROOT, '', $data['Path']),
							'extension' => $data['Extension'],]) . "' id='previewpic'></a>";
				}
			case we_base_ContentTypes::WEDOCUMENT:
			case we_base_ContentTypes::HTML:
			case we_base_ContentTypes::APPLICATION:
			case we_base_ContentTypes::PDF:
				$showPreview = $data['IsPublished'];
				break;
		}

		$previewFields = [
			'metainfos' => [
				'headline' => g_l('weClass', '[metainfo]'),
				'data' => []
			],
			'attributes' => [
				'headline' => g_l('weClass', '[attribs]'),
				'data' => []
			],
			'folders' => ['headline' => g_l('fileselector', '[folders]'), 'data' => []],
			'files' => ['headline' => g_l('fileselector', '[files]'), 'data' => []],
			'masterTemplate' => ['headline' => g_l('weClass', '[master_template]'), 'data' => []],
			'properties' => [
				'headline' => g_l('weClass', '[tab_properties]'),
				'data' => [
					$path ? [
					'caption' => g_l('fileselector', '[name]'),
					'content' => (
					$showPreview ?
					'<a href="' . $path . '" target="_blank" style="color:black"><i style="margin-right:4px" class="fa fa-external-link fa-lg"></i>' . $data['Text'] . '</a>' :
					$data['Text']
					)
					] : [],
					[
						'caption' => 'ID',
						'content' => $this->id . '<a href="javascript:WE().layout.openToEdit(\'' . $this->table . '\',\'' . $this->id . '\',\'' . $data['ContentType'] . '\')" class="edit"><div><i class="fa fa-edit fa-lg"></i></div></a>'
					]
				]],
		];
		if(!empty($data['Creation_Date'])){
			$previewFields['properties']['data'][] = ['caption' => g_l('fileselector', '[created]'),
				'content' => $data['Creation_Date']
			];
		}

		if(!empty($data['Mod_Date'])){
			$previewFields['properties']['data'][] = [
				'caption' => g_l('fileselector', '[modified]'),
				'content' => $data['Mod_Date']
			];
		}

		try{
			$ct = g_l('contentTypes', '[' . $data['ContentType'] . ']', true);
		} catch (Exception $e){
			$ct = $data['ContentType'];
		}
		$previewFields['properties']['data'][] = [
			'caption' => g_l('fileselector', '[type]'),
			'content' => $ct
		];

		if(!empty($data['Language'])){
			$langs = we_base_country::getWeFrontendLanguagesForBackend();
			$previewFields['properties']['data'][] = [
				'caption' => g_l('weClass', '[language]'),
				'content' => $data['Language'] ? $langs[$data['Language']] ?? '-' : '-'
			];
		}

		if($data['ContentType'] === we_base_ContentTypes::IMAGE){
			$previewFields['properties']['data'][] = [
				'caption' => g_l('weClass', '[width]') . ' x ' . g_l('weClass', '[height]'),
				'content' => $imagesize[0] . ' x ' . $imagesize[1] . ' px '
			];
		}

		switch($data['ContentType']){
			case we_base_ContentTypes::FOLDER_COMMON:
			case we_base_ContentTypes::FOLDER_DOCUMENT:
			case we_base_ContentTypes::FOLDER_TEMPLATE:
			case we_base_ContentTypes::FOLDER_OBJECTFILE:
			case we_base_ContentTypes::FOLDER_VFILE:
			case we_base_ContentTypes::TEMPLATE:
			case we_base_ContentTypes::OBJECT:
			case we_base_ContentTypes::OBJECT_FILE:
				break;
			default:
				$previewFields['properties']['data'][] = ['caption' => g_l('fileselector', '[filesize]'),
					'content' => $filesize
				];
		}

		if(isset($metainfos[we_contents_root::TITLE_FIELD])){
			$previewFields['metainfos']['data'][] = [
				'caption' => g_l('weClass', '[Title]'),
				'content' => $metainfos[we_contents_root::TITLE_FIELD]
			];
		}

		if(isset($metainfos[we_contents_root::DESCRIPTION_FIELD])){
			$previewFields['metainfos']['data'][] = [
				'caption' => g_l('weClass', '[Description]'),
				'content' => $metainfos[we_contents_root::DESCRIPTION_FIELD]
			];
		}

		if(isset($metainfos[we_contents_root::KEYWORD_FIELD])){
			$previewFields['metainfos']['data'][] = [
				'caption' => g_l('weClass', '[Keywords]'),
				'content' => $metainfos[we_contents_root::KEYWORD_FIELD]
			];
		}
		$this->getPreviewContentRepresentation($data['ContentType'], $metainfos, $previewFields, $folderFolders, $folderFiles, $data);

		$resp->setStatus(true);
		$resp->setData('path', $path);
		if($data['ContentType'] === we_base_ContentTypes::IMAGE){
			$resp->setData('imgPreview', $imagepreview);
		}
		$resp->setData('previewFields', $previewFields);
		echo $rpcView->getResponse($resp);

		return;
	}

}
