<?php
declare(strict_types=1);

/* webEdition CMS
 *
 * $Rev: 0627c1498cde $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2020-07-02 12:07 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */

abstract class we_tag_captcha extends we_weTag_tag{

	public static function parse(array $attribs, string $content): string{
		self::attributeMissing($attribs, ['width' => false, 'height' => false], __CLASS__);
		$templateID = we_base_WE::$currentParsedTemplate;

		$width = self::getParserAttribute('width', $attribs, 100, we_base_request::INT);
		$height = self::getParserAttribute('height', $attribs, 25, we_base_request::INT);
		$path = self::getParserAttribute('path', $attribs, '/', we_base_request::FILE);

		$maxlength = self::getParserAttribute('maxlength', $attribs, 5, we_base_request::INT);
		$type = self::getParserAttribute('type', $attribs, 'gif', we_base_request::STRING);

//		$font = self::getParserAttribute('font', $attribs);
		$fontsize = self::getParserAttribute('fontsize', $attribs, 14, we_base_request::INT);
		$fontcolor = self::getParserAttribute('fontcolor', $attribs, '#000000', we_base_request::STRING);

		$angle = self::getParserAttribute('angle', $attribs, 0, we_base_request::INT);

		$subset = self::getParserAttribute('subset', $attribs, 'alphanum', we_base_request::STRING);
		$case = self::getParserAttribute('case', $attribs, 'mix', we_base_request::STRING);
		$skip = self::getParserAttribute('skip', $attribs, 'i,I,l,L,0,o,O,1,g,9', we_base_request::STRING);

		$valign = self::getParserAttribute('valign', $attribs, 'random', we_base_request::STRING);
		$align = self::getParserAttribute('align', $attribs, 'random', we_base_request::STRING);

		$bgcolor = self::getParserAttribute('bgcolor', $attribs, '#ffffff', we_base_request::STRING);
		$transparent = self::getParserAttribute('transparent', $attribs, false, we_base_request::BOOL);

		$style = self::getParserAttribute('style', $attribs, '', we_base_request::STRING);
		$stylecolor = self::getParserAttribute('stylecolor', $attribs, '#cccccc', we_base_request::STRING);
		$stylenumber = self::getParserAttribute('stylenumber', $attribs, '5,10', we_base_request::STRING);

		// writing the temporary document
		$file = 'we_captcha_' . $templateID . '.php';
		$realPath = rtrim(realpath(WE_DOCUMENT_ROOT . $path) ?: '', '/') . '/' . $file;
		we_base_file::deleteLocalFile($realPath);


		DB_WE::$inst->query('REPLACE INTO ' . CAPTCHADEF_TABLE . ' SET ' . we_database_base::arraySetter([
				'ID' => $templateID,
				'width' => $width,
				'height' => $height,
				'maxlength' => $maxlength,
				'fontsize' => $fontsize,
				'fontcolor' => $fontcolor,
				'subset' => $subset,
				'case' => $case,
				'skip' => $skip,
				'align' => $align,
				'valign' => $valign,
				'bgcolor' => $bgcolor,
				'transparent' => (string) ((int) ($bgcolor && $transparent)),
				'style' => $style,
				'stylecolor' => $stylecolor,
				'stylenumber' => $stylenumber,
				'angle' => $angle,
				'type' => $bgcolor && $transparent ? 'gif' : $type
		]));

		return '<?= ' . we_weTag_parser::printTag('captcha', ['id' => $templateID]) . ';?>';
	}

	public static function tag(array $attribs = [], string $content = '', bool $internal = false): string{
		static $captcha = [];
		try{
			self::$doc = we_base_WE::getWeDoc();
		} catch (we_exception_base $e){

		}
		$tid = !self::$doc ? (empty($attribs['src']) ? 0 : intval($attribs['src'])) : self::$doc->TemplateID;
		if(!isset($attribs['alt'])){
			$attribs['alt'] = 'captcha';
		}

		if(!$tid){
			return '';
		}

		if(self::$doc && !self::$doc->IsDynamic){
			//static documents link to dynamic source
			$attribs['src'] = WEBEDITION_DIR . 'showCaptcha.php?id=' . $tid;
			return self::htmlTag('img', $attribs);
		}

		if(empty($captcha[$tid])){
			$attr = getHash('SELECT * FROM ' . CAPTCHADEF_TABLE . ' WHERE ID=' . $tid) ?: getHash('SELECT * FROM ' . CAPTCHADEF_TABLE . '  LIMIT 1');

			$image = new we_captcha_image($attr['width'], $attr['height'], $attr['maxlength']);
			$image->setFont($attr['fontsize'], $attr['fontcolor']);
			$image->setCharacterSubset($attr['subset'], $attr['case'], $attr['skip']);
			$image->setAlign($attr['align']);
			$image->setVerticalAlign($attr['valign']);
			$image->setBackground($attr['bgcolor'], (bool) $attr['transparent']);
			$image->setStyle($attr['style'], $attr['stylecolor'], $attr['stylenumber']);
			$image->setAngleRange($attr['angle']);
			if(!self::$doc){ //called via we_showCaptcha
				return we_captcha_captcha::display($image, $attr['type']);
			}

			[$type, $data] = we_captcha_captcha::get($image, $attr['type']);
			$captcha[$tid] = $type . ';base64,' . base64_encode($data);
		}
		$attribs['src'] = 'data:' . $captcha[$tid];
		return self::htmlTag('img', $attribs);
	}

}
