<?php
declare(strict_types=1);

/* webEdition CMS
 *
 * $Rev: 2fcfd49ac317 $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2020-07-06 14:53 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */

abstract class we_tag_formfield extends we_weTag_tag{
	/** @phan-var array<string,string> */
	protected static $availTypes = [];

	/**
	 * @param array<string,mixed> $attribs parsed tag-attributes
	 * @param string $content content between openingtag and closing tag
	 * @param bool $internal if used by we internally
	 * @return string output to print
	 * @throws we_exception_parser if any attribute is missing
	 */
	public static function tag(array $attribs = [], string $content = '', bool $internal = false): string{
		self::attributeMissing($attribs, ['name' => false], __CLASS__);

		$name = self::getAttribute('name', $attribs, '', we_base_request::STRING);

		$types = self::getAttribute('type', $attribs, ['textinput'], we_base_request::STRING_LIST);
		$attrs = self::getAttribute('attribs', $attribs, [], we_base_request::STRING_LIST);
		self::$doc = we_base_WE::getWeDoc();

		$type_sel = self::$doc->getElementFromKey($name, 'fftype') ?: ($types ? $types[0] : 'textinput');

		$ff = [];

		// here add some mandatory fields
		$mandatoryFields = [];
		$m = [];
		$removeAttribs = ['doc', 'type', 'attribs'];
		$attrs = array_merge($attrs, $mandatoryFields);

		//check tag attributes as defaults
		foreach($attribs as $k => $v){
			if(preg_match('/^([^_]+)_([^_]+)$/', $k, $m)){
				$removeAttribs[] = $k;
				if($m[1] === $type_sel){
					if(in_array($k, $attrs)){
						$ff[$m[2]] = [
							'value' => self::$doc->getElementFromKey($name, 'ff_' . $type_sel . '_' . $m[2]),
							'default' => $v,
							'change' => 0
						];
					} else {
						$ff[$m[2]] = [
							'change' => 0,
							'default' => $v
						];
					}
				}
			}
		}

		//check attribs attribute for changeable attributes
		foreach($attrs as $a){
			'@phan-var string $a';
			if(preg_match('/^([^_]+)_([^_]+)$/', $a, $m) && ($m[1] == $type_sel)){
				$ff[$m[2]]['change'] = 1;
				if(($t = self::$doc->getElementFromKey($name, 'ff_' . $type_sel . '_' . $m[2]))){
					$ff[$m[2]]['value'] = $t;
				}
			}
		}

		if(we_base_WE::$editmode){
			return self::edit($name, $types, $type_sel, $ff);
		}

		$tagAtts = self::removeAttribs($attribs, $removeAttribs);

		foreach(array_keys($ff) as $f){
			if(!((($f === 'value') && ($type_sel === 'textarea')) || $f === 'type')){
				if(($val = self::$doc->getElementFromKey($name, 'ff_' . $type_sel . '_' . $f))){
					$tagAtts[$f] = htmlspecialchars($val);
				}
			}
		}
		$tagAtts['name'] = htmlspecialchars(self::$doc->getElementFromKey($attribs['name'], 'ffname'));

		switch($type_sel){
			case 'checkbox':
			case 'radio':
				return self::radioCheck($type_sel, $name, $tagAtts);
			default:
			case 'textinput': // correct input type="text"
				return self::textinput($tagAtts);
			case 'textarea':
				return self::textarea($tagAtts, $ff);
			case 'select':
				return self::select($name, $tagAtts);
			case 'country':
				return self::country($name, $attribs, $attrs, $tagAtts);
			case 'language':
				return self::language($name, $attribs, $attrs, $tagAtts);
			case 'file':
				return self::file($tagAtts, $name);
		}
	}

	protected static function country(string $name, array $attribs, array $attrs, array $tagAtts): string{
		$orgVal = self::$doc->getElementFromKey($name, 'ffdefault');
		$docAttr = self::getAttribute('doc', $attribs, 'self', we_base_request::STRING);
		$doc = self::getDocForTag($docAttr);
		$lang = $doc->Language;
		$langcode = ($lang ? substr($lang, 0, 2) : (array_search(we_base_WE::$Language, we_base_country::getWELangs(), true) ?: 'de'));

		$topCountries = array_flip(explode(',', WE_COUNTRIES_TOP));
		foreach($topCountries as $countrykey => &$countryvalue){
			$countryvalue = we_base_country::getTranslation((string) $countrykey, we_base_country::TERRITORY, $langcode);
		}
		unset($countryvalue);
		$shownCountries = array_flip(explode(',', WE_COUNTRIES_SHOWN));
		foreach($shownCountries as $countrykey => &$countryvalue){
			$countryvalue = we_base_country::getTranslation((string) $countrykey, we_base_country::TERRITORY, $langcode);
		}
		unset($countryvalue);
		$oldLocale = setlocale(LC_ALL, NULL);
		setlocale(LC_ALL, $lang . '.UTF-8');
		asort($topCountries, SORT_LOCALE_STRING);
		asort($shownCountries, SORT_LOCALE_STRING);
		if($oldLocale){
			setlocale(LC_ALL, $oldLocale);
		}
		$tagContent = (WE_COUNTRIES_DEFAULT == '' ? '' : '<option value="--" ' . ($orgVal === '--' ? ' selected="selected">' : '>') . WE_COUNTRIES_DEFAULT . '</option>');

		foreach($topCountries as $countrykey => $countryvalue){
			$tagContent .= '<option value="' . $countrykey . '" ' . ($orgVal == $countrykey ? ' selected="selected">' : '>') . $countryvalue . '</option>';
		}
		$tagContent .= '<option value="-" disabled="disabled">----</option>';
		foreach($shownCountries as $countrykey => $countryvalue){
			$tagContent .= '<option value="' . $countrykey . '" ' . ($orgVal == $countrykey ? ' selected="selected">' : '>') . $countryvalue . '</option>';
		}
		$tagAtts['size'] = ($attrs['size'] ?? 1);
		return self::htmlTag('select', $tagAtts, $tagContent);
	}

	protected static function language(string $name, array $attribs, array $attrs, array $tagAtts): string{
		$orgVal = self::$doc->getElementFromKey($name, 'ffdefault');
		$docAttr = self::getAttribute('doc', $attribs, 'self', we_base_request::STRING);
		$doc = self::getDocForTag($docAttr);
		$lang = $doc->Language;
		$langcode = ($lang ? substr($lang, 0, 2) : array_search(we_base_WE::$Language, we_base_country::getWELangs(), true)) ?: 'de';
		$frontendL = we_base_WE::$Langs['FrontendLanguages'];
		foreach($frontendL as &$lcvalue){
			$lccode = explode('_', $lcvalue);
			$lcvalue = $lccode[0];
		}
		unset($lcvalue);
		$frontendLL = [];
		foreach($frontendL as $lcvalue){
			$frontendLL[$lcvalue] = we_base_country::getTranslation($lcvalue, we_base_country::LANGUAGE, $langcode);
		}

		$oldLocale = setlocale(LC_ALL, NULL);
		setlocale(LC_ALL, $lang . '.UTF-8');
		asort($frontendLL, SORT_LOCALE_STRING);
		if($oldLocale){
			setlocale(LC_ALL, $oldLocale);
		}
		$tagContent = '';
		foreach($frontendLL as $langkey => $langvalue){
			$tagContent .= '<option value="' . $langkey . '" ' . ($orgVal == $langkey ? ' selected="selected">' : '>') . $langvalue . '</option>';
		}
		$tagAtts['size'] = ($attrs['size'] ?? 1);

		return self::htmlTag('select', $tagAtts, $tagContent);
	}

	protected static function textarea(array $tagAtts, array $ff): string{
		$tagContent = (isset($ff['value']) ? htmlspecialchars($ff['value']['value']) : '');

		if(!array_key_exists('cols', $tagAtts)){
			$tagAtts['cols'] = 20;
		}
		if(!array_key_exists('rows', $tagAtts)){
			$tagAtts['rows'] = 5;
		}
		return self::htmlTag('textarea', $tagAtts, $tagContent, true);
	}

	protected static function select(string $name, array $tagAtts): string{
		$selected = self::$doc->getElementFromKey($name, 'ffdefault');
		$foo = explode('<_BR_>', strtr(self::$doc->getElementFromKey($name, 'ffvalues'), ["\r\n" => '<_BR_>', "\r" => '<_BR_>', "\n" => '<_BR_>',]));
		$tagContent = '';
		foreach($foo as $v){
			$atts = ['value' => htmlspecialchars(trim($v))];
			if($selected == $v){
				$atts['selected'] = 'selected';
			}
			$tagContent .= self::htmlTag('option', $atts, htmlspecialchars($v));
		}
		return self::htmlTag('select', $tagAtts, $tagContent, true);
	}

	protected static function radioCheck(string $type_sel, string $name, array $tagAtts): string{
		$tagAtts['type'] = $type_sel;
		if(self::$doc->getElementFromKey($name, 'ffchecked')){
			$tagAtts['checked'] = 'checked';
		}
		return self::htmlTag('input', $tagAtts);
	}

	protected static function textinput(array $tagAtts): string{
		$tagAtts['type'] = 'text';
		if(isset($tagAtts['size'])){
			$tagAtts['style'] = (empty($tagAtts['style']) ? '' : $tagAtts['style']) . 'width:' . $tagAtts['size'] . 'em;';
			unset($tagAtts['size']);
		}

		return self::htmlTag('input', $tagAtts);
	}

	protected static function file(array $tagAtts, string $name): string{
		return self::htmlTag('input', $tagAtts) .
			self::htmlTag('input', [
				'type' => 'hidden',
				'name' => 'MAX_FILE_SIZE',
				'value' => htmlspecialchars(self::$doc->getElementFromKey($name, 'ffmaxfilesize')),
		]);
	}

	protected static function setTypeNames(): void{
		self::$availTypes = [
			'checkbox' => g_l('modules_object', '[field][checkbox]'),
			'country' => g_l('modules_object', '[field][country]'),
			'file' => g_l('modules_object', '[field][binary]'),
			'language' => g_l('modules_object', '[field][language]'),
			'radio' => 'Radio',
			'select' => g_l('modules_object', '[field][meta]'),
			'textarea' => g_l('modules_object', '[field][textarea]'),
			'textinput' => g_l('modules_object', '[field][input]'),
		];
	}

	/**
	 *
	 * @staticvar boolean $availTypes
	 * @param string $name
	 * @param array $types
	 * @param string $type_sel
	 * @param array<string,array<string,mixed>> $ff
	 * @return string
	 */
	protected static function edit(string $name, array $types, string $type_sel, array $ff): string{
		if(!self::$availTypes){
			self::setTypeNames();
		}
		$ffname = self::$doc->getElementFromKey($name, 'ffname');
		$nameprefix = 'we_' . self::$doc->Name . '_txt[' . $name . '#';

		$tmp_select = '';
		foreach($types as $k){
			if(isset(self::$availTypes[$k])){
				$tmp_select .= '<option value="' . $k . '"' . (($k == $type_sel) ? ' selected="selected"' : '') . '>' . self::$availTypes[$k] . '</option>';
			}
		}

		$tbl = '<table class="we we_formfield weEditTable weEditmodeStyle"><tbody>
	<tr>
		<td>' . g_l('global', '[name]') . ':</td>
		<td><input type="text" name="' . $nameprefix . 'ffname]" style="width:20em" value="' . $ffname . '" required="required" pattern="' . we_base_constants::CHARS_PATTERN_BASIC . '" onblur="checkFormFields(this);" title="' . g_l('global', '[allowedChars][basic]') . '"/></td>
	</tr>
	<tr>
		<td>' . g_l('global', '[type]') . ':</td>
		<td><select name="' . $nameprefix . 'fftype]" onchange="setScrollTo();we_cmd(\'reload_editpage\');">' . $tmp_select . '</select></td>
	</tr>';

		if($ff){
			$tbl .= '	<tr>
		<td>' . g_l('global', '[attributes]') . ':</td>
		<td><table class="weEditTable">';

			foreach($ff as $f => $m){
				$tbl .= '<tr><td style="color: black; margin-left:5px;"><span class="small bold">' . g_l('weTag', '[formfield][' . $f . ']', true) . '(' . $f . ')' . ':</span></td><td>';
				$val = $m['value'] ?? '';

				$default = isset($m['default']) ? array_filter(explode(',', $m['default'])) : [];

				if($m['change'] == 1){
					if(count($default) > 1){
						$valselect = '<select name="' . $name . 'tmp" onchange="this.form.elements[\'' . $nameprefix . 'ff_' . $type_sel . '_' . $f . ']\'].value=this.options[this.selectedIndex].value;">' .
							'<option value=""></option>';
						foreach($default as $v){
							$valselect .= '<option value="' . $v . '">' . $v . '</option>';
						}
						$valselect .= '</select>';
					} else {
						$valselect = '';
					}
					if((!isset($m['value'])) && count($default) == 1){
						$val = $default[0];
					}
					$tbl .= '<input type="text" name="' . $nameprefix . 'ff_' . $type_sel . '_' . $f . ']" style="width:20em" ' . ($val ? ' value="' . $val . '"' : '') . ' />' . $valselect;
				} else {
					if(count($default) > 1){
						$val = self::$doc->getElementFromKey($name, 'ff_' . $type_sel . '_' . $f);
						if(count($default) > 1){
							$valselect = '<select name="' . $nameprefix . 'ff_' . $type_sel . '_' . $f . ']">';
							foreach($default as $v){
								$valselect .= '<option value="' . $v . '"' . (($v == $val) ? ' selected="selected"' : "") . '>' . $v . '</option>';
							}
							$valselect .= '</select>';
							$tbl .= $valselect;
						}
					} else {
						$foo = empty($default) ? '' : $default[0];
						$tbl .= $foo . '<input type="hidden" name="' . $nameprefix . 'ff_' . $type_sel . '_' . $f . ']" value="' . $foo . '" />';
					}
				}
				$tbl .= '</td></tr>';
			}
			$tbl .= '</table>
		</td>
	</tr>';
		}

		switch($type_sel){
			case 'select':
				$tbl .= '	<tr>
		<td>' . g_l('global', '[values]') . ':</td>
		<td><textarea name="' . $nameprefix . 'ffvalues]" style="width:20em;height:5ex;">' . self::$doc->getElementFromKey($name, 'ffvalues') . '</textarea></td>
	</tr>
	<tr>
		<td>' . g_l('global', '[default]') . ':</td>
		<td><input type="text" name="' . $nameprefix . 'ffdefault]" style="width:20em" value="' . self::$doc->getElementFromKey($name, 'ffdefault') . '" /></td>
	</tr>';
				break;
			case 'file':
				$tbl .= '	<tr>
		<td>' . g_l('global', '[max_file_size]') . ':</td>
		<td><input type="text" name="' . $nameprefix . 'ffmaxfilesize]" value="' . self::$doc->getElementFromKey($name, 'ffmaxfilesize') . '" /></td>
	</tr>';
				break;
			case 'radio':
			case 'checkbox':
				$tbl .= '	<tr>
		<td>' . g_l('global', '[checked]') . ':</td>
		<td><select name="' . $nameprefix . 'ffchecked]"><option value="0"' . (self::$doc->getElementFromKey($name, 'ffchecked') ? "" : " selected") . '>' . g_l('global', '[no]') . '</option><option value="1"' . (self::$doc->getElementFromKey($name, 'ffchecked') ? " selected" : "") . '>' . g_l('global', '[yes]') . '</option></select></td>
	</tr>';
				break;
		}
		$tbl .= '</tbody></table>';

		return $tbl;
	}

}
