<?php
declare(strict_types=1);

/* webEdition CMS
 *
 * $Rev: b26067fc7281 $
 * $Author: Marc Kraemer <marc.kraemer@webedition.org> $
 * $Date: 2020-02-04 13:57 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */

abstract class we_tag_ifDocument extends we_weTag_ifTag{

	public static function tag(array $attribs = [], string $content = '', bool $internal = false): bool{
		$type = self::getAttribute('doc', $attribs, self::getAttribute('type', $attribs, '', we_base_request::STRING, true), we_base_request::STRING);
		$ids = self::getAttribute('id', $attribs, [], we_base_request::INTLISTA);
		$doctypes = self::getAttribute('doctype', $attribs, [], we_base_request::STRING_LIST);
		$parentids = self::getAttribute('parentid', $attribs, [], we_base_request::INTLISTA);
		$contenttypes = self::getAttribute('contenttype', $attribs, [], we_base_request::STRING_LIST);
		$included = self::getAttribute('included', $attribs, '', we_base_request::BOOL);
		self::$doc = we_base_WE::getWeDoc();

		$we_doc = [];
		switch($type){
			case 'listview':
				switch(true){
					case (we_tag_listview::$current instanceof we_listview_document):
						$we_doc['id'] = we_tag_listview::$current->getDID();
						break;
					case (we_tag_listview::$current instanceof we_listview_search):
						$we_doc['id'] = we_tag_listview::$current->f('did');
						break;
					default:
						return false;
				}
				$we_doc['parentid'] = (int) we_tag_listview::$current->f(we_listview_base::PARENTID_FIELD);
				$we_doc['contenttype'] = we_tag_listview::$current->f(we_listview_base::CONTENTTYPE_FIELD);
				$we_doc['doctype'] = (int) we_tag_listview::$current->f(we_listview_base::DOCTYPE_FIELD);
				$we_doc['isIncluded'] = true;
				break;
			case 'self':
				$we_doc = [
					'id' => self::$doc->ID,
					'parentid' => self::$doc->ParentID,
					'contenttype' => self::$doc->ContentType,
					'doctype' => self::$doc->DocType,
					'isIncluded' => intval(we_base_WE::$mainDoc['doc']->ID) !== intval(self::$doc->ID),
				];
				break;
			default:
				$we_doc = [
					'id' => we_base_WE::$mainDoc['doc']->ID,
					'parentid' => we_base_WE::$mainDoc['doc']->ParentID,
					'contenttype' => we_base_WE::$mainDoc['doc']->ContentType,
					'doctype' => we_base_WE::$mainDoc['doc']->DocType,
					'isIncluded' => false,
				];
		}

		return $we_doc['contenttype'] !== we_base_ContentTypes::OBJECT_FILE &&
			(!$ids || in_array($we_doc['id'], $ids)) &&
			(!$contenttypes || in_array($we_doc['contenttype'], $contenttypes, true)) &&
			(!$doctypes || in_array(trim(id2path($we_doc['doctype'], DOC_TYPES_TABLE), '/'), $doctypes, true)) &&
			(!$parentids || in_array($we_doc['parentid'], $parentids)) &&
			($included === '' || ($included && $we_doc['isIncluded']) || (!$included && !$we_doc['isIncluded']));
	}

}

function we_tag_ifDocument(): bool{
	return we_tag_ifDocument::tag();
}
