<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: f642edb298bc $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2020-06-27 12:58 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
class we_tree_users extends we_tree_base{

	protected function customJSFile(): string{
		$startloc = (we_base_permission::hasPerm([we_base_perm::ADMINISTRATOR]) ?
			0 :
			fI('SELECT ParentID FROM ' . USER_TABLE . ' WHERE ID=' . we_users_user::getID())
			);

		return we_html_element::jsScript(JS_DIR . 'users_tree.js', 'initTree(' . $startloc . ');');
	}

	public static function getItems(int $ParentId, int $Offset = 0, int $Segment = 500, bool $sort = false): array{
		$items = [];
		$db = new DB_WE();
		if(we_base_permission::hasPerm([we_base_perm::USER_NEW, we_base_perm::USER_GROUP_NEW, we_base_perm::USER_SAVE, we_base_perm::USER_GROUP_SAVE, we_base_perm::USER_DELETE,
				we_base_perm::USER_GROUP_DELETE])){

			$parent_path = (we_base_permission::hasPerm([we_base_perm::ADMINISTRATOR]) ?
				'/' :
				str_replace('\\', '/', dirname(fS('SELECT Path FROM ' . USER_TABLE . ' WHERE ID=' . we_users_user::getID()))));

			foreach($db->getAllq('SELECT ID,ParentID,username,Type,Permissions,LoginDenied FROM ' . USER_TABLE . ' WHERE Path LIKE "' . $db->escape($parent_path) . '%" AND ParentID=' . $ParentId . ' ORDER BY username ASC') as $rec){
				switch($rec['Type']){
					case we_users_user::TYPE_USER_GROUP:
						$items[] = [
							'id' => intval($rec['ID']),
							'parentid' => intval($rec['ParentID']),
							'text' => addslashes($rec['username']),
							'typ' => 'group',
							'open' => 0,
							'contenttype' => we_base_ContentTypes::USERGROUP,
							'table' => USER_TABLE,
							'loaded' => 0,
							'checked' => false,
						];
						break;
					default:
						$p = we_unserialize($rec['Permissions']);

						$items[] = [
							'id' => intval($rec['ID']),
							'parentid' => intval($rec['ParentID']),
							'text' => addslashes($rec['username']),
							'typ' => 'item',
							'open' => 0,
							'contenttype' => ($rec['Type'] == we_users_user::TYPE_ALIAS ? we_base_ContentTypes::USERALIAS : we_base_ContentTypes::USER),
							'table' => USER_TABLE,
							'class' => (!empty($p[we_base_perm::ADMINISTRATOR]) ? 'bold ' : '') . ($rec['LoginDenied'] ? 'red' : '')
						];
				}
			}
		}
		return $items;
	}

}
