<?php
declare(strict_types=1);

/**

 * webEdition CMS
 *
 * $Rev: 4e4ff85098e9 $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2020-07-03 00:26 +0200 $
 */
class we_tw_ifField extends we_weTag_twbase{

	public function __construct(string $tagName){
		parent::__construct($tagName);
		$this->NeedsEndTag = true;
		$this->Groups = [self::GROUP_IF];
		$this->Description = g_l('weTag', '[' . $tagName . '][description]', true, true);

		$name = new we_weTag_textAttribute('name', true, '');
		$match = new we_weTag_textAttribute('match', true, '');
		$operator = new we_weTag_selectAttribute('operator', [
			new we_weTag_option('equal'),
			new we_weTag_option('less'),
			new we_weTag_option('less|equal'),
			new we_weTag_option('greater'),
			new we_weTag_option('greater|equal'),
			new we_weTag_option('contains'),
			new we_weTag_option('isin'),
			], false, '');
		$striphtml = new we_weTag_selectAttribute('striphtml', we_weTag_selectAttribute::getTrueFalse(true));
		$usekey = new we_weTag_selectAttribute('usekey', we_weTag_selectAttribute::getTrueFalse());

		if(we_modules_info::isActive(we_modules_info::SHOP)){
			$catfield = new we_weTag_selectAttribute('field', [
				new we_weTag_option('id'),
				new we_weTag_option('is_destinationprinciple'),
				new we_weTag_option('is_from doc_object'),
				new we_weTag_option('is_fallback_to_standard'),
				new we_weTag_option('is_fallback_to_active')
				], false, we_modules_info::SHOP);

			$options = [];
			$cats = we_shop_category::getShopCatsFromDir();
			foreach($cats as $cat){
				$options[] = new we_weTag_option((string) $cat->Path, (string) $cat->ID);
			}
			$catmatch = new we_weTag_selectAttribute('match', $options, false, we_modules_info::SHOP);
			$ignorefallbacks = new we_weTag_selectAttribute('ignorefallbacks', we_weTag_selectAttribute::getTrueFalse(), false, we_modules_info::SHOP);
			$vatfield = new we_weTag_selectAttribute('field', [new we_weTag_option('id'),
				new we_weTag_option('is_standard'),
				new we_weTag_option('is_fallback_to_standard'),
				new we_weTag_option('is_fallback_to_prefs'),
				new we_weTag_option('is_country_fallback_to_prefs')
				], false, we_modules_info::SHOP);

			$options = [];
			$vats = we_shop_vats::getAllShopVATs();
			foreach($vats as $vat){
				$options[] = new we_weTag_option($vat->vat . '% - ' . $vat->getNaturalizedText() . ' (' . $vat->territory . ')', $vat->id);
			}
			$vatmatch = new we_weTag_selectAttribute('match', $options, false, we_modules_info::SHOP);
			$shopVatAttributes = we_shop_category::isCategoryMode() ? [$vatfield, $vatmatch] : [$name, $match, $operator, $striphtml, $usekey];
		} else {
			$catfield = $catmatch = $ignorefallbacks = $shopVatAttributes = $vatfield = $vatmatch = null;
		}
		$format = new we_weTag_textAttribute('format');
		$num_format = new we_weTag_choiceAttribute('num_format', [
			new we_weTag_option('german'),
			new we_weTag_option('french'),
			new we_weTag_option('english'),
			new we_weTag_option('swiss'),
			], false, false);


		$this->TypeAttribute = new we_weTag_typeAttribute('type', [
			new we_weTag_option('text', false, '', [$name, $match, $operator, $striphtml, $usekey, $num_format], [$name, $match]),
			new we_weTag_option('date', false, '', [$name, $match, $operator, $striphtml, $usekey, $format], [$name, $match]),
			new we_weTag_option('img', false, '', [$name, $match, $operator, $striphtml, $usekey], [$name, $match]),
			new we_weTag_option('video', false, '', [$name, $match, $operator, $striphtml, $usekey], [$name, $match]),
			new we_weTag_option('audio', false, '', [$name, $match, $operator, $striphtml, $usekey], [$name, $match]),
			new we_weTag_option('href', false, '', [$name, $match, $operator, $striphtml, $usekey], [$name, $match]),
			new we_weTag_option('link', false, '', [$name, $match, $operator, $striphtml, $usekey], [$name, $match]),
			new we_weTag_option('day', false, '', [$name, $match, $operator, $striphtml, $usekey], [$name, $match]),
			new we_weTag_option('dayname', false, '', [$name, $match, $operator, $striphtml, $usekey], [$name, $match]),
			new we_weTag_option('month', false, '', [$name, $match, $operator, $striphtml, $usekey], [$name, $match]),
			new we_weTag_option('monthname', false, '', [$name, $match, $operator, $striphtml, $usekey], [$name, $match]),
			new we_weTag_option('year', false, '', [$name, $match, $operator, $striphtml, $usekey], [$name, $match]),
			new we_weTag_option('select', false, '', [$name, $match, $operator, $striphtml, $usekey], [$name, $match]),
			new we_weTag_option('binary', false, '', [$name, $match, $operator, $striphtml, $usekey], [$name, $match]),
			new we_weTag_option('float', false, '', [$name, $match, $operator, $striphtml, $usekey, $num_format], [$name, $match]),
			new we_weTag_option('int', false, '', [$name, $match, $operator, $striphtml, $usekey], [$name, $match]),
			new we_weTag_option('collection', false, '', [$name, $match, $operator, $striphtml, $usekey], [$name, $match]),
			new we_weTag_option('shopCategory', false, we_modules_info::SHOP, (we_modules_info::isActive(we_modules_info::SHOP) ? array_filter([$catfield, $catmatch, $ignorefallbacks]) : [
				]), []),
			new we_weTag_option('shopVat', false, we_modules_info::SHOP, $shopVatAttributes ?? [], we_modules_info::isActive(we_modules_info::SHOP) && !we_shop_category::isCategoryMode() ? [
				$name, $match] : []),
			new we_weTag_option('checkbox', false, '', [$name, $match, $operator, $striphtml, $usekey], [$name, $match]),
			], true, '');

		$this->Attributes = [$name, $match, $operator, $striphtml, $usekey];

		if(we_modules_info::isActive(we_modules_info::SHOP)){
			$this->Attributes = array_merge($this->Attributes, [$catfield, $catmatch, $ignorefallbacks, $vatfield, $vatmatch]);
		}
	}

}
