<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: 2fcfd49ac317 $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2020-07-06 14:53 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
class we_voting_frames extends we_modules_frame{
	const TAB_PROPERTIES = 1;
	const TAB_INQUIRY = 2;
	const TAB_OPTIONS = 3;
	const TAB_RESULT = 4;

	/** @var we_voting_view */
	protected $View;

	/** @var we_voting_tree */
	protected $Tree;

	public function __construct(){
		$this->module = we_modules_info::VOTING;
		parent::__construct();
		$this->Tree = new we_voting_tree($this->jsCmd);
		$this->View = new we_voting_view();
	}

	public function getHTML(string $what = '', int $mode = 0, int $step = 0): string{
		switch($what){
			case 'export_csv':
				return $this->getHTMLExportCsvMessage();
			case 'exportGroup_csv':
				return $this->getHTMLExportGroupCsvMessage();
			case 'reset_ipdata':
				return $this->getHTMLResetIPData();
			/* case "reset_logdata":
			  return $this->getHTMLResetLogData(); */
			case 'show_log':
				return $this->getHTMLShowLogNew();
			case 'delete_log':
				return $this->getHTMLDeleteLog();
			case 'frameset':
				$this->View->Model->clearSessionVars();
				return parent::getHTML($what, $mode, $step);
			case self::FOOTER:
				return $this->getHTMLEditorFooter([
						'main' => [
							we_html_button::FOOTER_SAVE => [[we_base_perm::VOTING_NEW, we_base_perm::VOTING_EDIT], ['save_voting']],
						],
						'optional' => [
							we_html_button::FOOTER_TRASH => [[we_base_perm::VOTING_DELETE], ['delete_voting']]
						]
				]);

			default:
				return parent::getHTML($what, $mode, $step);
		}
	}

	protected function getHTMLEditorHeader(): string{
		if(we_base_request::bool('home')){
			return parent::getHTMLEditorHeader();
		}

		$we_tabs = new we_gui_tabs();

		$we_tabs->addTab('', we_gui_tabs::WE_ICON_PROPERTIES, false, (string) (self::TAB_PROPERTIES), ['id' => "tab_" . self::TAB_PROPERTIES, 'title' => g_l('modules_voting', '[property]')]);
		if(!$this->View->Model->IsFolder){
			$we_tabs->addTab(g_l('modules_voting', '[inquiry]'), 'fa-comment', false, (string) (self::TAB_INQUIRY), ['id' => "tab_" . self::TAB_INQUIRY]);
			$we_tabs->addTab(g_l('modules_voting', '[options]'), 'fa-cogs', false, (string) (self::TAB_OPTIONS), ['id' => "tab_" . self::TAB_OPTIONS]);

			if($this->View->Model->ID){
				$we_tabs->addTab(g_l('modules_voting', '[result]'), 'fa-bar-chart', false, (string) (self::TAB_RESULT), ['id' => "tab_" . self::TAB_RESULT]);
			}
		}
		if($this->View->Model->ID){
			$this->jsCmd->addCmd('setTab', 1);
		}
		$tabsHead = we_html_element::cssLink(CSS_DIR . 'we_tab.css') .
			we_html_element::jsScript(JS_DIR . 'initTabs.js') .
			we_html_element::jsScript(WE_JS_MODULES_DIR . '/voting/voting_top.js');

		$body = we_html_element::body([
				'onresize' => "weTabs.setFrameSize()",
				'onload' => "weTabs.setFrameSize();document.getElementById('tab_'+top.content.activ_tab).className='tabActive';",
				'id' => "eHeaderBody"
				], '<div id="main"><div id="headrow"><b>' . str_replace(" ", "&nbsp;", g_l('modules_voting', ($this->View->Model->IsFolder ? '[group]' : '[voting]'))) . ':&nbsp;</b><span id="h_path" class="header_small"><b id="titlePath">' . str_replace(" ", "&nbsp;", $this->View->Model->Path) . '</b> (ID:' . $this->View->Model->ID . ')</span></div>' .
				$we_tabs->getHTML() .
				'</div>'
		);

		return $this->getHTMLDocument($body, $tabsHead);
	}

	protected function getHTMLEditorBody(): string{
		if(we_base_request::bool('home')){
			return $this->View->getHomeScreen();
		}

		$hiddens = ['cmd' => 'we_voting_edit', 'pnt' => 'edbody', 'vernr' => we_base_request::int('vernr', 0)];

		$body = we_html_element::body(['class' => 'weEditorBody', 'onload' => "loaded=1;setMultiEdits();", 'onunload' => "doUnload()"], we_html_element::form([
					'name' => we_base_constants::WE_FORM, 'onsubmit' => "return false"], $this->View->getCommonHiddens($hiddens) . $this->getHTMLProperties()));

		return $this->getHTMLDocument($body, $this->jsCmd->getCmds() . $this->View->getJSProperty());
	}

	function getHTMLTab1(): we_html_multiIconBoxContent{
		//$weSuggest = we_gui_suggest::getInstance();
		//FIXME: remove table, correct multichooserDiv!
		$table = (new we_html_table(['id' => 'ownersTable', 'style' => 'display: ' . ($this->View->Model->RestrictOwners ? 'block' : 'none') . ';'], 3, 2))
			->setCol(0, 1, g_l('modules_voting', '[limit_access_text]'), ['colspan' => '2', 'class' => 'defaultfont'])
			->setCol(1, 1, we_html_element::div(['class' => 'multichooserDiv'], we_html_element::div(['id' => 'owners', 'class' => 'multichooser', 'style' => 'width: 510px; height: 60px; border: #AAAAAA solid 1px;'])));
		$idname = 'owner_id';
		$textname = 'owner_text';

		// IMI: replace inline js
		$table->setCol(2, 0, we_html_element::hiddens([$idname => '',
				$textname => '']) .
			we_html_button::create_button(we_html_button::ADD, "javascript:top.content.setHot(); we_cmd('we_users_selector',{id:'" . $idname . "',targetName:'" . $textname . "',targetID:'" . $idname . "',jsCmd:'we_users_ownerAdd',multiple:true});")
			, ['colspan' => '2', 'style' => 'text-align:right']);

		$parts = (new we_html_multiIconBoxContent())
			->add((new we_html_multiIconBoxContentLine(g_l('modules_voting', '[property]'), we_html_element::hiddens([
						'owners_name' => '',
						'owners_count' => '0',
						'newone' => ($this->View->Model->ID == 0 ? '1' : '0')
					]) .
					we_html_tools::formElementTable(g_l('modules_voting', '[headline_name]'), we_html_tools::textInput('Text', $this->View->Model->Text, 0, ['style' => "width: 520px;",
							'id' => 'yuiAcInputPathName', 'onchange' => "top.content.setHot();",
							'onblur' => "parent.edheader.weTabs.setTitlePath(this.value);"])) .
					we_html_element::br() .
					$this->getHTMLDirChooser() .
					we_html_element::br() .
					(!$this->View->Model->IsFolder ? we_html_tools::formElementTable(g_l('modules_voting', '[headline_publish_date]'), we_html_tools::getNewDateInput('Publish_date[%s]', $this->View->Model->PublishDate, we_html_tools::FORMAT_DATE, true)) : ''), we_html_multiIconBox::PROP_PATH))
				->noline()
			)
			->add((new we_html_multiIconBoxContentLine('', we_html_forms::checkboxWithHidden($this->View->Model->RestrictOwners ? true : false, 'RestrictOwners', g_l('modules_voting', '[limit_access]'), false, 'defaultfont', 'top.content.setHot(); toggle(\'ownersTable\')')))
				->space(we_html_multiIconBox::SPACE_MED)
				->noline()
			)
			->add((new we_html_multiIconBoxContentLine('', $table->getHtml()))
			->space(we_html_multiIconBox::SPACE_MED)
		);

		if($this->View->Model->IsFolder){
			$parts->add((new we_html_multiIconBoxContentLine(g_l('modules_voting', '[control]'), we_html_button::formatButtons(we_html_button::create_button(we_html_button::LOG_SHOW, "javascript:we_cmd('show_log')") . we_html_button::create_button(we_html_button::DELETE, "javascript:we_cmd('delete_log')"))))
					->space(we_html_multiIconBox::SPACE_MED)
					->noline()
			);

			$ok = we_html_button::create_button(we_html_button::EXPORT, "javascript:we_cmd('exportGroup_csv')");

			$lineend = (new we_html_select(['name' => 'csv_lineend', 'class' => 'defaultfont', 'style' => 'width: 520px']))
				->addOptions(we_exim_ExportCSV::getCSVLineEnds());

			$charsets = we_base_charsetHandler::inst()->getCharsetsForTagWizzard();
			$importCharset = we_html_tools::textInput('the_charset', '', 255, [], 'text', '200px');
			$importCharsetChooser = we_html_tools::select("ImportCharsetSelect", $charsets, 1, '', false, ['onchange' => "document.forms[0].elements.the_charset.value=this.options[this.selectedIndex].value;this.selectedIndex=-1;"], true, 325, "defaultfont", false);
			$import_Charset = '<table class="default"><tr><td>' . $importCharset . '</td><td>' . $importCharsetChooser . '</td></tr></table>';

			$delimiter = (new we_html_select(['name' => 'csv_delimiter', 'class' => 'defaultfont', 'style' => 'width: 520px']))
				->addOptions(we_exim_ExportCSV::getCSVDelimeters());

			$enclose = (new we_html_select(['name' => 'csv_enclose', 'class' => 'defaultfont', 'style' => 'width: 520px']))
				->addOptions([
				'0' => g_l('export', '[double_quote]'),
				'1' => g_l('export', '[single_quote]')
			]);


			$export_box = (new we_html_table(['class' => 'default', 'style' => 'margin-top:10px;'], 12, 1))
				->setCol(1, 0, we_html_tools::formElementTable(g_l('export', '[dir]'), $this->formFileChooser('csv_dir', '/', '', we_base_ContentTypes::FOLDER_DOCUMENT)), [
					'style' => 'padding-bottom:5px;'])
				->setCol(3, 0, we_html_tools::formElementTable(g_l('export', '[csv_lineend]'), $lineend->getHtml()), ['class' => 'defaultfont', 'style' => 'padding-bottom:5px;'])
				->setCol(5, 0, we_html_tools::formElementTable(g_l('modules_voting', '[csv_charset]'), $import_Charset), ['class' => 'defaultfont', 'style' => 'padding-bottom:5px;'])
				->setCol(7, 0, we_html_tools::formElementTable(g_l('export', '[csv_delimiter]'), $delimiter->getHtml()), ['class' => 'defaultfont', 'style' => 'padding-bottom:5px;'])
				->setCol(9, 0, we_html_tools::formElementTable(g_l('export', '[csv_enclose]'), $enclose->getHtml()), ['style' => 'padding-bottom:5px;'])
				->setCol(11, 0, $ok);

			$parts->add((new we_html_multiIconBoxContentLine(g_l('modules_voting', '[export]'), we_html_tools::alertAttentionBox(g_l('modules_voting', '[export_txt]'), we_html_tools::TYPE_INFO, 520) .
						$export_box->getHtml()))
					->space(we_html_multiIconBox::SPACE_MED)
			);

			return $parts;
		}

		$activeTime = (new we_html_select(['name' => 'ActiveTime', 'class' => 'weSelect', 'style' => 'width:200', 'onchange' => 'top.content.setHot(); if(this.value!=0) setVisible(\'valid\',true); else setVisible(\'valid\',false);']))
			->addOptions([
				'0' => g_l('modules_voting', '[always]'),
				'1' => g_l('modules_voting', '[until]')
			])
			->selectOption($this->View->Model->ActiveTime);

		$table = (new we_html_table([], 4, 2))
			->setCol(0, 0, we_html_tools::alertAttentionBox(g_l('modules_voting', '[valid_txt]'), we_html_tools::TYPE_INFO, 520, false, 60), ['colspan' => '2'])
			->setCol(1, 0, we_html_forms::checkboxWithHidden($this->View->Model->Active ? true : false, 'Active', g_l('modules_voting', '[active_till]'), false, 'defaultfont', 'toggle(\'activetime\');if(!this.checked) setVisible(\'valid\',false); else if(document.we_form.ActiveTime.value==1) setVisible(\'valid\',true); else setVisible(\'valid\',false);'), [
				'colspan' => '2'])
			->setCol(2, 1, we_html_element::div(['id' => 'activetime', 'style' => 'display: ' . ($this->View->Model->Active ? 'block' : 'none') . ';'], $activeTime->getHtml()
				)
			)
			->setCol(3, 1, we_html_element::div(['id' => 'valid', 'style' => 'display: ' . ($this->View->Model->Active && $this->View->Model->ActiveTime ? 'block' : 'none') . ';'], we_html_tools::formElementTable('', we_html_tools::getNewDateInput('Valid_date[%s]', $this->View->Model->Valid, we_html_tools::FORMAT_DATE_TIME, true))
			)
		);

		$parts->add((new we_html_multiIconBoxContentLine(g_l('modules_voting', '[valid]'), $table->getHtml(), we_html_multiIconBox::PROP_TIME))
				->space(we_html_multiIconBox::SPACE_MED)
				->noline()
		);
		return $parts;
	}

	private function getHTMLTab2(): we_html_multiIconBoxContent{
		$successor_box = (new we_html_table(['class' => 'default', 'style' => 'margin-top:10px;'], 2, 1))
			->setCol(1, 0, we_html_tools::formElementTable(g_l('modules_voting', '[voting-successor]'), $this->formFileChooser('Successor', '/', '', '')));


		$displaySuccessor = ($this->View->Model->AllowSuccessor ? 'block' : 'none');

		$parts = (new we_html_multiIconBoxContent())
			->add((new we_html_multiIconBoxContentLine(g_l('modules_voting', '[headline_datatype]'),
																									we_html_forms::checkboxWithHidden($this->View->Model->IsRequired ? true : false, 'IsRequired', g_l('modules_voting', '[IsRequired]'), false, 'defaultfont', 'top.content.setHot();') . we_html_element::br() .
				we_html_forms::checkboxWithHidden($this->View->Model->AllowFreeText ? true : false, 'AllowFreeText', g_l('modules_voting', '[AllowFreeText]'), false, 'defaultfont', 'top.content.setHot();answers_edit.toggleMinCount();') . we_html_element::br() .
				we_html_forms::checkboxWithHidden($this->View->Model->AllowImages ? true : false, 'AllowImages', g_l('modules_voting', '[AllowImages]'), false, 'defaultfont', 'top.content.setHot();answers_edit.toggleImages();') . we_html_element::br() .
				we_html_forms::checkboxWithHidden($this->View->Model->AllowMedia ? true : false, 'AllowMedia', g_l('modules_voting', '[AllowMedia]'), false, 'defaultfont', 'top.content.setHot();answers_edit.toggleMedia();') . we_html_element::br() .
				we_html_forms::checkboxWithHidden($this->View->Model->AllowSuccessor ? true : false, 'AllowSuccessor', g_l('modules_voting', '[AllowSuccessor]'), false, 'defaultfont', 'top.content.setHot(); toggle(\'Successor\')') . we_html_element::br() .
				we_html_tools::formElementTable('', we_html_tools::textInput('Successor', (string) $this->View->Model->Successor, 0, ['style' => 'width: 520px;display:' . $displaySuccessor,
						'id' => 'Successor', 'onchange' => "top.content.setHot();"])) .
				we_html_forms::checkboxWithHidden($this->View->Model->AllowSuccessors ? true : false, 'AllowSuccessors', g_l('modules_voting', '[AllowSuccessors]'), false, 'defaultfont', 'top.content.setHot();answers_edit.toggleSuccessors();') . $successor_box->getHtml()))
			->space(we_html_multiIconBox::SPACE_MED)
		);

		$select = new we_html_select(['name' => 'selectVar', 'class' => 'weSelect', 'onchange' => 'top.content.setHot();question_edit.showVariant(this.value);answers_edit.showVariant(this.value);document.we_form.vernr.value=this.value;refreshTexts();',
			'style' => 'width:450px;']);
		foreach(array_keys($this->View->Model->QASet) as $variant){
			$select->addOption((string) $variant, g_l('modules_voting', '[variant]') . ' ' . ($variant + 1));
		}
		$select->selectOption(we_base_request::int('vernr', 0));

		$selectCode = (new we_html_table(['class' => 'default'], 1, 3))
			->setCol(0, 0, $select->getHtml())
			->setCol(0, 1, we_html_button::create_button(we_html_button::PLUS, "javascript:top.content.setHot();question_edit.addVariant();answers_edit.addVariant();question_edit.showVariant(question_edit.variantCount-1);answers_edit.showVariant(answers_edit.variantCount-1);document.we_form.selectVar.options[document.we_form.selectVar.options.length] = new Option('" . g_l('modules_voting', '[variant]') . " '+question_edit.variantCount,question_edit.variantCount-1,false,true);"))
			->setCol(0, 2, we_html_button::create_button(we_html_button::TRASH, "javascript:top.content.setHot();if(question_edit.variantCount>1){ question_edit.deleteVariant(document.we_form.selectVar.selectedIndex);answers_edit.deleteVariant(document.we_form.selectVar.selectedIndex);document.we_form.selectVar.options.length--;document.we_form.selectVar.selectedIndex=question_edit.currentVariant;refreshTexts();} else {WE().util.showMessage(WE().consts.g_l.voting.variant_limit,WE().consts.message.WE_MESSAGE_ERROR, window);}"))
			->setColAttributes(0, 1, ['style' => "padding:0 5px;"])
			->getHtml();

		$table = (new we_html_table(['class' => 'default'], 3, 1))
			->setCol(0, 0, $selectCode)
			->setCol(1, 0, we_html_tools::formElementTable(g_l('modules_voting', '[inquiry_question]'), we_html_element::div(['id' => 'question'])), ['padding-top:10px;'])
			->setCol(2, 0, we_html_tools::formElementTable(g_l('modules_voting', '[inquiry_answers]'), we_html_element::div(['id' => 'answers'])), ['padding-top:10px;']);

		$parts->add((new we_html_multiIconBoxContentLine(g_l('modules_voting', '[headline_data]'), we_html_element::hiddens([
						'question_name' => '',
						'variant_count' => '0',
						'answers_name' => '',
						'item_count' => '0',
						'iptable_name' => '',
						'iptable_count' => '0']) .
					$table->getHtml() .
					we_html_button::create_button(we_html_button::PLUS, "javascript:top.content.setHot();answers_edit.addItem()")))
				->space(we_html_multiIconBox::SPACE_MED)
		);

		return $parts;
	}

	private function getHTMLTab3(): we_html_multiIconBoxContent{
		$parts = new we_html_multiIconBoxContent();

		$selectTime = (new we_html_select(['name' => 'RevoteTime', 'class' => 'weSelect', 'style' => 'width:200', 'onchange' => 'top.content.setHot(); if(this.value==0) setVisible(\'method_table\',false); else setVisible(\'method_table\',true);']))
			->addOptions([
				'630720000' => g_l('modules_voting', '[never]'),
				(string) (we_base_util::SECONDS_DAY) => g_l('modules_voting', '[one_day]'),
				'3600' => g_l('modules_voting', '[one_hour]'),
				'1800' => g_l('modules_voting', '[thirthty_minutes]'),
				'900' => g_l('modules_voting', '[feethteen_minutes]'),
				'0' => g_l('modules_voting', '[always]')
			])
			->selectOption($this->View->Model->RevoteTime);

		$datasize = fI('SELECT (LENGTH(Revote)+LENGTH(RevoteUserAgent)) AS Size FROM ' . VOTING_TABLE);

		$table = (new we_html_table(['id' => 'method_table', 'style' => 'display: ' . ($this->View->Model->RevoteTime ? 'block' : 'none')], 10, 2))
			->setCol(0, 0, we_html_tools::alertAttentionBox(
					we_html_element::b(g_l('modules_voting', '[cookie_method]')) . we_html_element::br() .
					g_l('modules_voting', '[cookie_method_help]') .
					we_html_element::br() . we_html_element::b(g_l('modules_voting', '[ip_method]')) . we_html_element::br() .
					g_l('modules_voting', '[ip_method_help]'), we_html_tools::TYPE_INFO, 520, false, 100
				), ['colspan' => '2']
			)
			->setCol(1, 0, we_html_forms::radiobuttonV('1', (string) $this->View->Model->RevoteControl, 'RevoteControl', g_l('modules_voting', '[cookie_method]'), true, "defaultfont", "top.content.setHot();"), [
				'colspan' => '2'])
			->setCol(2, 0, we_html_forms::checkboxWithHidden($this->View->Model->FallbackIp ? true : false, 'FallbackIp', g_l('modules_voting', '[fallback]'), false, "defaultfont", "top.content.setHot();"), [
				'style' => 'padding-left:10px;'])
			->setCol(3, 0, we_html_forms::radiobuttonV('0', (string) $this->View->Model->RevoteControl, 'RevoteControl', g_l('modules_voting', '[ip_method]'), true, "defaultfont", "top.content.setHot();"), [
				'colspan' => '2', 'style' => 'padding-top:10px;'])
			->setCol(4, 1, we_html_forms::checkboxWithHidden($this->View->Model->UserAgent ? true : false, 'UserAgent', g_l('modules_voting', '[save_user_agent]'), false, "defaultfont", "top.content.setHot();"))
			->setCol(5, 1, we_html_tools::alertAttentionBox(sprintf(g_l('modules_voting', '[delete_ipdata_text]'), we_html_element::span([
							'id' => 'ip_mem_size'], (string) ($datasize))), we_html_tools::TYPE_INFO, 500, false, 100) .
				we_html_button::create_button(we_html_button::DELETE, "javascript:we_cmd('reset_ipdata')")
				, ['id' => 'delete_ip_data', 'style' => 'display: ' . ($datasize > 0 ? 'block' : 'none')])
			->setCol(6, 0, we_html_forms::radiobuttonV('2', (string) $this->View->Model->RevoteControl, 'RevoteControl', g_l('modules_voting', '[userid_method]', false, true), true, '', "top.content.setHot();"), [
			'colspan' => '2', 'style' => 'padding-top:10px;']);


		$parts->add((new we_html_multiIconBoxContentLine(g_l('modules_voting', '[headline_revote]'), we_html_tools::alertAttentionBox(g_l('modules_voting', '[time_after_voting_again_help]'), we_html_tools::TYPE_INFO, 520, false, 100) .
					we_html_element::br() .
					we_html_tools::formElementTable(g_l('modules_voting', '[time_after_voting_again]'), $selectTime->getHtml()) .
					we_html_element::br() .
					$table->getHtml()))
				->space(we_html_multiIconBox::SPACE_MED)
		);

		$table = we_html_element::div(['id' => 'LogData', 'style' => 'display: ' . ($this->View->Model->Log ? 'block' : 'none') . ';'], we_html_button::formatButtons(we_html_button::create_button(we_html_button::LOG_SHOW, "javascript:we_cmd('show_log')") . we_html_button::create_button(we_html_button::DELETE, "javascript:we_cmd('delete_log')"))
		);

		$parts->add((new we_html_multiIconBoxContentLine(g_l('modules_voting', '[control]'), we_html_forms::checkboxWithHidden($this->View->Model->Log ? true : false, 'Log', g_l('modules_voting', '[voting_log]'), false, 'defaultfont', 'top.content.setHot(); toggle(\'LogData\')') .
					$table))
				->space(we_html_multiIconBox::SPACE_MED)
				->noline()
			)
			->add((new we_html_multiIconBoxContentLine('', we_html_forms::checkboxWithHidden($this->View->Model->RestrictIP ? true : false, 'RestrictIP', g_l('modules_voting', '[forbid_ip]'), false, 'defaultfont', 'top.content.setHot(); toggle(\'RestrictIPDiv\')')))
				->space(we_html_multiIconBox::SPACE_MED)
				->noline()
		);

		$table = (new we_html_table(['id' => 'RestrictIPDiv', 'style' => 'display: ' . ($this->View->Model->RestrictIP ? 'block' : 'none') . ';'], 2, 1))
			->setCol(0, 0, we_html_element::div(['id' => 'iptable', 'class' => 'blockWrapper', 'style' => 'width: 510px; height: 60px; border: #AAAAAA solid 1px;padding: 5px;']), [
				'style' => 'padding-left:10px;'])
			->setCol(1, 0, we_html_button::create_button(we_html_button::DELETE_ALL, "javascript:top.content.setHot(); removeAll()") .
			we_html_button::create_button(we_html_button::ADD, "javascript:top.content.setHot(); newIp()")
			, ['colspan' => '2', 'style' => 'text-align:right']);

		$parts->add((new we_html_multiIconBoxContentLine('', $table->getHtml()))
				->space(we_html_multiIconBox::SPACE_MED)
		);

		return $parts;
	}

	function getHTMLTab4(): we_html_multiIconBoxContent{
		$total_score = array_sum($this->View->Model->Scores);

		$version = we_base_request::int('vernr', 0);

		$table = new we_html_table(['class' => 'defaultfont', 'style' => 'width: 520px'], 1, 5);
		if(isset($this->View->Model->QASet[$version])){
			$table->setCol(0, 0, we_html_element::b(we_html_element::span(['id' => 'question_score'], htmlspecialchars(stripslashes($this->View->Model->QASet[$version]['question'])))), [
				'colspan' => '5', 'class' => 'defaultfont']);
		}
		$i = 1;
		if(isset($this->View->Model->QASet[$version])){
			foreach($this->View->Model->QASet[$version]['answers'] as $key => $value){
				$key = (int) $key;
				if(!isset($this->View->Model->Scores[$key])){
					$this->View->Model->Scores[$key] = 0;
				}

				$percent = we_base_util::getPercent((int) $total_score, $this->View->Model->Scores[$key], 2);

				$pb = new we_gui_progressBar($percent, 150, 'item' . $key);

				$table->addRows(1, ['id' => 'row_scores_' . $key]);
				$table
					->setCol($i, 0, we_html_element::span(['id' => 'answers_score_' . $key], htmlspecialchars(stripslashes($value))), ['style' => 'width: 400px'])
					->setCol($i, 1, $pb->getHTML())
					->setCol($i, 2, '&nbsp;')
					->setCol($i, 3, we_html_tools::textInput('scores_' . $key, (string) $this->View->Model->Scores[$key], 0, ['id' => "scores_" . $key, 'onkeyup' => "checkValue(this," . $this->View->Model->Scores[$key] . ");"]));
				$i++;
			}
		}
		$table
			->setCol($i, 0, we_html_element::b(g_l('modules_voting', '[total_voting]') . ':') . we_html_element::hidden("updateScores", false, 'updateScores'))
			->setCol($i, 3, we_html_element::b(we_html_element::span(['id' => 'total'], (string) ($total_score))), ['colspan' => '3']);

		$butt = we_html_button::create_button('reset_score', "javascript:top.content.setHot();resetScores();");

		$js = we_gui_progressBar::getJSCode();

		$ok = we_html_button::create_button(we_html_button::EXPORT, "javascript:we_cmd('export_csv')");

		$lineend = (new we_html_select(['name' => 'csv_lineend', 'class' => 'defaultfont', 'style' => 'width: 520px']))
			->addOptions(we_exim_ExportCSV::getCSVLineEnds());

		$delimiter = (new we_html_select(['name' => 'csv_delimiter', 'class' => 'defaultfont', 'style' => 'width: 520px']))
			->addOptions(we_exim_ExportCSV::getCSVDelimeters());

		$enclose = (new we_html_select(['name' => 'csv_enclose', 'class' => 'defaultfont', 'style' => 'width: 520px']))
			->addOptions([
			'0' => g_l('export', '[double_quote]'),
			'1' => g_l('export', '[single_quote]')
		]);
		$export_box = (new we_html_table(['class' => 'default', 'style' => 'margin-top:10px;'], 5, 1))
			->setCol(0, 0, we_html_tools::formElementTable(g_l('export', '[dir]'), $this->formFileChooser('csv_dir', '/', '', we_base_ContentTypes::FOLDER_DOCUMENT)), [
				'style' => 'padding-bottom:5px;'])
			->setCol(1, 0, we_html_tools::formElementTable(g_l('export', '[csv_lineend]'), $lineend->getHtml()))
			->setCol(2, 0, we_html_tools::formElementTable(g_l('export', '[csv_delimiter]'), $delimiter->getHtml()), ['padding-top:5px;'])
			->setCol(3, 0, we_html_tools::formElementTable(g_l('export', '[csv_enclose]'), $enclose->getHtml()), ['padding-top:5px;'])
			->setCol(4, 0, $ok, ['padding-top:5px;']);

		return (new we_html_multiIconBoxContent())
				->add((new we_html_multiIconBoxContentLine(g_l('modules_voting', '[inquiry]'), $js .
						we_html_element::hidden('scores_changed', 0) .
						$table->getHTML() .
						we_html_element::br() . $butt))
					->space(we_html_multiIconBox::SPACE_MED)
				)
				->add((new we_html_multiIconBoxContentLine(g_l('modules_voting', '[export]'), we_html_tools::alertAttentionBox(g_l('modules_voting', '[export_txt]'), we_html_tools::TYPE_INFO, 520) .
						$export_box->getHtml()))
					->space(we_html_multiIconBox::SPACE_MED)
		);
	}

	function getHTMLProperties($preselect = ''): string{// TODO: move to weVotingView
		$t = we_base_request::int('tabnr', 1);
		$tabNr = ($this->View->Model->IsFolder && $t != 1) ? 1 : $t;

		return we_html_element::jsScript(JS_DIR . 'utils/multi_editMulti.js') .
			we_html_element::div(['id' => 'tab1', 'style' => ($tabNr == 1 ? '' : 'display: none')],
												(new we_html_multiIconBox($this->getHTMLTab1()))
					->marginLeft()
					->headline($preselect)
					->get()) .
			(!$this->View->Model->IsFolder ?
			(
			we_html_element::div(['id' => 'tab2', 'style' => ($tabNr == 2 ? '' : 'display: none')], (new we_html_multiIconBox($this->getHTMLTab2()))->marginLeft()->headline($preselect)->get()) .
			we_html_element::div(['id' => 'tab3', 'style' => ($tabNr == 3 ? '' : 'display: none')], (new we_html_multiIconBox($this->getHTMLTab3()))->marginLeft()->headline($preselect)->get()) .
			we_html_element::div(['id' => 'tab4', 'style' => ($tabNr == 4 ? '' : 'display: none')], (new we_html_multiIconBox($this->getHTMLTab4()))->marginLeft()->headline($preselect)->get())
			) : '');
	}

	private function getHTMLDirChooser(): string{
		$path = id2path($this->View->Model->ParentID ?? 0, VOTING_TABLE);

		$weSuggest = we_gui_suggest::getInstance();
		$weSuggest->setAcId('PathGroup')
			->setContentType([we_base_ContentTypes::FOLDER_COMMON])
			->setInput('ParentPath', $path, [], false, true)
			->setMaxResults(10)
			->setResult('ParentID', (string) ($this->View->Model->ParentID ?: 0))
			->setSelector(we_gui_suggest::DirSelector)
			->setTable(VOTING_TABLE)
			->setWidth(416)
			->setSelectButton(we_html_button::create_button(we_html_button::SELECT, "javascript:top.content.setHot(); we_cmd('we_voting_dirSelector',{id:document.we_form.elements.ParentID.value,targetID:'ParentID',targetName:'ParentPath'})"))
			->setLabel(g_l('modules_voting', '[group]'));

		return $weSuggest->getHTML();
	}

	protected function getHTMLCmd(): string{
		if(($pid = we_base_request::int('pid', -1)) < 0){
			return $this->getHTMLDocument(we_html_element::body(), $this->jsCmd->getCmds());
		}

		$offset = we_base_request::int("offset", 0);
		$this->jsCmd->addCmd('loadTree', ['clear' => !$pid, 'items' => we_voting_tree::getItems($pid, $offset, $this->Tree->default_segment)]);

		return $this->getHTMLDocument(
				we_html_element::body([], we_html_element::form(
						['name' => we_base_constants::WE_FORM], we_html_element::hiddens([
							'pnt' => "cmd",
							'cmd' => "no_cmd"]
						)
					)
		));
	}

	private function getHTMLExportCsvMessage(): string{
		$link = we_base_request::_(we_base_request::FILE, "lnk");
		if($link === false){
			return '';
		}

		$table = (new we_html_table(['class' => 'default withSpace'], 3, 1))
			->setCol(0, 0, sprintf(g_l('modules_voting', '[csv_export]'), $link))
			->setCol(1, 0, we_backup_wizard::getDownloadLinkText())
			->setCol(2, 0, we_html_element::a(['href' => we_http_util::getServerUrl(true) . $link, 'download' => basename($link)], g_l('modules_voting', '[csv_download]')));

		$close = we_html_button::create_button(we_html_button::CLOSE, "javascript:WE().util.dialogTop(window).doClose();");

		$body = we_html_element::body(['class' => 'weDialogBody', 'onload' => 'self.focus();'], we_html_element::form(
					['name' => we_base_constants::WE_FORM, 'method' => 'post'], we_html_element::hidden("group", "") .
					we_html_tools::dialogLayout(
						$table->getHtml(), g_l('modules_voting', '[csv_download]'), we_html_button::formatButtons($close)
					)
				)
		);

		return $this->getHTMLDocument($body);
	}

	private function getHTMLExportGroupCsvMessage(): string{
		$link = we_base_request::_(we_base_request::FILE, "lnk");
		if($link === false){
			return '';
		}
		$table = (new we_html_table(['class' => 'default withSpace'], 3, 1))
			->setCol(0, 0, sprintf(g_l('modules_voting', '[csv_export]'), $link))
			->setCol(1, 0, we_backup_wizard::getDownloadLinkText())
			->setCol(2, 0, we_html_element::a(['href' => we_http_util::getServerUrl(true) . $link], g_l('modules_voting', '[csv_download]')));

		$close = we_html_button::create_button(we_html_button::CLOSE, "javascript:WE().util.dialogTop(window).doClose();");

		$body = we_html_element::body(['class' => 'weDialogBody', 'onload' => 'self.focus();'], we_html_element::form(['name' => we_base_constants::WE_FORM, 'method' => 'post'], we_html_element::hidden("group", '') .
					we_html_tools::dialogLayout(
						$table->getHtml(), g_l('modules_voting', '[csv_download]'), we_html_button::formatButtons($close)
					)
				)
		);

		return $this->getHTMLDocument($body);
	}

	private function getHTMLResetIPData(): string{
		$this->View->Model->resetIpData();

		$close = we_html_button::create_button(we_html_button::CLOSE, "javascript:WE().util.dialogTop(window).doClose();");

		$body = we_html_element::body(['class' => 'weDialogBody', 'onload' => 'self.focus();'], we_html_tools::dialogLayout(
					we_html_element::span(['class' => 'defaultfont'], g_l('modules_voting', '[data_deleted_info]')), g_l('modules_voting', '[voting]'), we_html_button::formatButtons($close)
				)
		);
		return $this->getHTMLDocument($body);
	}

	private function getHTMLDeleteLog(): string{
		$this->View->Model->deleteLogData();

		$close = we_html_button::create_button(we_html_button::CLOSE, "javascript:WE().util.dialogTop(window).doClose();");

		$body = we_html_element::body(['class' => 'weDialogBody', 'onload' => 'self.focus();'], we_html_tools::dialogLayout(
					we_html_element::span(['class' => 'defaultfont'], g_l('modules_voting', '[data_deleted_info]')), g_l('modules_voting', '[voting]'), we_html_button::formatButtons($close)
				)
		);
		return $this->getHTMLDocument($body);
	}

	private function getMessageForStatus(int $status): string{
		switch($status){
			case we_voting_voting::SUCCESS:
				return g_l('modules_voting', '[log_success]');
			case we_voting_voting::ERROR :
				$mess = g_l('modules_voting', '[log_error]');
				break;
			case we_voting_voting::ERROR_ACTIVE :
				$mess = g_l('modules_voting', '[log_error_active]');
				break;
			case we_voting_voting::ERROR_REVOTE :
				$mess = g_l('modules_voting', '[log_error_revote]');
				break;
			case we_voting_voting::ERROR_BLACKIP :
				$mess = g_l('modules_voting', '[log_error_blackip]');
				break;
			default:
				$mess = g_l('modules_voting', '[log_error]');
		}
		return we_html_element::span(['style' => 'color: red;'], $mess);
	}

	private function getHTMLShowLogNew(): string{
		$close = we_html_button::create_button(we_html_button::CLOSE, "javascript:WE().util.dialogTop(window).doClose();");
		$refresh = we_html_button::create_button(we_html_button::REFRESH, "javascript:location.reload();");

		$voting = new we_voting_voting();
		$voting->load($this->View->Model->ID ?? 0);
		$log = $voting->loadDB($voting->ID ?? 0);


		$headline = [
			we_html_element::b(g_l('modules_voting', '[voting-session]')),
			we_html_element::b(g_l('modules_voting', '[voting-id]')),
			we_html_element::b(g_l('modules_voting', '[time]')),
			we_html_element::b(g_l('modules_voting', '[ip]')),
			we_html_element::b(g_l('modules_voting', '[user_agent]')),
			we_html_element::b(g_l('modules_voting', '[cookie]')),
			we_html_element::b(g_l('modules_voting', '[log_fallback]')),
			we_html_element::b(g_l('modules_voting', '[status]')),
			we_html_element::b(g_l('modules_voting', '[answerID]')),
			we_html_element::b(g_l('modules_voting', '[answerText]')),
			we_html_element::b(g_l('modules_voting', '[voting-successor]')),
			we_html_element::b(g_l('modules_voting', '[voting-additionalfields]')),
		];

		$content = [];

		$count = 15;
		$size = count($log);

		if($size > 0){
			$size--;
			$start = min(max(we_base_request::int('start', $size), 0), $size);
			$back = min($start + $count, $size);
			$next = max($start - $count, -1);

			for($i = $start; $i > $next; $i--){
				if($i < 0){
					break;
				}
				$data = $log[$i];

				$addData = we_unserialize($data['additionalfields']);
				$addDataString = '';
				foreach($addData as $key => $value){
					$addDataString .= $key . ': ' . $value . '<br/>';
				}

				$content[] = [
					(string) $data['votingsession'],
					(string) $data['voting'],
					date(g_l('weEditorInfo', '[date_format]'), $data['time']),
					(string) inet_ntop($data['ip']),
					$data['agent'],
					g_l('modules_voting', $data['cookie'] ? '[enabled]' : '[disabled]'),
					g_l('global', $data['fallback'] ? '[yes]' : '[no]'),
					$this->getMessageForStatus($data['status']),
					$data['answer'],
					$data['answertext'],
					(string) $data['successor'],
					$addDataString,
				];
			}

			$nextprev = '<table style="margin-top: 10px;" class="default"><tr><td>' .
				($start < $size ?
				we_html_button::create_button(we_html_button::BACK, WEBEDITION_DIR . 'we_cmd.php?we_cmd[0]=we_showMod&mod=voting&pnt=show_log&start=' . $back) : //bt_back
				we_html_button::create_button(we_html_button::BACK, "", '', "", true)
				) . "</td><td style='text-align:center' class='defaultfont' width='120'><b>" . ($size - $start + 1) . "&nbsp;-&nbsp;" .
				($size - $next) .
				"&nbsp;" . g_l('global', '[from]') . " " . ($size + 1) . '</b></td><td>' .
				($next > 0 ?
				we_html_button::create_button(we_html_button::NEXT, WEBEDITION_DIR . 'we_cmd.php?we_cmd[0]=we_showMod&mod=voting&pnt=show_log&start=' . $next) : //bt_next
				we_html_button::create_button(we_html_button::NEXT, "", "", "", true)
				) .
				"</td></tr></table>";

			$parts = (new we_html_multiIconBoxContent())
				->add((new we_html_multiIconBoxContentLine('', we_html_tools::dialogBorder(1000, $content, $headline) . $nextprev))
				->noline()
			);
		} else {
			$parts = (new we_html_multiIconBoxContent())
				->add((new we_html_multiIconBoxContentLine('', we_html_element::span(['class' => 'middlefont lowContrast'], g_l('modules_voting', '[log_is_empty]')) .
					we_html_element::br(2)))
				->noline()
			);
		}

		$body = we_html_element::body(['class' => 'weDialogBody', 'onload' => 'self.focus();'], (new we_html_multiIconBox($parts, "show_log_data"))->marginLeft()->buttons(we_html_button::position_yes_no_cancel($refresh, $close))
					->headline(g_l('modules_voting', '[voting]'))
					->get()
		);
		return $this->getHTMLDocument($body);
	}

	function getHTMLShowGroupLog(): string{//FIXME: unused??
		$close = we_html_button::create_button(we_html_button::CLOSE, "javascript:WE().util.dialogTop(window).doClose();");
		$refresh = we_html_button::create_button(we_html_button::REFRESH, "javascript:location.reload();");

		$voting = new we_voting_voting();
		$voting->load($this->View->Model->ID);
		$log = $voting->loadDB($voting->ID);


		$headline = [
			we_html_element::b(g_l('modules_voting', '[time]')),
			we_html_element::b(g_l('modules_voting', '[ip]')),
			we_html_element::b(g_l('modules_voting', '[user_agent]')),
			we_html_element::b(g_l('modules_voting', '[cookie]')),
			we_html_element::b(g_l('modules_voting', '[log_fallback]')),
			we_html_element::b(g_l('modules_voting', '[status]')),
			we_html_element::b(g_l('modules_voting', '[answerID]')),
			we_html_element::b(g_l('modules_voting', '[answerText]')),
		];

		$content = [];

		$count = 15;
		$size = count($log);

		if($size > 0){
			$size--;
			$start = min(max(we_base_request::int('start', $size), 0), $size);
			$back = min($start + $count, $size);
			$next = max($start - $count, -1);

			for($i = $start; $i > $next; $i--){
				if($i < 0){
					break;
				}
				$data = $log[$i];

				$content[] = [
					date(g_l('weEditorInfo', '[date_format]'), $data['time']),
					inet_ntop($data['ip']),
					$data['agent'],
					g_l('modules_voting', $data['cookie'] ? '[enabled]' : '[disabled]'),
					g_l('global', $data['fallback'] ? '[yes]' : '[no]'),
					$this->getMessageForStatus($data['status']),
					$data['answer'],
					$data['answertext'],
				];
			}

			$nextprev = '<table style="margin-top: 10px;" class="default"><tr><td>' .
				($start < $size ?
				we_html_button::create_button(we_html_button::BACK, WEBEDITION_DIR . 'we_cmd.php?we_cmd[0]=we_showMod&mod=voting&pnt=show_log&start=' . $back) : //bt_back
				we_html_button::create_button(we_html_button::BACK, "", '', "", true)
				) . "</td><td style='text-align:center' class='defaultfont' width='120'><b>" . ($size - $start + 1) . "&nbsp;-&nbsp;" .
				($size - $next) .
				"&nbsp;" . g_l('global', '[from]') . ' ' . ($size + 1) . '</b></td><td>' .
				($next > 0 ?
				we_html_button::create_button(we_html_button::NEXT, WEBEDITION_DIR . 'we_cmd.php?we_cmd[0]=we_showMod&mod=voting&pnt=show_log&start=' . $next) : //bt_next
				we_html_button::create_button(we_html_button::NEXT, "", "", "", true)
				) .
				'</td></tr></table>';

			$parts = (new we_html_multiIconBoxContent())
				->add((new we_html_multiIconBoxContentLine('', we_html_tools::dialogBorder(730, $content, $headline) . $nextprev))
				->noline()
			);
		} else {
			$parts = (new we_html_multiIconBoxContent())
				->add((new we_html_multiIconBoxContentLine('', we_html_element::span(['class' => 'middlefont lowContrast'], g_l('modules_voting', '[log_is_empty]')) .
					we_html_element::br(2)))
				->noline()
			);
		}

		$body = we_html_element::body(['class' => 'weDialogBody', 'onload' => 'self.focus();'], (new we_html_multiIconBox($parts, "show_log_data"))
					->marginLeft()
					->buttons(we_html_button::position_yes_no_cancel($refresh, $close))
					->headline(g_l('modules_voting', '[voting]'))
					->get()
		);
		return $this->getHTMLDocument($body);
	}

	function getHTMLDeleteGroupLog(): string{
		$this->View->Model->deleteGroupLogData();

		$close = we_html_button::create_button(we_html_button::CLOSE, "javascript:WE().util.dialogTop(window).doClose();");

		return $this->getHTMLDocument(
				we_html_element::body(['class' => 'weDialogBody', 'onload' => 'self.focus();'], we_html_tools::dialogLayout(
						we_html_element::span(['class' => 'defaultfont'], g_l('modules_voting', '[data_deleted_info]')), g_l('modules_voting', '[voting]'), we_html_button::formatButtons($close))
				)
		);
	}

}
