<?php

declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: e186d0beab45 $
 * $Author: Marc Kraemer <marc.kraemer@webedition.org> $
 * $Date: 2020-02-27 18:05 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
abstract class we_weTag_twbase{

	const GROUP_INPUT = 'input_tags';
	const GROUP_IF = 'if_tags';
	const GROUP_NAVIGATION = 'navigation_tags';
	public const DIR = 'tw';

	/**
	 * @var string
	 */
	public $Name;

	/**
	 * @var we_weTag_typeAttribute
	 */
	protected $TypeAttribute = null;

	/** @var array<we_weTag_attribute>	 */
	protected $Attributes = [];

	/** @var array<we_weTag_attribute>	 */
	protected $UsedAttributes = null;

	/** @var string	 */
	protected $Description;

	/** @var string	 */
	protected $DefaultValue = '';

	/** @var bool */
	protected $NeedsEndTag = false;

	/** @var string	 */
	protected $Module = 'basis';

	/** @var string[]	 */
	protected $Groups = [];

	/** @var bool */
	protected $Deprecated = false;

	/** @var bool */
	protected $noDocuLink = false;

	protected function __construct(string $tagName){
		$this->Name = $tagName;
	}

	/**
	 *
	 * @param string $tagName
	 * @return we_weTag_twbase
	 * @throws we_exception_file
	 */
	private static function getInstance(string $tagName): self{
		// include the selected tag, its either normal, or custom tag
		$name = 'we_tw_' . $tagName;
		if(file_exists(WE_CLASS_PATH . 'tw/' . $name . '.class.php')){
			return new $name($tagName);
		}
		$noNotname = str_replace('Not', '', $name);
		if(file_exists(WE_CLASS_PATH . 'tw/' . $noNotname . '.class.php')){
			class_alias($noNotname, $name);
			return new $name($tagName);
		}
		if(file_exists(($path = WE_CUSTOM_PATH . self::DIR . '/' . $name . '.class.php'))){
			require_once($path);
			$ret = new $name($tagName);
			$ret->Groups[] = 'custom';
			$ret->noDocuLink = true;
			return $ret;
		}
		if(file_exists(($path = WE_CUSTOM_PATH . self::DIR . '/' . $noNotname . '.class.php'))){
			require_once($path);
			class_alias($noNotname, $name);
			$ret = new $name($tagName);
			$ret->Groups[] = 'custom';
			$ret->noDocuLink = true;
			return $ret;
		}
		throw new we_exception_file('requested help entry of tag ' . $tagName . ' not found');
	}

	private function updateTag(): void{
		if(strpos($this->Name, 'if') !== 0){
			$to = new we_weTag_choiceAttribute('to', [
				new we_weTag_option('global'),
				new we_weTag_option('request'),
				new we_weTag_option('get'),
				new we_weTag_option('post'),
				new we_weTag_option('session'),
				new we_weTag_option('top'),
				new we_weTag_option('self'),
				new we_weTag_option('block'),
				new we_weTag_option('sessionfield'),
				new we_weTag_option('screen'),
					], false, false, '');
			$nameto = new we_weTag_textAttribute('nameto', false, '');
		} else {
			$nameto = $to = null;
		}

		if($this->TypeAttribute){
			$options = $this->TypeAttribute->getOptions();
			if(!$this->noDocuLink){
				foreach($options as &$value){
					$tmp = new we_weTag_cmdAttribute('TagReferenz', false, '', ['open_tagreference', strtolower($this->Name) . ($value->Name ? '-' . $this->TypeAttribute->getName() . '-' . $value->Name : '')], '<i class="fa fa-external-link"></i> ' . g_l('taged', '[tagreference_linktext]'));
					$value->AllowedAttributes[] = $tmp;
					if($value->Value != '-'){
						$this->Attributes[] = $tmp;
					}
				}
				unset($value);
			}
			//fix common error: not all type attributes are present in attributes
			foreach($options as $value){
				$tmp = $value->AllowedAttributes;
				foreach($tmp as $cur){
					if($cur != $this->TypeAttribute && !empty($cur) && !in_array($cur, $this->Attributes, true)){
						$this->Attributes[] = $cur;
					}
				}
				if($to){
					$value->addTypeAttribute($to);
					$value->addTypeAttribute($nameto);
				}
			}
		} elseif(!$this->noDocuLink){
			$this->Attributes[] = new we_weTag_cmdAttribute('TagReferenz', false, '', ['open_tagreference', strtolower($this->Name)], g_l('taged', '[tagreference_linktext]')); // Bug #6341
		}

		if($to){
			$this->Attributes[] = $to;
			$this->Attributes[] = $nameto;
		}
		usort($this->Attributes, function (we_weTag_attribute $a, we_weTag_attribute $b): int{
			return (($a instanceof we_weTag_cmdAttribute ? 0 : ($a->IsRequired() ? 1 : 2)) . $a->getName()) <=> (($b instanceof we_weTag_cmdAttribute ? 0 : ($b->IsRequired() ? 1 : 2)) . $b->getName());
		});
	}

	private function updateUsedAttributes(): void{
		$this->UsedAttributes = [];
		if($this->TypeAttribute){
			$this->UsedAttributes[] = $this->TypeAttribute;
		}
		foreach($this->Attributes as $pos => $attr){
			if($attr === null){
				continue;
			}
			if(!is_object($attr)){
				t_e('Error in Attributes of we:' . $this->Name, $attr, $pos);
			} else if($attr->useAttribute()){
				$this->UsedAttributes[] = $attr;
			}
		}
	}

	/**
	 * @return string
	 */
	function getName(): string{
		return $this->Name;
	}

	function getModule(): string{
		return $this->Module;
	}

	function getGroups(): array{
		return $this->Groups;
	}

	function isDeprecated(): bool{
		return $this->Deprecated;
	}

	/**
	 * @return string
	 */
	function getDescription(): string{
		return $this->Description ?? '';
	}

	/**
	 * @param string $tagName
	 * @return we_weTag_twbase
	 * @throws we_exception_file
	 */
	static function getTagData(string $tagName): self{
		static $tags = [];
		if(isset($tags[$tagName])){
			$tag = $tags[$tagName];
		} else {
			$tag = self::getInstance($tagName);
			$tag->updateTag();
			$tags[$tagName] = $tag;
		}
		$tag->updateUsedAttributes();
		return $tag;
	}

	/**
	 * @return boolean
	 */
	function needsEndTag(): bool{
		return $this->NeedsEndTag;
	}

	/**
	 * @return array
	 */
	function getAllAttributes(bool $idPrefix = false): array{
		$attribs = [];
		foreach($this->UsedAttributes as $attrib){
			$attribs[] = ($idPrefix ?
					$attrib->getIdName() :
					$attrib->getName()
					);
		}
		return $attribs;
	}

	/**
	 * @return we_weTag_typeAttribute|null
	 */
	function getTypeAttribute(): ?we_weTag_typeAttribute{
		return $this->TypeAttribute;
	}

	/**
	 * @return array
	 */
	function getRequiredAttributes(): array{
		$req = [];

		foreach($this->UsedAttributes as $attrib){
			if($attrib->IsRequired()){
				$req[] = $attrib->getIdName();
			}
		}
		return $req;
	}

	/**
	 * @return array
	 */
	function getTypeAttributeOptions(): array{
		if($this->TypeAttribute){
			return $this->TypeAttribute->getOptions();
		}
		return [];
	}

	function getAttributesForCM(): array{
		$attr = [];

		foreach($this->UsedAttributes as $attribute){
			if(!$attribute->IsDeprecated() && $attribute->useAttribute() && !($attribute instanceof we_weTag_linkAttribute) && !($attribute instanceof we_weTag_cmdAttribute)){
				$attr[] = $attribute->getName();
			}
		}
		return $attr;
	}

	/**
	 * @return string
	 */
	function getAttributesCodeForTagWizard(): string{
		$ret = '';

		$typeAttrib = $this->getTypeAttribute();

		if(count($this->UsedAttributes) > 1 || (count($this->UsedAttributes) && !$typeAttrib)){

			$ret = '<ul>';
			foreach($this->UsedAttributes as $attribute){
				if($attribute !== $this->TypeAttribute){
					$ret .= '<li ' . ($typeAttrib ? 'style="display:none;"' : '') . ' id="li_' . $attribute->getIdName() . '">' . $attribute->getCodeForTagWizard() . '</li>';
				}
			}
			$ret .= '</ul>';
		}
		return $ret;
	}

	/**
	 * @return string
	 */
	function getTypeAttributeCodeForTagWizard(): string{
		if($this->TypeAttribute){
			return '<ul>' .
					'<li>' . $this->TypeAttribute->getCodeForTagWizard() . '</li>' .
					'</ul>';
		}
		return '';
	}

	/**
	 * @return string
	 */
	function getDefaultValueCodeForTagWizard(): string{

		return we_html_element::textArea([
					'name' => 'weTagData_defaultValue',
					'id' => 'weTagData_defaultValue',
					'class' => 'wetextinput wetextarea'
						], $this->DefaultValue);
	}

}
