<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: e5bf7f91c9c3 $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2019-12-02 13:06 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */

/**
 * General Definition of WebEdition Workflow Task
 */
class we_workflow_task extends we_workflow_base{
	/** @var int */
	var $ID = 0;

	/** @var int */
	var $stepID = 0;

	/** @var int */
	var $userID = 0;

	/** @var string */
	var $username = '';

	/** @var bool */
	var $Edit = false;

	/** @var bool */
	var $Mail = false;

	/**
	 * Default Constructor
	 *
	 * Can load or create new Workflow Task Definition depends of parameter
	 */
	function __construct(int $taskID = 0){
		parent::__construct(WORKFLOW_TASK_TABLE);

		$this->persistent_slots = [
			'ID' => we_base_request::INT,
			'userID' => we_base_request::INT,
			'Edit' => we_base_request::BOOL,
			'Mail' => we_base_request::BOOL,
			'stepID' => we_base_request::INT,
		];

		if($taskID){
			$this->load($taskID);
		}
	}

	/**
	 * get all workflow tasks from database (STATIC)
	 */
	public static function getAllTasks(int $stepID): array{
		$data = DB_WE::$inst->getAllq('SELECT ID FROM ' . WORKFLOW_TASK_TABLE . ' WHERE stepID=' . $stepID . ' ORDER BY ID', true);
		$tasks = [];
		foreach($data as $id){
			$tasks[] = new self((int) $id);
		}
		return $tasks;
	}

	/**
	 * Load task from database
	 */
	function load(int $id = 0, bool $loadData = true): bool{
		if($id){
			$this->ID = $id;
		}
		if($this->ID){
			parent::load();
			return true;
		}
		t_e('we_workflow_task::load: No Task with ID ' . $id . ' !');
		return false;
	}

	public function update(int $userID, string $username, int $stepID, bool $edit, bool $mail, int $id = 0): void{
		$this->stepID = $stepID;
		$this->userID = $userID;
		$this->username = $username;
		$this->Edit = $edit;
		$this->Mail = $mail;
		if($id){
			$this->ID = $id;
		}
	}

}
