<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: a5a5d3741437 $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2020-07-05 14:14 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
//make sure we know which browser is used

class we_wysiwyg_editor{
	private const htmlGalleryTag = 'we-gallery';

	/** @var string */
	private $editorType = '';

	/** @var array<string,bool|int|string> */
	private $dialogProperties = [];

	/** @var string */
	private $name = '';

	/** @var string */
	private $origName = '';

	/** @var string */
	private $fieldName = '';

	/** @var string */
	private $fieldName_clean = '';

	/** @var string */
	private $width = '600';

	/** @var string */
	private $height = '400';

	/** @var string */
	private $ref = '';

	/** @var array */
	private $propArr = [];

	/** @var we_wysiwyg_ToolbarButton[] */
	private $elementsToolbar = [];

	/** @var we_wysiwyg_ToolbarButton[] */
	private $elementsQuickbar = [];

	/** @var string */
	private $value = '';

	/** @var string */
	private $contextmenu = '';

	/** @var string */
	private $restrictContextmenu = '';

	/** @var array<string,array> */
	private $usedPlugins = ['all' => [], 'internal' => [], 'external' => []];

	/** @var string[] */
	private $externalPlugins = [
		//'wecontextmenu',
		'wegallery',
		'weimage',
		'webreak',
		'welink',
		'weutil',
		'wevisualaid',
		'wexhtmlxtras',
		'wefullscreen',
		'wesource',
		'wequickbar',
		'weselects',
		'wetable',
		'wepastetext',
	];

	/** @var string[] */
	private $internalPlugins = [
		'table',
		'directionality',
		'lists',
		'paste',
		'wordcount',
		'importcss',
		'advlist',
		'anchor',
		'charmap',
		//'code',
		'hr',
		'insertdatetime',
		'nonbreaking',
		'searchreplace',
		'template',
		//'bbcode',
		'codesample',
		//'contextmenu',
		'fullscreen',
		'visualchars',
		'visualblocks',
		'noneditable',
	];

	/** @var string[] */
	private $thirdPartyPlugins = [
		//'codemirror'
	];

	/** @var bool */
	private $createContextmenu = false;

	/** @var we_wysiwyg_ToolbarButton[] */
	private $filteredToolbar = [];

	/** @var string */
	private $bgcolor = '';

	/** @var string */
	private $tinyParamsStr = '';

	/** @var array[] */
	private $tinyParams = [];

	/** @var string */
	private $templates = '';

	/** @var string */
	private $className = '';

	/** @var bool */
	private $removeFirstParagraph = true;

	/** @var bool */
	private $inlineedit = true;

	/** @var string */
	private $cssClasses = '';

	/** @var bool */
	private $blockImportCss = false;

	/** @var string */
	private $language = '';

	/** @var string */
	private $editorLanguage = '';

	/** @var string */
	private $imagePath;

	/** @var string */
	private $image_languagePath;

	/** @var string */
	private $baseHref = '';

	/** @var bool */
	private $showSpell = true;

	/** @var bool */
	private $isFrontendEdit = false;

	/** @var bool */
	private $htmlSpecialchars = true; // in wysiwyg default was "true" (although Tag-Hilfe says "false")

	/** @var string */
	private $contentCss = '';

	/** @var int */
	private $imageStartID = 0;

	/** @var int[] */
	private $galleryTemplates = [];

	/** @var string */
	private $formats = '';

	/** @var string */
	private $toolbar = '';

	/** @var array<string,string[]> */
	private $quickbarCmds = [];

	/** @var array<string,string[]> */
	private $quickbarSubcmds = [];

	/** @var string */
	private $quickbar = '';

	/** @var string[] */
	private $usedCommands = [];

	/** @var string */
	private $selectorClass = '';

	/** @var string */
	private $fontsizes = '';

	/** @var string[] */
	private $tableToolbar = [];

	/** @var string */
	private $size = '';

	/** @var array<string,array> */
	private $selects = [
		'toolbar' => [],
		'toolbarFormats' => [],
		'toolbarSettings' => [
			'fontsizes' => [],
			'blockselect' => [],
			'styleselect' => []
		],
	];

	/** @var array */
	private static $dataConfigurations = [];

	/** @var array<string,bool|int|string> */
	private static $dataDialogProperties = [
		'isDialog' => false,
		//'weEditorType' => '',
		//'weIsFrontendEdit' => false,
		//'weFieldname' => ''
	];

	/** @var int */
	private static $countInstances = 0;

	/** @var array<string:array> */
	private const DEFAULT_BLOCKFORMATS = [
		'p' => ['title' => 'Paragraph', 'value' => 'p'],
		'div' => ['title' => 'Div', 'value' => 'div'],
		'h1' => ['title' => 'Header 1', 'value' => 'h1'],
		'h2' => ['title' => 'Header 2', 'value' => 'h2'],
		'h3' => ['title' => 'Header 3', 'value' => 'h3'],
		'h4' => ['title' => 'Header 4', 'value' => 'h4'],
		'h5' => ['title' => 'Header 5', 'value' => 'h5'],
		'h6' => ['title' => 'Header 6', 'value' => 'h6'],
		'pre' => ['title' => 'Pre', 'value' => 'pre'],
	];

	/** @var array<string:array> */
	private const DEFAULT_FONTSIZES = [
		'0.5em' => ['title' => '0.5em', 'value' => '0.5em'],
		'0.8em' => ['title' => '0.8em', 'value' => '0.8em'],
		'1em' => ['title' => '1em', 'value' => '1em'],
		'1.2em' => ['title' => '1.2em', 'value' => '1.2em'],
		'1.5em' => ['title' => '1.5em', 'value' => '1.5em'],
		'2em' => ['title' => '2em', 'value' => '2em'],
		'8px' => ['title' => '8px', 'value' => '8px'],
		'10px' => ['title' => '10px', 'value' => '10px'],
		'12px' => ['title' => '12px', 'value' => '12px'],
		'14px' => ['title' => '14px', 'value' => '14px'],
		'18px' => ['title' => '18px', 'value' => '18px'],
		'24px' => ['title' => '24px', 'value' => '24px'],
		'36px' => ['title' => '36px', 'value' => '36px'],
		'xx-small' => ['title' => 'xx-small', 'value' => 'xx-small'],
		'x-small' => ['title' => 'x-small', 'value' => 'x-small'],
		'small' => ['title' => 'small', 'value' => 'small'],
		'medium' => ['title' => 'medium', 'value' => 'medium'],
		'large' => ['title' => 'large', 'value' => 'large'],
		'x-large' => ['title' => 'x-large', 'value' => 'x-large'],
		'xx-large' => ['title' => 'xx-large', 'value' => 'xx-large'],
		'smaller' => ['title' => 'smaller', 'value' => 'smaller'],
		'larger' => ['title' => 'larger', 'value' => 'larger'],
		'inherit' => ['title' => 'inherit', 'value' => 'inherit'],
	];
	private const LEGACY_COMMANDS = [
		'applystyle' => 'styleselect',
		'createlink' => 'link',
		'fontsize' => 'sizeselect',
		'formatblock' => 'blockselect',
		'insertbreak' => 'break',
		'insertgallery' => 'gallery',
		'inserthorizontalrule' => 'hr',
		'insertimage' => 'image',
		'insertorderedlist' => 'numlist',
		'insertunorderedlist' => 'bullist',
		'insertspecialchar' => 'charmap',
		'justifycenter' => 'aligncenter',
		'justifyfull' => 'alignjustify',
		'justifyleft' => 'alignleft',
		'justifyright' => 'alignright',
		'removeformat' => 'removeformat',
		'visibleborders' => 'visualaid',
		'replace' => 'search',
		'searchreplace' => 'search',
		'decreasecolspan' => 'table',
		'deletecol' => 'table',
		'deleterow' => 'table',
		'deletetable' => 'table',
		'editcell' => 'table',
		'editrow' => 'table',
		'increasecolspan' => 'table',
		'insertcolumnleft' => 'table',
		'insertcolumnright' => 'table',
		'insertrowabove' => 'table',
		'insertrowbelow' => 'table',
		'inserttable' => 'table',
		// dissolved or renamed groups
		'font' => 'sizeselect',
		'copypaste' => 'pastetext',
		'justify' => 'align',
	];
	private const COMMAND_GROUPS = [
		'advanced' => [
			'codesample',
			'editsource',
			'template',
		],
		'color' => [
			'backcolor',
			'forecolor',
		],
		'essential' => [
			'charmap',
			'fullscreen',
			'pastetext',
			'redo',
			'search',
			'selectall',
			'undo',
			'visualaid',
		],
		'insert' => [
			'break',
			'gallery',
			'hr',
			'image',
			'link',
			'nonbreaking',
			'table',
		],
		'align' => [
			'aligncenter',
			'alignjustify',
			'alignleft',
			'alignright',
		],
		'list' => [
			'blockquote',
			'indent',
			'numlist',
			'bullist',
			'outdent',
		],
		'prop' => [
			'blockselect',
			'bold',
			'italic',
			'removeformat',
			'sizeselect',
			'strikethrough',
			'styleselect',
			'subscript',
			'superscript',
			'underline',
		],
		'xhtmlxtras' => [
			'abbr',
			'acronym',
			'cite',
			'del',
			'ins',
			'lang',
			'ltr',
			'rtl',
		],
	];
	private const TABLE_COMMANDS = [
		'tableprops',
		'tabledelete',
		'|',
		'tablecellprops',
		'|',
		'tablerowprops',
		'tableinsertrowbefore',
		'tableinsertrowafter',
		'tabledeleterow',
		'|',
		'tableinsertcolbefore',
		'tableinsertcolafter',
		'tabledeletecol',
		/*
		  'tablesplitcells ',
		  'tablemergecells',
		  'tablemenucell',
		  'tablemenucolumn',
		  'tablemenurow',
		  'tablerowcopypaste',
		  'tablecutrow',
		  'tablecopyrow',
		  'tablepasterowbefore',
		  'tablepasterowafter',
		 *
		 */
	];
	private const FIELD_CLEAN = ['-' => '_minus_', '.' => '_dot_', '#' => '_sharp_', ' ' => '_blank_'];
	const TYPE_INLINE_TRUE = 'inlineTrue';
	const TYPE_INLINE_FALSE = 'inlineFalse';
	const TYPE_EDITBUTTON = 'editButton';

	function __construct(array $dialogProperties = [], string $editorType = '', string $name = '', $width = '600', $height = '400', string $value = '', string $propString = '', string $bgcolor = '', string $className = '', string $obsolete_1 = '', bool $removeFirstParagraph = true, bool $inlineedit = true, string $baseHref = '', string $cssClasses = '', string $language = '', bool $spell = true, bool $isFrontendEdit = false, string $obsolete_2 = 'top', bool $oldHtmlspecialchars = true, string $contentCss = '', string $origName = '', string $tinyParams = '', string $contextmenu = '', string $templates = '', string $formats = '', int $imageStartID = 0, array $galleryTemplates = [
		], string $fontsizes = '', string $obsolete_3 = '', string $size = 'medium'){
		$this->editorType = $editorType ?: self::TYPE_INLINE_TRUE;
		$this->dialogProperties = $dialogProperties;

		if($this->editorType === self::TYPE_INLINE_FALSE){
			if(empty($this->dialogProperties)){
				t_e('attempt to initialize wysiwyg editor type "' . $this->editorType . '" without dialogProperties');
				//exit();
			}

			// we only init props that we need in function getHTMLEditor() when loading tiny to popup/overlay;
			$this->width = '100%';
			$this->height = '100%';
			$this->name = $this->dialogProperties['weName'];
			$this->selectorClass = $this->dialogProperties['weSelectorClass'];
			$this->isFrontendEdit = $this->dialogProperties['weIsFrontendEdit'];
			self::$dataDialogProperties = ['isDialog' => true, 'weEditorType' => $this->editorType, 'weIsFrontendEdit' => $this->isFrontendEdit, 'weFieldname' => $this->dialogProperties['weFieldName']];
		} else {
			$this->name = $name;
			$this->width = $width;
			$this->height = $height;
			$this->size = $size ?: 'fullscreen';
			$this->value = $value;
			$this->propArr = array_merge(we_base_request::filterStringList($propString));
			$this->bgcolor = $bgcolor;
			$this->className = $className;
			$this->removeFirstParagraph = $removeFirstParagraph;
			$this->inlineedit = $inlineedit; // replace by editorType
			$this->baseHref = $baseHref;
			$this->cssClasses = trim($cssClasses);
			$this->language = $language;
			$this->showSpell = $spell;
			$this->isFrontendEdit = $isFrontendEdit;
			$this->htmlSpecialchars = $oldHtmlspecialchars;
			$this->contentCss = $contentCss;
			$this->origName = $origName; // obsolete?
			$this->tinyParamsStr = $tinyParams;
			$this->contextmenu = $contextmenu;
			$this->templates = $templates;
			$this->formats = $formats;
			$this->imageStartID = $imageStartID;
			$this->galleryTemplates = $galleryTemplates;
			$this->fontsizes = $fontsizes;

			$this->preprocessProps();
			$this->initializeCommands();
			$this->applyFiltersToCommands();
			$this->processToolbar();
			$this->processQuickbar();
			$this->processTableToolbar();
			foreach(['fontsizeselect', 'blockselect'] as $what){
				$this->processSelect($what);
			}
			$this->processStyleselect();
			$this->processPlugins();
		}
	}

	private function preprocessProps(): void{
		$this->preprocessCommands();

		if(preg_match('|^.+\[.+\]$|i', $this->name)){
			$this->fieldName_clean = strtr($this->fieldName, self::FIELD_CLEAN);
		}
		if(preg_match('%^.+_te?xt\[.+\]$%i', $this->name)){
			$this->fieldName = preg_replace('/^.+_te?xt\[(.+)\]$/', '${1}', $this->name);
		} else if(preg_match('|^.+_input\[.+\]$|i', $this->name)){
			$this->fieldName = preg_replace('/^.+_input\[(.+)\]$/', '${1}', $this->name);
		} else if(preg_match('|^we_ui.+\[.+\]$|i', $this->name)){//we_user_input
			$this->fieldName = preg_replace('/^we_ui.+\[(.+)\]$/', '${1}', $this->name);
		}
		$this->fieldName_clean = strtr($this->fieldName, self::FIELD_CLEAN);
		$this->selectorClass = strtr($this->name, ['[' => '_', ']' => '']);
		$this->width = $this->width ? (is_numeric(($width = we_base_util::convertUnits((string) $this->width))) ? $width . 'px' : $width) : 0;
		$this->height = $this->height ? (is_numeric(($height = we_base_util::convertUnits((string) $this->height))) ? $height . 'px' : $height) : 0;

		if(preg_match('/^#[a-f0-9]{6}$/i', $this->bgcolor)){
			$this->bgcolor = substr($this->bgcolor, 1);
		} else if(!preg_match('/^[a-f0-9]{6}$/i', $this->bgcolor) && !preg_match('/^[a-z]*$/i', $this->bgcolor)){
			$this->bgcolor = '';
		}

		$this->contentCss = ($this->contentCss === '/' ? '' : $this->contentCss);

		$this->tinyParams = array_filter(array_map(function(string $el): array{
				$parts = explode(':', $el, 2);
				if(count(($parts = explode(':', $el, 2))) === 2){
					[$k, $v] = $parts;
					return ['name' => $k, 'value' => trim($v, "'")];
				}
				return [];
			}, array_filter(explode("',", trim(str_replace(' ', '', $this->tinyParamsStr), ' ,')))));

		$this->restrictContextmenu = $this->contextmenu ? ',' . urldecode($this->contextmenu) . ',' : '';
		$this->createContextmenu = !(trim($this->contextmenu, " ,'") === 'none' || trim($this->contextmenu, " ,'") === 'false');
		$this->templates = trim($this->templates, ',');

		$this->imageStartID = intval($this->imageStartID);

		$templ = (array_filter(we_base_request::filterIntList($this->galleryTemplates)));

		$this->galleryTemplates = $templ ? DB_WE::$inst->getAllq('SELECT ID FROM ' . TEMPLATES_TABLE . ' WHERE ID IN(' . implode(',', $templ) . ')', true) : [];

		$this->imagePath = IMAGE_DIR . 'wysiwyg/';
		$this->image_languagePath = WE_INCLUDES_DIR . 'language/' . we_base_WE::$Language . '/wysiwyg/';
		$this->ref = preg_replace('%[^0-9a-zA-Z_]%', '', $this->name);

		// editorLanguage: in backend we allways use backend language. in frontend we take doc language if defined, if not we use weDefaultFrontendLanguage
		[$docLang] = explode('_', $this->language);
		[$defaultFrontendLang] = explode('_', we_base_WE::$Langs['DefaultFrontendLanguage']);
		$this->editorLanguage = $this->isFrontendEdit ? ($docLang ?: $defaultFrontendLang) : (($lang = (array_search(we_base_WE::$Language, we_base_country::getWELangs(), true))) ? $lang : 'de');

		//FIXME: what to do with scripts??
	}

	public function getIsFrontend(): bool{
		return $this->isFrontendEdit;
	}

	public function isGallerytemplate(): bool{
		return !empty($this->galleryTemplates);
	}

	public function getPropArr(): array{
		return $this->propArr;
	}

	public function getRestrictContextmenu(): string{
		return $this->restrictContextmenu;
	}

	public static function getEditorCommands(bool $isTag): array{
		$tmp = array_keys(self::COMMAND_GROUPS);
		if($isTag){
			$ret = [new we_weTag_option(g_l('wysiwyg', '[groups]'), we_html_tools::OPTGROUP)];

			foreach($tmp as $command){
				$ret[] = new we_weTag_option($command);
			}
			foreach(self::COMMAND_GROUPS as $key => $values){
				$ret[] = new we_weTag_option($key, we_html_tools::OPTGROUP);
				foreach($values as $value){
					$ret[] = new we_weTag_option($value);
				}
			}

			return $ret;
		}

		$ret = array_merge(['', g_l('wysiwyg', '[groups]') => we_html_tools::OPTGROUP], $tmp);
		foreach(self::COMMAND_GROUPS as $key => $values){
			$ret = array_merge($ret, [$key => we_html_tools::OPTGROUP], $values);
		}
		return $ret;
	}

	public static function getAttributeOptions(string $name = '', bool $isTag = false, bool $leadingEmpty = true): array{
		switch($name){
			case 'formats':
				$options = array_keys(self::DEFAULT_BLOCKFORMATS);
				break;
			case 'fontsizes':
				$options = array_keys(self::DEFAULT_FONTSIZES);
				break;
			default:
				return [];
		}

		if($isTag){
			foreach($options as &$opt){
				$opt = new we_weTag_option($opt);
			}
			return $options;
		}

		if($leadingEmpty){
			array_unshift($options, '---');
		}

		return $options;
	}

	static function getHTMLHeader(bool $frontendEdit = false, bool $loadConfigs = false, bool $inlineEdit = false): string{
		if(defined('WE_WYSIWG_HEADER')){
			return '';
		}
		define('WE_WYSIWG_HEADER', 1);

		$frontendHeader = '';
		if($frontendEdit){
			//FIXME: all headers should be added at a central instance in header (we_templateHead).
			if(!defined('WE_FRONTEND_EDIT_HEADER')){
				define('WE_FRONTEND_EDIT_HEADER', 1);
				$frontendHeader .= we_html_element::jsScript(JS_DIR . 'weFrontendEdit_header.js') .
					we_html_element::jsScript(JS_DIR . 'windows.js') .
					we_html_element::jsScript(JS_DIR . 'overlay.js') .
					we_html_tools::getStyleSheet(true) .
					we_html_element::cssLink(CSS_DIR . 'overlay.css', ['rel' => 'preload', 'as' => 'style']) .
					we_html_element::jsScript(WE_LIB_DIR . 'jquery/jquery.js', '', ['rel' => 'preload', 'as' => 'script']) .
					we_html_element::jsScript(WE_JS_TINYMCE_DIR . 'weTinyMce_functionsTop.js', '', ['id' => 'loadWeicons',
						'data-weicons' => setDynamicVar(we_wysiwyg_editor::getWeicons())
				]);
			}
			if($inlineEdit && !defined('WE_FRONTEND_WYSIWYG_INLINE_HEADER')){
				define('WE_FRONTEND_WYSIWYG_INLINE_HEADER', 1);
				//$frontendHeader .= we_html_tools::getStyleSheet(true);
			}
			if(!$inlineEdit && !defined('WE_FRONTEND_WYSIWYG_EDITBBUTTON_HEADER')){
				define('WE_FRONTEND_WYSIWYG_EDITBBUTTON_HEADER', 1);
				//$frontendHeader .= we_html_element::cssLink(CSS_DIR . 'we_button.css', ['rel' => 'preload', 'as' => 'style']);
			}



			$frontendHeader .= we_html_element::jsScript(WE_JS_TINYMCE_DIR . 'weTinyMce_config.js', '', ['id' => 'loadVarWeTinyMce_config', 'data-consts' => setDynamicVar(self::getFrontendHeaderConsts())]); // .
			//we_html_element::jsScript(WE_JS_TINYMCE_DIR . 'weTinyMce_functionsTop.js');
		}

		return we_html_element::cssLink(CSS_DIR . 'wysiwyg/tinymce/toolbar.css') .
			we_html_element::cssLink(CSS_DIR . 'wysiwyg/tinymce/tinyDialogCss.css') .
			we_html_element::jsScript(TINYMCE_SRC_DIR . 'tinymce.js') .
			($frontendEdit ? $frontendHeader : '') .
			we_html_element::jsScript(WE_JS_TINYMCE_DIR . 'weTinyMce_init.js', '', ($loadConfigs ? ['id' => 'loadVar_tinyConfigs',
				'data-dialogProperties' => setDynamicVar(self::$dataDialogProperties),
				'data-configurations' => setDynamicVar(self::$dataConfigurations),
				] : []));
	}

	public static function getAllCmds(): array{
		$allCmds = [];
		foreach(self::COMMAND_GROUPS as $group){
			$allCmds = array_merge($allCmds, $group);
		}
		sort($allCmds);

		return $allCmds;
	}

	private function initializeCommands(): void{// TODO: declare setToolbarElements
		$quickbar = we_wysiwyg_ToolbarElement::SCOPE_QUICKBAR;
		$this->elementsQuickbar = array_filter([
			new we_wysiwyg_ToolbarButton($this, 'bold', $quickbar, 'format'),
			new we_wysiwyg_ToolbarButton($this, 'italic', $quickbar, 'format'),
			new we_wysiwyg_ToolbarButton($this, 'underline', $quickbar, 'format'),
			new we_wysiwyg_ToolbarButton($this, 'subscript', $quickbar, 'format'),
			new we_wysiwyg_ToolbarButton($this, 'superscript', $quickbar, 'format'),
			new we_wysiwyg_ToolbarButton($this, 'strikethrough', $quickbar, 'format'),
			new we_wysiwyg_ToolbarButton($this, 'styleprops', $quickbar, 'format'),
			new we_wysiwyg_ToolbarButton($this, 'removeformat', $quickbar, 'rmformat'),
			new we_wysiwyg_ToolbarButton($this, 'link', $quickbar, 'link'),
			new we_wysiwyg_ToolbarButton($this, 'unlink', $quickbar, 'link'),
			new we_wysiwyg_ToolbarButton($this, 'anchor', $quickbar, 'link'),
			new we_wysiwyg_ToolbarButton($this, 'forecolor', $quickbar, 'color'),
			new we_wysiwyg_ToolbarButton($this, 'backcolor', $quickbar, 'color'),
			new we_wysiwyg_ToolbarButton($this, 'lang', $quickbar, 'xhtml'),
			new we_wysiwyg_ToolbarButton($this, 'cite', $quickbar, 'xhtml'),
			new we_wysiwyg_ToolbarButton($this, 'acronym', $quickbar, 'xhtml'),
			new we_wysiwyg_ToolbarButton($this, 'abbr', $quickbar, 'xhtml'),
			new we_wysiwyg_ToolbarButton($this, 'ins', $quickbar, 'xhtml'),
			new we_wysiwyg_ToolbarButton($this, 'del', $quickbar, 'xhtml'),
		]);

		$sep = new we_wysiwyg_ToolbarSeparator($this, '');
		$this->elementsToolbar = array_filter([
			new we_wysiwyg_ToolbarButton($this, 'sizeselect'),
			new we_wysiwyg_ToolbarButton($this, 'blockselect'),
			new we_wysiwyg_ToolbarButton($this, 'styleselect'),
			$sep,
			new we_wysiwyg_ToolbarButton($this, 'bullist'),
			new we_wysiwyg_ToolbarButton($this, 'numlist'),
			$sep,
			new we_wysiwyg_ToolbarButton($this, 'alignleft'),
			new we_wysiwyg_ToolbarButton($this, 'aligncenter'),
			new we_wysiwyg_ToolbarButton($this, 'alignright'),
			new we_wysiwyg_ToolbarButton($this, 'alignjustify'),
			new we_wysiwyg_ToolbarButton($this, 'indent'),
			new we_wysiwyg_ToolbarButton($this, 'outdent'),
			new we_wysiwyg_ToolbarButton($this, 'blockquote'),
			$sep,
			new we_wysiwyg_ToolbarButton($this, 'tableinsert'),
			new we_wysiwyg_ToolbarButton($this, 'tablemenu'),
			new we_wysiwyg_ToolbarButton($this, 'image'),
			new we_wysiwyg_ToolbarButton($this, 'gallery'),
			new we_wysiwyg_ToolbarButton($this, 'template'),
			new we_wysiwyg_ToolbarButton($this, 'codesample'),
			$sep,
			new we_wysiwyg_ToolbarButton($this, 'charmap'),
			new we_wysiwyg_ToolbarButton($this, 'break'),
			new we_wysiwyg_ToolbarButton($this, 'nonbreaking'),
			new we_wysiwyg_ToolbarButton($this, 'hr'),
			$sep,
			new we_wysiwyg_ToolbarButton($this, 'ltr'),
			new we_wysiwyg_ToolbarButton($this, 'rtl'),
			$sep,
			new we_wysiwyg_ToolbarButton($this, 'pastetext'),
			new we_wysiwyg_ToolbarButton($this, 'undo'),
			new we_wysiwyg_ToolbarButton($this, 'redo'),
			new we_wysiwyg_ToolbarButton($this, 'selectall'),
			new we_wysiwyg_ToolbarButton($this, 'search'),
			new we_wysiwyg_ToolbarButton($this, 'fullscreen'),
			new we_wysiwyg_ToolbarButton($this, 'visualaid'),
			new we_wysiwyg_ToolbarButton($this, 'editsource'),
		]);
	}

	private function applyFiltersToCommands(): void{
		$lastSep = true;
		foreach($this->elementsToolbar as $elem){
			if($elem->isShowInToolbar()){
				if((!$lastSep) || !($elem->isSeparator())){
					$this->filteredToolbar[] = $elem;
				}
				$lastSep = ($elem->isSeparator());
			}
		}
		if($this->filteredToolbar){
			$el = end($this->filteredToolbar);
			if($el && $el->isSeparator()){
				array_pop($this->filteredToolbar);
			}
		}
	}

	/**
	 *
	 * @staticvar int $t stores current time
	 * @param string $value
	 * @return string
	 */
	static function parseInternalImageSrc(string $value): string{
		static $t = 0;
		$t = ($t ?: time());
		$regs = [];

		// IMPORTANT: we process tiny content both from db and session: the latter uses paths?id=xy instead of document:xy
		if(preg_match_all('/<img [^>]*(src="(' . we_base_link::TYPE_INT_PREFIX . '|[^" >]*\?id=)(\d+)[^"]*")[^>]*>/i', $value, $regs, PREG_SET_ORDER)){
			$ids = [];
			foreach($regs as $reg){
				$ids[] = intval($reg[3]);
			}
			$ids = array_unique(array_filter($ids));
			$lookup = ($ids ?
				DB_WE::$inst->getAllFirstq('SELECT ID,Path FROM ' . FILE_TABLE . ' WHERE ID IN (' . implode(',', $ids) . ')', false) :
				[]);

			foreach($regs as $reg){
				$path = empty($lookup[intval($reg[3])]) ? '' : $lookup[intval($reg[3])];
				$value = (string) ($path ?
					str_replace($reg[1], 'src="' . $path . '?id=' . $reg[3] . '&time=' . $t . '"', $value) :
					str_replace($reg[0], '<img src="' . ICON_DIR . 'no_image.gif?id=0">', $value));
			}
		}

		if(preg_match_all('/<img [^>]*(src="(' . we_base_link::TYPE_THUMB_PREFIX . '|[^" >]*\?thumb=)(\d+,\d+)[^"]*")[^>]*>/i', $value, $regs, PREG_SET_ORDER)){
			foreach($regs as $reg){
				[$imgID, $thumbID] = explode(',', $reg[3]);
				$thumbObj = new we_contents_thumbnail();
				$imageExists = $thumbObj->initByImageIDAndThumbID(intval($imgID), intval($thumbID));
				$value = (string) ($imageExists ?
					str_replace($reg[1], 'src="' . $thumbObj->getOutputPath() . "?thumb=" . $reg[3] . '&time=' . $t . '"', $value) :
					str_replace($reg[0], '<img src="' . ICON_DIR . 'no_image.gif?id=0">', $value));
				unset($thumbObj);
			}
		}

		return $value;
	}

	public static function parseGalleryTags(string $content): string{
		if(!strpos($content, '</' . self::htmlGalleryTag . '>')){
			return $content;
		}

		$gallery = false;
		$splitVal = preg_split('&(<' . self::htmlGalleryTag . ')([^>]*)>[^<]*</' . self::htmlGalleryTag . '>&i', $content, -1, PREG_SPLIT_DELIM_CAPTURE);

		ob_start();
		foreach($splitVal as $split){
			if($split === '<' . self::htmlGalleryTag){
				$gallery = true;
				continue;
			}
			if($gallery){
				$gallery = false;
				$galleryAttribs = we_weTag_parser::parseAttribs($split);
				if($galleryAttribs['id'] && $galleryAttribs['tmpl']){
					we_tag_listview::$collectionID = $galleryAttribs['id'];
					if(($we_inc = we_weTag_util::tag('include', ['type' => 'template', 'id' => intval($galleryAttribs['tmpl']), '_parsed' => true]))){
						include($we_inc);
					}
					we_tag_listview::$collectionID = 0;
				}
				continue;
			}

			echo $split;
		}
		return ob_get_clean() ?: '';
	}

	/**
	 * this function is used to prepare textaea content for db
	 * it returns an array of img/href-ids (for use in registerMediaLinks)
	 */
	public static function reparseInternalLinks(string $content): string{// FIXME: move to we_document?
		$regs = [];

		// replace real links by internals when necessery (in documents links are parsed already, in modules not)
		if(preg_match_all('{src="/[^">]+\\?id=(\\d+)(&amp;|&)?("|[^"]+")}i', $content, $regs, PREG_SET_ORDER)){
			foreach($regs as $reg){
				$content = str_replace($reg[0], 'src="' . we_base_link::TYPE_INT_PREFIX . $reg[1] . '"', $content);
			}
		}
		if(preg_match_all('{src="/[^">]+\\?thumb=(\\d+,\\d+)(&amp;|&)?("|[^"]+")}i', $content, $regs, PREG_SET_ORDER)){
			foreach($regs as $reg){
				$content = str_replace($reg[0], 'src="' . we_base_link::TYPE_THUMB_PREFIX . $reg[1] . '"', $content);
			}
		}

		// replace original content if necessary
		return $content;
	}

	/**
	 * @param string $origContent
	 * @param string $name
	 * @return array	 it returns an array of img/href-ids (for use in registerMediaLinks)

	 */
	public static function getAllMediaLinks(string $origContent, string $name = ''): array{
		$regs = $ret = $internalIDs = [];
		// FIXME: the obove stuff has to be done before parsing medialinks!
		// FIXME: make we_wysiwyg_editor::registerMedisLinks() just looking for IDs and writing the correct MediaLinks (after making internal link elsewhere)!
		// parse internal IDs in one step to preserve order!
		$content = str_replace(we_base_link::TYPE_THUMB_PREFIX, we_base_link::TYPE_INT_PREFIX, self::reparseInternalLinks($origContent));
		if(preg_match_all('/(src|href)="' . we_base_link::TYPE_INT_PREFIX . '(\\d+),?(\\d*)["|?]/i', $content, $regs, PREG_SET_ORDER)){
			foreach($regs as $reg){
				$internalIDs[] = intval($reg[2]);
			}
		}

		$c = 0;
		foreach($internalIDs as $id){
			$ret['textarea[name=' . $name . '] [src/href ' . ++$c . ']'] = $id;
		}

		return $ret;
	}

	private function processToolbar(): void{
		$tmpElements = $this->filteredToolbar;
		$toolbarStr = '';
		$allCmds = [];
		foreach($tmpElements as $elem){
			if(!$elem->cmd){
				$toolbarStr .= '| ';
			} else {
				if(($cmd = self::tinyCmdToWE($elem->cmd)) && !in_array($cmd, $allCmds, true)){
					$toolbarStr .= $cmd . ' ';
					$allCmds[] = $cmd;
				}
			}
		}

		$this->toolbar = str_replace(['insertdatetime insertdatetime', 'blockselect'], ['insertdatetime', 'formatselect'], trim($toolbarStr));
		$this->usedCommands = $allCmds;
	}

	private function processQuickbar(): void{
		$quickbarCmds = [];
		foreach($this->elementsQuickbar as $elem){
			if($elem->isShowInQuickbar() && ($group = $elem->getGroup())){
				$quickbarCmds[$group] = $quickbarCmds[$group] ?? [];
				$quickbarCmds[$group][] = self::tinyCmdToWE($elem->cmd);
			}
		}

		if(count($quickbarCmds, COUNT_RECURSIVE) - count($quickbarCmds) > 6){
			$this->setPlugin('wequickbar', true);
			if(!empty($quickbarCmds['xhtml'])){
				$first = array_shift($quickbarCmds['xhtml']);
				$this->quickbarSubcmds['xhtml'] = $quickbarCmds['xhtml'];
				$quickbarCmds['xhtml'] = [$first, 'morexhtml'];
			}
			if(!empty($quickbarCmds['format']) && count($quickbarCmds['format']) > 4){
				$this->quickbarSubcmds['format'] = array_slice($quickbarCmds['format'], 3);
				$quickbarCmds['format'] = array_merge(array_slice($quickbarCmds['format'], 0, 3), ['moreformat']);
			}
		}

		foreach($quickbarCmds as $k => $v){
			$quickbarCmds[$k] = implode(' ', $v);
		}

		$this->quickbar = implode(' | ', $quickbarCmds);
	}

	private function processStyleselect(): void{
		$what = 'styleselect';
		if(!$this->cssClasses){
			return;
		}
		if(in_array('styleselect', $this->usedCommands, true) && $this->cssClasses && $this->cssClasses[0] === '['){
			if(($items = we_unserialize(str_replace("'", '"', $this->cssClasses)))){
				if(in_array($what, $this->usedCommands, true)){
					$this->selects['toolbar'][] = $what;
					$this->selects['toolbarSettings'][$what] = $items;
				}
			}
			return;
		}

		$this->processSelect($what);
	}

	private function processSelect(string $what = ''): void{
		if(in_array($what, $this->usedCommands, true)){
			switch($what){
				case 'fontsizeselect':
					$conf = ['defaults' => self::DEFAULT_FONTSIZES, 'attrib' => $this->fontsizes, 'del' => ' ', 'title' => g_l('wysiwyg', '[fontsize]')];
					break;
				case 'blockselect':
					$conf = ['defaults' => self::DEFAULT_BLOCKFORMATS, 'attrib' => $this->formats, 'del' => ';', 'title' => 'Textformat'];
					break;
				case 'styleselect':
					$conf = ['defaults' => [], 'attrib' => $this->cssClasses, 'del' => '', 'title' => 'CSS-Klassen'];
					break;
				default:
					return;
			}

			$rawEntries = [];
			$toolbarEntries = [];
			$formats = [];

			if($conf['attrib']){
				foreach(explode(',', $conf['attrib']) as $val){
					$raw = trim($val);
					if(strpos($raw, '=') !== false){
						[$title, $value] = explode('=', $raw, 2);
						$rawEntries[] = ['title' => $title, 'value' => $value];
					} else {
						if(isset($conf['defaults'][$raw])){
							$rawEntries[] = ['title' => $conf['defaults'][$raw]['title'], 'value' => $conf['defaults'][$raw]['value']];
						} else {
							$rawEntries[] = ['title' => ($what === 'styleselect' ? $raw : ucfirst($raw)), 'value' => $raw];
						}
					}
				}
			} else {
				$rawEntries = array_values($conf['defaults']);
			}

			foreach($rawEntries as $entry){
				$title = trim($entry['title']);
				$value = trim($entry['value']);

				switch($what){
					case 'fontsizeselect':
						$toolbarEntries[] = $value;
						$formats[] = $value;
						break;
					case 'blockselect':
						$toolbarEntries[] = $title . '=' . $value;
						$formats[] = $value;
						break;
					case 'styleselect':
						$selector = '';
						$inline = 'span';
						$parts = explode('.', $value, 2);
						if(!empty($parts[1])){
							$selector = strtolower(trim($parts[0]));
							$value = trim($parts[1]);
							$inline = '';
						}
						$toolbarEntries[] = ['title' => $title, 'selector' => $selector, 'inline' => $inline, 'classes' => $value];
						break;
				}
			}

			if(in_array($what, $this->usedCommands, true) && $toolbarEntries){
				$this->selects['toolbar'][] = $what;
				$this->selects['toolbarSettings'][$what] = $what === 'styleselect' ? $toolbarEntries : implode($conf['del'], $toolbarEntries);
				$this->selects['toolbarFormats'][$what] = $formats;
			}
		}
	}

	function processTableToolbar(): void{
		$this->tableToolbar = self::TABLE_COMMANDS;
	}

	private function preprocessCommands(): void{
		$tmpCmds = [];
		foreach($this->propArr as $cmd){
			$tmpCmds[] = self::LEGACY_COMMANDS[$cmd] ?? $cmd;
		}

		$this->propArr = array_intersect($tmpCmds, array_merge(array_keys(self::COMMAND_GROUPS), $this->getAllCmds()));
	}

	private static function tinyCmdToWE(string $cmd): string{
		$cmdMapping = [
			'abbr' => 'weabbr',
			'acronym' => 'weacronym',
			'break' => 'webreak',
			'fullscreen' => 'wefullscreen',
			'gallery' => 'wegallery',
			'image' => 'weimage',
			'lang' => 'welang',
			'link' => 'welink',
			'search' => 'searchreplace',
			'sizeselect' => 'fontsizeselect',
			'unlink' => 'weunlink',
			'visualaid' => 'wevisualaid',
			'editsource' => 'wesource',
			//'editsource' => 'code',
			'tableinsert' => 'wetableinsert',
			'tablemenu' => 'wetablemenu',
			'pastetext' => 'wepastetext',
		];

		return $cmdMapping[$cmd] ?? $cmd;
	}

	function setPlugin(string $name, bool $doSet): bool{
		if($doSet){
			$this->usedPlugins['all'][] = $name;
		}
		return $doSet;
	}

	private function processPlugins(): void{
		$this->usedPlugins['all'] = array_filter(array_unique(array_merge($this->usedPlugins['all'], ['weutil', 'wordcount', 'noneditable', 'paste', ($this->cssClasses ? '' : 'importcss')])));
		foreach($this->usedPlugins['all'] as $plugin){
			if(in_array($plugin, $this->externalPlugins, true)){
				$this->usedPlugins['external'][$plugin] = WE_JS_TINYMCE_DIR . 'plugins/' . $plugin . '/plugin.js';
				continue;
			}
			if(in_array($plugin, $this->internalPlugins, true)){
				$this->usedPlugins['internal'][] = $plugin;
				continue;
			}
			if(in_array($plugin, $this->thirdPartyPlugins, true)){
				$this->usedPlugins['external'][$plugin] = TINYMCE_SRC_DIR . 'thirdpartyPlugins/' . $plugin . '/plugin.js';
				continue;
			}
		}
	}

	private function getTemplates(): array{
		if(!$this->templates){
			return [];
		}

		//FIXME: the ParentID query will only hold for depth 1 folders
		$templates = array_filter(we_base_request::filterIntList($this->templates));
		if(!$templates){
			return [];
		}
		$tmp = implode(',', $templates);
		$tmplArr = DB_WE::$inst->getAllq('SELECT ID,ContentType FROM ' . FILE_TABLE . ' WHERE (ID IN (' . $tmp . ') OR ParentID IN (' . $tmp . ') ) AND IsPublished=1 AND isFolder=0 AND (ContentType="' . we_base_ContentTypes::WEDOCUMENT . '" OR (ContentType="' . we_base_ContentTypes::APPLICATION . '" AND Extension IN(".html",".htm") ))', false);

		$ret = [];
		foreach($tmplArr as $i => $cur){
			['ID' => $id, 'ContentType' => $ct] = $cur;
			$tmplDoc = we_base_ContentTypes::getObject($ct, $id);
			assert($tmplDoc instanceof we_document_files);
			$ret[] = [
				'title' => $tmplDoc->getElementS(we_contents_root::TITLE_FIELD, 'no title ' . ($i + 1), true),
				'src' => $tmplDoc->Path . '?pub=' . ($tmplDoc->Publish_Date ? $tmplDoc->Publish_Date->format('U') : 0),
				'description' => $tmplDoc->getElementS(we_contents_root::DESCRIPTION_FIELD, 'no description ' . ($i + 1), true)
			];
		}

		return $ret;
	}

	private function getPropertiesDialog(string $forEditorType = ''): array{
		return [
			'weEditorType' => $forEditorType,
			'weName' => $this->name,
			'weFieldName' => $this->fieldName,
			'weSelectorClass' => $this->selectorClass,
			'weIsFrontendEdit' => $this->isFrontendEdit
		];
	}

	private function getPropertiesEditor(): array{
		return [
			'weEditorType' => $this->editorType,
			'weIsFrontend' => $this->isFrontendEdit,
			'weName' => $this->name,
			'weOrigName' => $this->origName,
			'weFieldName' => $this->fieldName,
			'weFieldNameClean' => $this->fieldName_clean,
			'weImageStartID' => intval($this->imageStartID),
			'weGalleryTemplates' => implode(',', $this->galleryTemplates),
			'weRemoveFirstParagraph' => $this->removeFirstParagraph,
			'weTinyParams' => $this->tinyParams,
			'weContentCssParts' => [
				'start' => implode(',', [we_html_element::getUnCache(WE_LIB_DIR . 'fontawesome5/css/all.css'),
					we_html_element::getUnCache(WE_LIB_DIR . 'fontawesome5/css/v4-shims.css'),
					we_html_element::getUnCache(CSS_DIR . 'wysiwyg/tinymce/contentCssFirst.php') . '&tinyMceBackgroundColor=' . $this->bgcolor
				]),
				'end' => ($this->contentCss ?: '')
			],
			'toolbar' => $this->toolbar,
			'weQuickbarSubcmds' => $this->quickbarSubcmds,
			//'weContextmenuCommands' => $this->getContextmenuCommands(),
			'weClassName' => $this->className,
			'language' => $this->editorLanguage,
			'element_format' => 'html',
			'popup_css_add' => implode(',', [
				we_html_element::getUnCache(WE_LIB_DIR . 'fontLiberation/stylesheet.css'),
				we_html_element::getUnCache(WE_LIB_DIR . 'fontawesome5/css/all.css'),
				we_html_element::getUnCache(WE_LIB_DIR . 'fontawesome5/css/v4-shims.css'),
				we_html_element::getUnCache(CSS_DIR . 'wysiwyg/tinymce/tinyDialogCss.css')
			]),
			'template_templates' => (in_array('template', $this->usedCommands, true) && $this->templates ? $this->getTemplates() : ''),
			'weSelectorClass' => $this->selectorClass,
			'table_toolbar' => implode(' ', $this->tableToolbar),
			'quickbars_selection_toolbar' => $this->quickbar,
			'weSelects' => $this->selects,
			'weBlockImportCss' => $this->blockImportCss,
			'pluginsInt' => implode(',', $this->usedPlugins['internal']),
			'pluginsExt' => $this->usedPlugins['external'],
		];
	}

	private static function getFrontendHeaderConsts(): array{
		return [
			'g_l' => ['tinyMceTranslationObject' => self::getTranslationObject()],
			'tables' => ['FILE_TABLE' => FILE_TABLE, 'OBJECT_FILES_TABLE' => OBJECT_FILES_TABLE],
			'dirs' => ['WE_JS_TINYMCE_DIR' => WE_JS_TINYMCE_DIR],
			'linkPrefix' => ['TYPE_INT_PREFIX' => we_base_link::TYPE_INT_PREFIX,
				'TYPE_OBJ_PREFIX' => we_base_link::TYPE_OBJ_PREFIX]
		];
	}

	public static function getTranslationObject(): array{
		return [
			'group_link' => g_l('wysiwyg', '[links]'),
			'group_copypaste' => g_l('wysiwyg', '[import_text]'),
			'group_advanced' => g_l('wysiwyg', '[advanced]'),
			'group_insert' => g_l('wysiwyg', '[insert]'),
			'group_indent' => g_l('wysiwyg', '[indent]'),
			'group_view' => g_l('wysiwyg', '[view]'),
			'group_table' => g_l('wysiwyg', '[table]'),
			'group_edit' => g_l('wysiwyg', '[edit]'),
			'group_xhtml' => g_l('wysiwyg', '[xhtml_extras]'),
			'cm_inserttable' => g_l('wysiwyg', '[insert_table]'),
			'cm_table_props' => g_l('wysiwyg', '[edit_table]'),
			'ed_removedInlinePictures' => g_l('wysiwyg', '[removedInlinePictures]'),
			'tt_webreak' => g_l('wysiwyg', '[insert_br]'),
			'tt_welink' => g_l('wysiwyg', '[hyperlink]'),
			'tt_weimage' => g_l('wysiwyg', '[insert_edit_image]'),
			'tt_welang' => g_l('wysiwyg', '[language]'),
			'tt_wevisualaid' => g_l('wysiwyg', '[visualaid]'),
			'tt_wegallery' => g_l('wysiwyg', '[addGallery]'),
			'tt_weabbr' => g_l('wysiwyg', '[abbr]'),
			'tt_weacronym' => g_l('wysiwyg', '[acronym]'),
			'tt_cite' => g_l('wysiwyg', '[cite]'),
			'tt_del' => g_l('wysiwyg', '[del]'),
			'tt_ins' => g_l('wysiwyg', '[ins]'),
			'tt_weunlink' => g_l('wysiwyg', '[unlink]'),
			'search_all' => g_l('wysiwyg', '[selectAll]'),
			'search_selection' => g_l('wysiwyg', '[selection]'),
			'search_wholeWords' => g_l('wysiwyg', '[wholeWordsOnly]'),
			'selectsFormatsTitle' => g_l('wysiwyg', '[formatSelectTitel]'),
			'selectsStylesTitle' => g_l('wysiwyg', '[styleSelectTitel]'),
		];
	}

	function getHTML(): string{
		switch($this->editorType){
			case self::TYPE_EDITBUTTON:
				self::$dataConfigurations[] = $this->getPropertiesEditor();
				self::$countInstances++;
				return $this->getHTMLEditButton();
			case self::TYPE_INLINE_TRUE:
				$configuration = $this->getPropertiesEditor();
				self::$dataConfigurations[] = $configuration;
				self::$countInstances++;
			/* fall through */
			case self::TYPE_INLINE_FALSE:
				return $this->getHTMLEditor();
			default:
				return '';
		}
	}

	public static function getHTMLConfigurationsTag(): string{
		self::$countInstances = 0;

		return we_html_baseElement::getHtmlCode(new we_html_baseElement('we-dataTiny', false, ['id' => 'loadVar_tinyConfigs',
					'data-dialogProperties' => setDynamicVar(self::$dataDialogProperties),
					'data-configurations' => setDynamicVar(self::$dataConfigurations),
		]));
	}

	public static function isWysiwygInstances(): bool{
		return self::$countInstances !== 0;
	}

	public static function getWeicons(): array{
		$icons = [];
		foreach(array_diff(scandir(WEBEDITION_PATH . 'we/images/icons/wysiwyg') ?: [], ['.', '..']) as $file){
			$icon = substr($file, 0, -4);
			$icons[$icon] = '<svg class="we_svg_size"><use xlink:href="' . WEBEDITION_DIR . 'we/images/icons/wysiwyg/' . $icon . '.svg#' . str_replace('-', '', $icon) . '" '
				. 'href="' . WEBEDITION_DIR . 'we/images/icons/wysiwyg/' . $icon . '.svg#' . str_replace('-', '', $icon) . '"/></svg>';
		}

		return $icons;
	}

	private function getHTMLEditButton(): string{
		switch($this->size){
			case 'small':
				$width = '400';
				$height = '300';
				break;
			case 'big':
				$width = '800';
				$height = '600';
				break;
			case 'fullscreen':
				$width = '10000';
				$height = '10000';
				break;
			case 'medium':
				$width = '600';
				$height = '400';
				break;
			default:
				$width = $this->width ? intval($this->width) : 600;
				$height = $this->height ? intval($this->height) : 400;
				break;
		}

		$dialogProperties = urlencode((string) json_encode($this->getPropertiesDialog(self::TYPE_INLINE_FALSE)) ?: '');

		return we_html_button::create_button(we_html_button::EDIT, "javascript:WE().layout.we_tinyMCE.functions.open_dialog_wysiwyg(window, " . ($this->isFrontendEdit ? 'true' : 'false') . ", '" . $dialogProperties . "', " . $width . ", " . $height . ");");
	}

	private function getHTMLEditor(): string{
		return /* $this->getHTMLSupportText() . */ we_weTag_tag::htmlTag('textarea', [
				'wrap' => 'off',
				'style' => 'visibility:hidden;color:#eeeeee; background-color:transparent; border: none;' . ($this->width ? ' width:' . $this->width . ';' : '') . ($this->height ? ' height:' . $this->height . ';' : ''),
				'name' => $this->name,
				'id' => ($this->isFrontendEdit ? $this->name : ''), // FIXME: do we need this condition?
				'class' => 'wetextarea' . ($this->className ? ' ' . $this->className : '') . ' ' . $this->selectorClass
				], strtr(self::parseInternalImageSrc($this->value), ['\n' => '', '&' => '&amp;']), true);
	}

}
