#!/usr/bin/php
<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: b2b4ec24b156 $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2020-07-08 00:19 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
/**
 * The script exports webEdition backup file to the given file
 * webEdition must be installed
 */
define('WE_DOCUMENT_ROOT', (string) realpath(__DIR__ . '/../../../'));
require_once(WE_DOCUMENT_ROOT . '/webEdition/custom/conf/cliConfig.php'); // @phan-suppress-current-line PhanMissingRequireFile
/* print("This script is currently unsupported due to changes on main core.");
  exit(1);
 */

// CONFIGURATION BEGINS ---------------------------------------------------------

/**
 * Path to backup file
 */
$_backup_filename = WE_DOCUMENT_ROOT . '/weBackup_daily';

$_REQUEST = [
	'compress' => true,
// export details

	/**
	 * export webEdition Core Data (Documents, Templates and Navigation)
	 */
	'handle_core' => true,
	/**
	 * export binary data
	 */
	'handle_binary' => true,
	/**
	 * export version data
	 */
	'handle_versions' => true,
	/**
	 * export version binary files
	 */
	'handle_versions_binarys' => true,
	/**
	 * export user data
	 */
	'handle_user' => true,
	/**
	 * export customer data
	 */
	'handle_customer' => true,
	/**
	 * export shop data
	 */
	'handle_shop' => true,
	/**
	 * export workflow data
	 */
	'handle_workflow' => true,
	/**
	 * export user data
	 */
	'handle_todo' => true,
	/**
	 * export newsletter data
	 */
	'handle_newsletter' => true,
	/**
	 * export temporary data
	 */
	'handle_temporary' => true,
	/**
	  /**
	 * export objects and classes
	 */
	'handle_object' => true,
	/**
	 * export scheduler data
	 */
	'handle_schedule' => true,
	/**
	 * export settings and preferences
	 */
	'handle_settings' => true,
	/**
	 * export configuration data
	 */
	'handle_configuration' => true,
	/**
	 * export webEdition export data
	 */
	'handle_export' => true,
	/**
	 * export voting data
	 */
	'handle_voting' => true,
	/**
	 * export extern files
	 */
	'handle_extern' => false,
// be user friendly :-)
	'verbose' => true,
];

// CONFIGURATION ENDS ---------------------------------------------------------
// we want to see errors
ini_set('display_errors', '1');
error_reporting(E_ALL);
// Define exit codes for errors
define('NO_ARGS', 10);
define('INVALID_OPTION', 11);


if(!isset($_SERVER['SERVER_NAME'])){
	$_SERVER['SERVER_NAME'] = $SERVER_NAME;
}

//use we-error handler; ignore if logging is disabled!
require_once(WE_DOCUMENT_ROOT . '/webEdition/we/include/we.inc.php');
if(!defined('WE_ERROR_SHOW')){
	define('WE_ERROR_SHOW', 1);
}
if(!defined('WE_ERROR_LOG')){
	define('WE_ERROR_LOG', 1);
}

we_base_errorHandler::start(false);

abstract class makeBackup{
	const HELP = 'Usage: makeBackup.php [options] [file]
Options:
  -z, --compress             gzip compression for output file

  -v, --verbose              Verbosely list files processed

  --help                     Prints out this help

 Options what to backup:
  --all                      Backups everything except external files
  --core                     Documents and templates
  --versions                 Versions
  --versions_binarys         Versions binarys
  --binary                   Binary data
  --user                     User data
  --customer                 Customer data
  --shop                     Shop data
  --workflow                 Workflow data
  --newsletter               Newsletter data
  --temporary                Temporary data (unpublished data)
  --object                   Objects and Classes
  --schedule                 Scheduler data
  --settings                 Settings
  --configuration            Configuration Data
  --export                   webEdition-Export Data
  --voting                   Voting data
  --extern                   Extern files (only use this if you have
                             enough memory and cpu power)
';

	public static function _checkAll(bool $flag = true): void{
		$_REQUEST['handle_core'] = $flag;
		$_REQUEST['handle_binary'] = $flag;
		$_REQUEST['handle_versions'] = $flag;
		$_REQUEST['handle_versions_binarys'] = $flag;
		$_REQUEST['handle_user'] = $flag;
		$_REQUEST['handle_customer'] = $flag;
		$_REQUEST['handle_shop'] = $flag;
		$_REQUEST['handle_workflow'] = $flag;
		$_REQUEST['handle_todo'] = $flag;
		$_REQUEST['handle_newsletter'] = $flag;
		$_REQUEST['handle_temporary'] = $flag;
		$_REQUEST['handle_object'] = $flag;
		$_REQUEST['handle_schedule'] = $flag;
		$_REQUEST['handle_settings'] = $flag;
		$_REQUEST['handle_configuration'] = $flag;
		$_REQUEST['handle_export'] = $flag;
		$_REQUEST['handle_voting'] = $flag;
	}

	public static function getOptions($args){

// Short options
		$short_opts = 'zv';

// Long options
		$long_opts = [
			'all',
			'core',
			'versions',
			'versions_binarys',
			'binary',
			'user',
			'customer',
			'shop',
			'workflow',
			'todo',
			'newsletter',
			'temporary',
			'object',
			'schedule',
			'settings',
			'configuration',
			'export',
			'voting',
			'extern',
			'verbose',
			'help',
			'compress'
		];

// Convert the arguments to options - check for the first argument
		return ($_SERVER['argv'] && realpath($_SERVER['argv'][0]) == __FILE__ ?
			Console_Getopt::getOpt($args, $short_opts, $long_opts) :
			Console_Getopt::getOpt2($args, $short_opts, $long_opts));
	}

	public static function doBackup(): void{
// knock out identifiation and permissions
		$_SESSION['user']['isWeSession'] = true;
		$_SESSION['perms'] = ['ADMINISTRATOR' => true];
		$_SESSION['user']['Username'] = 1;


// Reading the incoming arguments - same as $argv
		$args = Console_Getopt::readPHPArgv();




// Make sure we got them (for non CLI binaries)
		if(PEAR::isError($args)){
			fwrite(STDERR, $args->getMessage() . "\n");
			exit(NO_ARGS);
		}
		$options = makeBackup::getOptions($args);
// Check the options are valid
		if(PEAR::isError($options)){
			fwrite(STDERR, $options->getMessage() . "\n");
			fwrite(STDERR, makeBackup::HELP . "\n");
			exit(INVALID_OPTION);
		}
		if($args){
			$_REQUEST['compress'] = false;
			$_REQUEST['verbose'] = false;
			$_REQUEST['handle_extern'] = false;
			makeBackup::_checkAll(false);
		}

		foreach($options[0] as $opt){
			switch($opt[0]){
				case '--all':
					makeBackup::_checkAll(true);
					break;

				case '--compress':
				case 'z':
					$_REQUEST['compress'] = true;
					break;

				case 'v':
				case '--verbose':
					$_REQUEST['verbose'] = true;
					break;

				case '--help':
					echo makeBackup::HELP;
					exit(0);

				default:
					$_REQUEST['handle_' . preg_replace('/^--/', '', $opt[0])] = true;
			}
		}


// check if no option is checked

		$optionsSelected = false;

		foreach($_REQUEST as $_k => $_v){
			if(substr($_k, 0, 7) == "handle_"){
				if($_v){
					$optionsSelected = true;
					break;
				}
			}
		}

// if no option is checked, then check the dafaults
		if(!$optionsSelected){
			$_REQUEST['handle_core'] = true;
		}


		if(isset($options[1][0])){
			$GLOBALS['_backup_filename'] = $options[1][0];
		}


		$_REQUEST['filename'] = basename($GLOBALS['_backup_filename']);

		try{
			we_backup_settings::i();
		} catch (Exception $e){
			we_backup_settings::clearSessionData();
			if(!we_backup_preparer::prepareExport()){
				return;
			}
			$settings = we_backup_settings::i();
			'@phan-var we_backup_settings $settings';
			if($_REQUEST['verbose']){
				print "\nExporting to " . $GLOBALS['_backup_filename'] . "\n";
			}

			if($settings->extern_files){
				if($_REQUEST['verbose']){
					echo "Exporting extern files...\n";
				}

				we_backup_util::exportFiles($settings->extern_files);
			}

			if($_REQUEST['verbose']){
				echo "\nExporting repository\n";
			}

			while(($settings->row_counter < $settings->row_count) || we_backup_util::hasNextTable()){
				if($_REQUEST['verbose']){
					print '-';
				}
				if(!we_backup_export::export()){
					// force end
					$settings->row_counter = $settings->row_count;
				}
			}

			if($settings->isHandled('settings')){
				we_backup_util::exportFiles([
					WE_CUSTOM_DIR . 'conf/cliConfig.php',
					WE_CUSTOM_DIR . 'conf/we_conf_global.inc.php',
					WE_CUSTOM_DIR . 'conf/we_active_integrated_modules.inc.php',
					WE_CUSTOM_DIR . 'conf/we_conf_language.inc.php'
				]);
			}
			we_base_file::save($settings->backup_file, we_backup_util::weXmlExImFooter, 'ab');

			//copy the file to right location
			rename($settings->backup_file, $GLOBALS['_backup_filename']);
		}

		if($_REQUEST['verbose']){
			print "\nDone\n";
		}

		we_backup_settings::clearSessionData();
	}

}

makeBackup::doBackup();
