/* global WE, top */

/**
 * webEdition SDK
 *
 * webEdition CMS
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 *
 * This source is part of the webEdition SDK. The webEdition SDK is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU Lesser General Public License can be found at
 * http://www.gnu.org/licenses/lgpl-3.0.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionSDK/License.txt
 *
 *
 * @license    http://www.gnu.org/licenses/lgpl-3.0.html  LGPL
 */
'use strict';

var loaded;

var workflow = WE().util.getDynamicVar(document, 'loadVarWorkflowProp', 'data-workflow');

function doUnload() {
	WE().util.jsWindow.prototype.closeAll(window);
}

function getNumOfDocs() {
	return workflow.noDocs;
}

function we_cmd() {
	/*jshint validthis:true */
	var caller = (this && this.window === this ? this : window),
		args = WE().util.getWe_cmdArgsArray(Array.prototype.slice.call(arguments)),
		url = WE().util.getWe_cmdArgsUrl(args);

	switch (args[0]) {
		case "we_users_selector":
			(new (WE().util.jsOverlay)(caller, 'top',url, "browse_users", WE().consts.size.dialog.small, WE().consts.size.dialog.smaller)).setResizeable().open();
			break;
		case "we_selector_directory":
			(new (WE().util.jsOverlay)(caller, 'top',url, "we_fileselector", WE().consts.size.dialog.big, WE().consts.size.dialog.small)).setResizeable().open();
			break;
		case "we_selector_document":
			(new (WE().util.jsOverlay)(caller, 'top',url, "we_fileselector", WE().consts.size.dialog.big, WE().consts.size.dialog.small)).setResizeable().open();
			break;
		case "openObjselector":
			url = WE().consts.dirs.WEBEDITION_DIR + "we_cmd.php?we_cmd[0]=we_selector_document&we_cmd[table]=" + WE().consts.tables.OBJECT_TABLE + "&we_cmd[jsCmd]=" + args[5] + "&we_cmd[filter]=object&we_cmd[open_doc]=1";
			(new (WE().util.jsOverlay)(caller, 'top',url, "we_objectselector", WE().consts.size.dialog.big, WE().consts.size.dialog.medium)).setResizeable().open();
			break;
		case "add_cat":
		case "add_objcat":
			document.we_form.wcmd.value = args[0];
			document.we_form.wcat.value = args[1].allIDs.join(",");
			submitForm();
			break;
		case "del_cat":
		case "del_all_cats":
			document.we_form.wcmd.value = args[0];
			document.we_form.wcat.value = args[1];
			submitForm();
			break;
		case "del_objcat":
		case "del_all_objcats":
			document.we_form.wcmd.value = args[0];
			document.we_form.wocat.value = args[1];
			submitForm();
			break;
		case "add_folder":
			document.we_form.wcmd.value = args[0];
			document.we_form.wfolder.value = args[1].allIDs.join(',');
			submitForm();
			break;
		case "add_object_file_folder":
			document.we_form.wcmd.value = args[0];
			document.we_form.woffolder.value = args[1].allIDs.join(',');
			submitForm();
			break;
		case "del_folder":
		case "del_all_folders":
			document.we_form.wcmd.value = args[0];
			document.we_form.wfolder.value = args[1];
			submitForm();
			break;
		case "del_object_file_folder":
		case "del_all_object_file_folders":
			document.we_form.wcmd.value = args[0];
			document.we_form.woffolder.value = args[1];
			submitForm();
			break;
		case "add_object":
			document.we_form.wcmd.value = args[0];
			document.we_form.wobject.value = args[1].allIDs.join(',');
			submitForm();
			break;
		case "del_object":
		case "del_all_objects":
			document.we_form.wcmd.value = args[0];
			document.we_form.wobject.value = args[1];
			submitForm();
			break;
		case "switchPage":
			document.we_form.wcmd.value = args[0];
			document.we_form.page.value = args[1];
			submitForm();
			break;
		default:
			top.content.we_cmd.apply(caller, Array.prototype.slice.call(arguments));
	}
}

function submitForm() {
	var f = window.document.we_form;
	f.action = WE().consts.dirs.WEBEDITION_DIR + "we_cmd.php?we_cmd[0]=we_showMod&mod=workflow";
	f.target = "edbody";
	f.method = "post";
	f.submit();
}

function setStatus(val) {
	document.we_form[workflow.uid + "_Status"].value = val;

}

function getStatusContol() {
	return document.we_form[workflow.uid + "_Status"].value;
}

function clickCheck(a) {
	a.value = (a.checked ? 1 : 0);
}

function addStep() {
	document.we_form.wsteps.value++;
	document.we_form.wcmd.value = "reload_table";
	submitForm();
}

function addTask() {
	document.we_form.wtasks.value++;
	document.we_form.wcmd.value = "reload_table";
	submitForm();
}


function delStep() {
	if (document.we_form.wsteps.value > 1) {
		document.we_form.wsteps.value--;
		document.we_form.wcmd.value = "reload_table";
		submitForm();
	} else {
		WE().util.showMessage(WE().consts.g_l.workflow.prop.del_last_step, WE().consts.message.WE_MESSAGE_ERROR, window);
	}
}

function delTask() {
	if (document.we_form.wtasks.value > 1) {
		document.we_form.wtasks.value--;
		document.we_form.wcmd.value = "reload_table";
		submitForm();
	} else {
		WE().util.showMessage(WE().consts.g_l.workflow.prop.del_last_task, WE().consts.message.WE_MESSAGE_ERROR, window);
	}
}


function checkData() {
	var txt, j, userempty;
	var nsteps = document.we_form.wsteps;
	var ntasks = document.we_form.wtasks;
	if (document.we_form[workflow.uid + "_Text"].value === "") {
		WE().util.showMessage(WE().consts.g_l.workflow.prop.name_empty, WE().consts.message.WE_MESSAGE_ERROR, window);
		return false;
	}

	if (document.we_form[workflow.uid + "_Folders"].value === "" && document.we_form[workflow.uid + "_Type"].value === "1") {
		WE().util.showMessage(WE().consts.g_l.workflow.prop.folders_empty, WE().consts.message.WE_MESSAGE_ERROR, window);
		return false;
	}

	if (document.we_form[workflow.uid + "_ObjectFileFolders"].value === "" && document.we_form[workflow.uid + "_Type"].value === "2") {
		WE().util.showMessage(WE().consts.g_l.workflow.prop.folders_empty, WE().consts.message.WE_MESSAGE_ERROR, window);
		return false;
	}

	if ((document.we_form[workflow.uid + "_DocType[]"].value === "" && document.we_form[workflow.uid + "_Categories"].value === "") && document.we_form[workflow.uid + "_Type"].value === "0") {
		//note we only get the first selected value, but we only want to know if sth. is selected!
		WE().util.showMessage(WE().consts.g_l.workflow.prop.doctype_empty, WE().consts.message.WE_MESSAGE_ERROR, window);
		return false;
	}

	if (document.we_form[workflow.uid + "_Objects"].value === "" && document.we_form[workflow.uid + "_Type"].value == 2) {
		WE().util.showMessage(WE().consts.g_l.workflow.prop.objects_empty, WE().consts.message.WE_MESSAGE_ERROR, window);
		return false;
	}

	for (var i = 0; i < nsteps.value; i++) {
		if (document.we_form[workflow.uid + '_step' + i + '_Worktime'].value === "") {
			txt = WE().consts.g_l.workflow.prop.worktime_empty;
			WE().util.showMessage(txt.replace(/%s/, i + 1), WE().consts.message.WE_MESSAGE_ERROR, window);
			return false;
		}
		userempty = true;
		for (j = 0; j < ntasks.value; j++) {
			if (document.we_form[workflow.uid + '_task_' + i + '_' + j + '_userid'].value !== 0) {
				userempty = false;
			}
		}
		if (userempty) {
			txt = WE().consts.g_l.workflow.prop.user_empty;
			WE().util.showMessage(txt.replace(/%s/, i + 1), WE().consts.message.WE_MESSAGE_ERROR, window);
			return false;
		}

	}
	return true;
}